/****************************************************************************
** Meta object code from reading C++ file 'qsetmaster.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qsetmaster.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsetmaster.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qsetmaster_t {
    QByteArrayData data[14];
    char stringdata0[192];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qsetmaster_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qsetmaster_t qt_meta_stringdata_seq66__qsetmaster = {
    {
QT_MOC_LITERAL(0, 0, 17), // "seq66::qsetmaster"
QT_MOC_LITERAL(1, 18, 18), // "conditional_update"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 13), // "slot_set_name"
QT_MOC_LITERAL(4, 52, 14), // "slot_show_sets"
QT_MOC_LITERAL(5, 67, 14), // "slot_move_down"
QT_MOC_LITERAL(6, 82, 12), // "slot_move_up"
QT_MOC_LITERAL(7, 95, 11), // "slot_delete"
QT_MOC_LITERAL(8, 107, 19), // "slot_table_click_ex"
QT_MOC_LITERAL(9, 127, 3), // "row"
QT_MOC_LITERAL(10, 131, 17), // "slot_cell_changed"
QT_MOC_LITERAL(11, 149, 6), // "column"
QT_MOC_LITERAL(12, 156, 24), // "slot_toggle_trigger_mode"
QT_MOC_LITERAL(13, 181, 10) // "slot_set_0"

    },
    "seq66::qsetmaster\0conditional_update\0"
    "\0slot_set_name\0slot_show_sets\0"
    "slot_move_down\0slot_move_up\0slot_delete\0"
    "slot_table_click_ex\0row\0slot_cell_changed\0"
    "column\0slot_toggle_trigger_mode\0"
    "slot_set_0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qsetmaster[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x08 /* Private */,
       3,    0,   65,    2, 0x08 /* Private */,
       4,    0,   66,    2, 0x08 /* Private */,
       5,    0,   67,    2, 0x08 /* Private */,
       6,    0,   68,    2, 0x08 /* Private */,
       7,    0,   69,    2, 0x08 /* Private */,
       8,    4,   70,    2, 0x08 /* Private */,
      10,    2,   79,    2, 0x08 /* Private */,
      12,    0,   84,    2, 0x08 /* Private */,
      13,    0,   85,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    9,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    9,   11,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qsetmaster::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qsetmaster *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->conditional_update(); break;
        case 1: _t->slot_set_name(); break;
        case 2: _t->slot_show_sets(); break;
        case 3: _t->slot_move_down(); break;
        case 4: _t->slot_move_up(); break;
        case 5: _t->slot_delete(); break;
        case 6: _t->slot_table_click_ex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 7: _t->slot_cell_changed((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->slot_toggle_trigger_mode(); break;
        case 9: _t->slot_set_0(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qsetmaster::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qsetmaster.data,
    qt_meta_data_seq66__qsetmaster,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qsetmaster::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qsetmaster::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qsetmaster.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qsetmaster::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
