/********************************************************************************
** Form generated from reading UI file 'qsbuildinfo.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSBUILDINFO_H
#define QSBUILDINFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>

QT_BEGIN_NAMESPACE

class Ui_qsbuildinfo
{
public:
    QDialogButtonBox *buttonBox;
    QLabel *buildProgramLabel;
    QLabel *buildVersionLabel;
    QPlainTextEdit *buildInfoTextEdit;

    void setupUi(QDialog *qsbuildinfo)
    {
        if (qsbuildinfo->objectName().isEmpty())
            qsbuildinfo->setObjectName(QString::fromUtf8("qsbuildinfo"));
        qsbuildinfo->resize(503, 489);
        buttonBox = new QDialogButtonBox(qsbuildinfo);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setGeometry(QRect(204, 440, 91, 32));
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Ok);
        buildProgramLabel = new QLabel(qsbuildinfo);
        buildProgramLabel->setObjectName(QString::fromUtf8("buildProgramLabel"));
        buildProgramLabel->setGeometry(QRect(28, 12, 57, 16));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        buildProgramLabel->setFont(font);
        buildProgramLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        buildVersionLabel = new QLabel(qsbuildinfo);
        buildVersionLabel->setObjectName(QString::fromUtf8("buildVersionLabel"));
        buildVersionLabel->setGeometry(QRect(304, 8, 77, 20));
        buildVersionLabel->setFont(font);
        buildVersionLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        buildInfoTextEdit = new QPlainTextEdit(qsbuildinfo);
        buildInfoTextEdit->setObjectName(QString::fromUtf8("buildInfoTextEdit"));
        buildInfoTextEdit->setGeometry(QRect(30, 40, 441, 381));
        buildInfoTextEdit->setReadOnly(true);

        retranslateUi(qsbuildinfo);
        QObject::connect(buttonBox, SIGNAL(accepted()), qsbuildinfo, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), qsbuildinfo, SLOT(reject()));

        QMetaObject::connectSlotsByName(qsbuildinfo);
    } // setupUi

    void retranslateUi(QDialog *qsbuildinfo)
    {
        qsbuildinfo->setWindowTitle(QCoreApplication::translate("qsbuildinfo", "Build Information", nullptr));
        buildProgramLabel->setText(QCoreApplication::translate("qsbuildinfo", "Seq66", nullptr));
        buildVersionLabel->setText(QCoreApplication::translate("qsbuildinfo", "Version", nullptr));
        buildInfoTextEdit->setPlaceholderText(QCoreApplication::translate("qsbuildinfo", "Build Information", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsbuildinfo: public Ui_qsbuildinfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSBUILDINFO_H
