/****************************************************************************
** Meta object code from reading C++ file 'qsmainwnd.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qsmainwnd.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsmainwnd.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qsmainwnd_t {
    QByteArrayData data[99];
    char stringdata0[1403];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qsmainwnd_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qsmainwnd_t qt_meta_stringdata_seq66__qsmainwnd = {
    {
QT_MOC_LITERAL(0, 0, 16), // "seq66::qsmainwnd"
QT_MOC_LITERAL(1, 17, 17), // "signal_set_change"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 5), // "setno"
QT_MOC_LITERAL(4, 42, 18), // "signal_song_action"
QT_MOC_LITERAL(5, 61, 6), // "action"
QT_MOC_LITERAL(6, 68, 20), // "slot_open_edit_prefs"
QT_MOC_LITERAL(7, 89, 17), // "slot_summary_save"
QT_MOC_LITERAL(8, 107, 13), // "slot_tutorial"
QT_MOC_LITERAL(9, 121, 16), // "slot_user_manual"
QT_MOC_LITERAL(10, 138, 13), // "slot_set_home"
QT_MOC_LITERAL(11, 152, 11), // "update_bank"
QT_MOC_LITERAL(12, 164, 7), // "newBank"
QT_MOC_LITERAL(13, 172, 16), // "update_bank_text"
QT_MOC_LITERAL(14, 189, 13), // "start_playing"
QT_MOC_LITERAL(15, 203, 8), // "set_loop"
QT_MOC_LITERAL(16, 212, 4), // "loop"
QT_MOC_LITERAL(17, 217, 13), // "pause_playing"
QT_MOC_LITERAL(18, 231, 12), // "stop_playing"
QT_MOC_LITERAL(19, 244, 13), // "set_song_mode"
QT_MOC_LITERAL(20, 258, 9), // "song_mode"
QT_MOC_LITERAL(21, 268, 14), // "song_recording"
QT_MOC_LITERAL(22, 283, 6), // "record"
QT_MOC_LITERAL(23, 290, 12), // "recording_ex"
QT_MOC_LITERAL(24, 303, 5), // "panic"
QT_MOC_LITERAL(25, 309, 10), // "update_bpm"
QT_MOC_LITERAL(26, 320, 3), // "bpm"
QT_MOC_LITERAL(27, 324, 8), // "edit_bpm"
QT_MOC_LITERAL(28, 333, 17), // "update_set_change"
QT_MOC_LITERAL(29, 351, 18), // "update_song_action"
QT_MOC_LITERAL(30, 370, 10), // "playaction"
QT_MOC_LITERAL(31, 381, 19), // "update_ppqn_by_text"
QT_MOC_LITERAL(32, 401, 4), // "text"
QT_MOC_LITERAL(33, 406, 15), // "update_midi_bus"
QT_MOC_LITERAL(34, 422, 6), // "bindex"
QT_MOC_LITERAL(35, 429, 24), // "update_beats_per_measure"
QT_MOC_LITERAL(36, 454, 7), // "bmindex"
QT_MOC_LITERAL(37, 462, 18), // "update_beat_length"
QT_MOC_LITERAL(38, 481, 7), // "blindex"
QT_MOC_LITERAL(39, 489, 16), // "open_recent_file"
QT_MOC_LITERAL(40, 506, 8), // "new_file"
QT_MOC_LITERAL(41, 515, 11), // "new_session"
QT_MOC_LITERAL(42, 527, 9), // "save_file"
QT_MOC_LITERAL(43, 537, 11), // "std::string"
QT_MOC_LITERAL(44, 549, 5), // "fname"
QT_MOC_LITERAL(45, 555, 10), // "updatemenu"
QT_MOC_LITERAL(46, 566, 12), // "save_session"
QT_MOC_LITERAL(47, 579, 12), // "save_file_as"
QT_MOC_LITERAL(48, 592, 14), // "export_project"
QT_MOC_LITERAL(49, 607, 19), // "export_file_as_midi"
QT_MOC_LITERAL(50, 627, 20), // "export_file_as_smf_0"
QT_MOC_LITERAL(51, 648, 11), // "export_song"
QT_MOC_LITERAL(52, 660, 4), // "quit"
QT_MOC_LITERAL(53, 665, 20), // "import_midi_into_set"
QT_MOC_LITERAL(54, 686, 24), // "import_midi_into_session"
QT_MOC_LITERAL(55, 711, 14), // "import_project"
QT_MOC_LITERAL(56, 726, 15), // "import_playlist"
QT_MOC_LITERAL(57, 742, 20), // "select_and_load_file"
QT_MOC_LITERAL(58, 763, 21), // "show_open_list_dialog"
QT_MOC_LITERAL(59, 785, 21), // "show_save_list_dialog"
QT_MOC_LITERAL(60, 807, 22), // "show_open_mutes_dialog"
QT_MOC_LITERAL(61, 830, 22), // "show_save_mutes_dialog"
QT_MOC_LITERAL(62, 853, 12), // "show_qsabout"
QT_MOC_LITERAL(63, 866, 16), // "show_qsbuildinfo"
QT_MOC_LITERAL(64, 883, 14), // "show_qsappinfo"
QT_MOC_LITERAL(65, 898, 14), // "show_qslogview"
QT_MOC_LITERAL(66, 913, 16), // "tabWidgetClicked"
QT_MOC_LITERAL(67, 930, 8), // "newindex"
QT_MOC_LITERAL(68, 939, 18), // "conditional_update"
QT_MOC_LITERAL(69, 958, 11), // "load_editor"
QT_MOC_LITERAL(70, 970, 5), // "seqid"
QT_MOC_LITERAL(71, 976, 17), // "load_event_editor"
QT_MOC_LITERAL(72, 994, 13), // "load_qseqedit"
QT_MOC_LITERAL(73, 1008, 16), // "load_qseqedit_ex"
QT_MOC_LITERAL(74, 1025, 6), // "active"
QT_MOC_LITERAL(75, 1032, 14), // "load_qperfedit"
QT_MOC_LITERAL(76, 1047, 2), // "on"
QT_MOC_LITERAL(77, 1050, 15), // "load_live_frame"
QT_MOC_LITERAL(78, 1066, 5), // "ssnum"
QT_MOC_LITERAL(79, 1072, 18), // "load_session_frame"
QT_MOC_LITERAL(80, 1091, 15), // "load_set_master"
QT_MOC_LITERAL(81, 1107, 16), // "load_mute_master"
QT_MOC_LITERAL(82, 1124, 18), // "toggle_time_format"
QT_MOC_LITERAL(83, 1143, 10), // "reset_sets"
QT_MOC_LITERAL(84, 1154, 21), // "open_performance_edit"
QT_MOC_LITERAL(85, 1176, 20), // "apply_song_transpose"
QT_MOC_LITERAL(86, 1197, 18), // "reload_mute_groups"
QT_MOC_LITERAL(87, 1216, 17), // "clear_mute_groups"
QT_MOC_LITERAL(88, 1234, 16), // "set_song_mute_on"
QT_MOC_LITERAL(89, 1251, 17), // "set_song_mute_off"
QT_MOC_LITERAL(90, 1269, 20), // "set_song_mute_toggle"
QT_MOC_LITERAL(91, 1290, 19), // "set_playscreen_copy"
QT_MOC_LITERAL(92, 1310, 20), // "set_playscreen_paste"
QT_MOC_LITERAL(93, 1331, 12), // "learn_toggle"
QT_MOC_LITERAL(94, 1344, 3), // "tap"
QT_MOC_LITERAL(95, 1348, 8), // "queue_it"
QT_MOC_LITERAL(96, 1357, 9), // "slot_test"
QT_MOC_LITERAL(97, 1367, 20), // "slot_close_externals"
QT_MOC_LITERAL(98, 1388, 14) // "slot_show_hide"

    },
    "seq66::qsmainwnd\0signal_set_change\0\0"
    "setno\0signal_song_action\0action\0"
    "slot_open_edit_prefs\0slot_summary_save\0"
    "slot_tutorial\0slot_user_manual\0"
    "slot_set_home\0update_bank\0newBank\0"
    "update_bank_text\0start_playing\0set_loop\0"
    "loop\0pause_playing\0stop_playing\0"
    "set_song_mode\0song_mode\0song_recording\0"
    "record\0recording_ex\0panic\0update_bpm\0"
    "bpm\0edit_bpm\0update_set_change\0"
    "update_song_action\0playaction\0"
    "update_ppqn_by_text\0text\0update_midi_bus\0"
    "bindex\0update_beats_per_measure\0bmindex\0"
    "update_beat_length\0blindex\0open_recent_file\0"
    "new_file\0new_session\0save_file\0"
    "std::string\0fname\0updatemenu\0save_session\0"
    "save_file_as\0export_project\0"
    "export_file_as_midi\0export_file_as_smf_0\0"
    "export_song\0quit\0import_midi_into_set\0"
    "import_midi_into_session\0import_project\0"
    "import_playlist\0select_and_load_file\0"
    "show_open_list_dialog\0show_save_list_dialog\0"
    "show_open_mutes_dialog\0show_save_mutes_dialog\0"
    "show_qsabout\0show_qsbuildinfo\0"
    "show_qsappinfo\0show_qslogview\0"
    "tabWidgetClicked\0newindex\0conditional_update\0"
    "load_editor\0seqid\0load_event_editor\0"
    "load_qseqedit\0load_qseqedit_ex\0active\0"
    "load_qperfedit\0on\0load_live_frame\0"
    "ssnum\0load_session_frame\0load_set_master\0"
    "load_mute_master\0toggle_time_format\0"
    "reset_sets\0open_performance_edit\0"
    "apply_song_transpose\0reload_mute_groups\0"
    "clear_mute_groups\0set_song_mute_on\0"
    "set_song_mute_off\0set_song_mute_toggle\0"
    "set_playscreen_copy\0set_playscreen_paste\0"
    "learn_toggle\0tap\0queue_it\0slot_test\0"
    "slot_close_externals\0slot_show_hide"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qsmainwnd[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      83,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  429,    2, 0x06 /* Public */,
       4,    1,  432,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,  435,    2, 0x08 /* Private */,
       7,    0,  436,    2, 0x08 /* Private */,
       8,    0,  437,    2, 0x08 /* Private */,
       9,    0,  438,    2, 0x08 /* Private */,
      10,    0,  439,    2, 0x08 /* Private */,
      11,    1,  440,    2, 0x08 /* Private */,
      13,    0,  443,    2, 0x08 /* Private */,
      14,    0,  444,    2, 0x08 /* Private */,
      15,    1,  445,    2, 0x08 /* Private */,
      17,    0,  448,    2, 0x08 /* Private */,
      18,    0,  449,    2, 0x08 /* Private */,
      19,    1,  450,    2, 0x08 /* Private */,
      21,    1,  453,    2, 0x08 /* Private */,
      23,    1,  456,    2, 0x08 /* Private */,
      24,    0,  459,    2, 0x08 /* Private */,
      25,    1,  460,    2, 0x08 /* Private */,
      27,    0,  463,    2, 0x08 /* Private */,
      28,    1,  464,    2, 0x08 /* Private */,
      29,    1,  467,    2, 0x08 /* Private */,
      31,    1,  470,    2, 0x08 /* Private */,
      33,    1,  473,    2, 0x08 /* Private */,
      35,    1,  476,    2, 0x08 /* Private */,
      37,    1,  479,    2, 0x08 /* Private */,
      39,    0,  482,    2, 0x08 /* Private */,
      40,    0,  483,    2, 0x08 /* Private */,
      41,    0,  484,    2, 0x08 /* Private */,
      42,    2,  485,    2, 0x08 /* Private */,
      42,    1,  490,    2, 0x28 /* Private | MethodCloned */,
      42,    0,  493,    2, 0x28 /* Private | MethodCloned */,
      46,    0,  494,    2, 0x08 /* Private */,
      47,    0,  495,    2, 0x08 /* Private */,
      48,    1,  496,    2, 0x08 /* Private */,
      48,    0,  499,    2, 0x28 /* Private | MethodCloned */,
      49,    1,  500,    2, 0x08 /* Private */,
      49,    0,  503,    2, 0x28 /* Private | MethodCloned */,
      50,    1,  504,    2, 0x08 /* Private */,
      50,    0,  507,    2, 0x28 /* Private | MethodCloned */,
      51,    1,  508,    2, 0x08 /* Private */,
      51,    0,  511,    2, 0x28 /* Private | MethodCloned */,
      52,    0,  512,    2, 0x08 /* Private */,
      53,    0,  513,    2, 0x08 /* Private */,
      54,    0,  514,    2, 0x08 /* Private */,
      55,    0,  515,    2, 0x08 /* Private */,
      56,    0,  516,    2, 0x08 /* Private */,
      57,    0,  517,    2, 0x08 /* Private */,
      58,    0,  518,    2, 0x08 /* Private */,
      59,    0,  519,    2, 0x08 /* Private */,
      60,    0,  520,    2, 0x08 /* Private */,
      61,    0,  521,    2, 0x08 /* Private */,
      62,    0,  522,    2, 0x08 /* Private */,
      63,    0,  523,    2, 0x08 /* Private */,
      64,    0,  524,    2, 0x08 /* Private */,
      65,    0,  525,    2, 0x08 /* Private */,
      66,    1,  526,    2, 0x08 /* Private */,
      68,    0,  529,    2, 0x08 /* Private */,
      69,    1,  530,    2, 0x08 /* Private */,
      71,    1,  533,    2, 0x08 /* Private */,
      72,    1,  536,    2, 0x08 /* Private */,
      73,    2,  539,    2, 0x08 /* Private */,
      75,    1,  544,    2, 0x08 /* Private */,
      77,    1,  547,    2, 0x08 /* Private */,
      79,    0,  550,    2, 0x08 /* Private */,
      80,    0,  551,    2, 0x08 /* Private */,
      81,    0,  552,    2, 0x08 /* Private */,
      82,    1,  553,    2, 0x08 /* Private */,
      83,    0,  556,    2, 0x08 /* Private */,
      84,    0,  557,    2, 0x08 /* Private */,
      85,    0,  558,    2, 0x08 /* Private */,
      86,    0,  559,    2, 0x08 /* Private */,
      87,    0,  560,    2, 0x08 /* Private */,
      88,    0,  561,    2, 0x08 /* Private */,
      89,    0,  562,    2, 0x08 /* Private */,
      90,    0,  563,    2, 0x08 /* Private */,
      91,    0,  564,    2, 0x08 /* Private */,
      92,    0,  565,    2, 0x08 /* Private */,
      93,    0,  566,    2, 0x08 /* Private */,
      94,    0,  567,    2, 0x08 /* Private */,
      95,    0,  568,    2, 0x08 /* Private */,
      96,    0,  569,    2, 0x08 /* Private */,
      97,    0,  570,    2, 0x08 /* Private */,
      98,    0,  571,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   20,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void, QMetaType::Bool,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   26,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,   30,
    QMetaType::Void, QMetaType::QString,   32,
    QMetaType::Void, QMetaType::Int,   34,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void, QMetaType::Int,   38,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 43, QMetaType::Bool,   44,   45,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool,
    QMetaType::Bool, 0x80000000 | 43,   44,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   67,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   70,
    QMetaType::Void, QMetaType::Int,   70,
    QMetaType::Void, QMetaType::Int,   70,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   70,   74,
    QMetaType::Void, QMetaType::Bool,   76,
    QMetaType::Void, QMetaType::Int,   78,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   76,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qsmainwnd::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qsmainwnd *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signal_set_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signal_song_action((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slot_open_edit_prefs(); break;
        case 3: _t->slot_summary_save(); break;
        case 4: _t->slot_tutorial(); break;
        case 5: _t->slot_user_manual(); break;
        case 6: _t->slot_set_home(); break;
        case 7: _t->update_bank((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->update_bank_text(); break;
        case 9: _t->start_playing(); break;
        case 10: _t->set_loop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->pause_playing(); break;
        case 12: _t->stop_playing(); break;
        case 13: _t->set_song_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->song_recording((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->recording_ex((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->panic(); break;
        case 17: _t->update_bpm((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 18: _t->edit_bpm(); break;
        case 19: _t->update_set_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->update_song_action((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->update_ppqn_by_text((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->update_midi_bus((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->update_beats_per_measure((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->update_beat_length((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->open_recent_file(); break;
        case 26: _t->new_file(); break;
        case 27: _t->new_session(); break;
        case 28: { bool _r = _t->save_file((*reinterpret_cast< const std::string(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->save_file((*reinterpret_cast< const std::string(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->save_file();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->save_session();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->save_file_as();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->export_project((*reinterpret_cast< const std::string(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->export_project();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->export_file_as_midi((*reinterpret_cast< const std::string(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->export_file_as_midi();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 37: { bool _r = _t->export_file_as_smf_0((*reinterpret_cast< const std::string(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 38: { bool _r = _t->export_file_as_smf_0();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 39: { bool _r = _t->export_song((*reinterpret_cast< const std::string(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 40: { bool _r = _t->export_song();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 41: _t->quit(); break;
        case 42: _t->import_midi_into_set(); break;
        case 43: _t->import_midi_into_session(); break;
        case 44: _t->import_project(); break;
        case 45: _t->import_playlist(); break;
        case 46: _t->select_and_load_file(); break;
        case 47: _t->show_open_list_dialog(); break;
        case 48: _t->show_save_list_dialog(); break;
        case 49: _t->show_open_mutes_dialog(); break;
        case 50: _t->show_save_mutes_dialog(); break;
        case 51: _t->show_qsabout(); break;
        case 52: _t->show_qsbuildinfo(); break;
        case 53: _t->show_qsappinfo(); break;
        case 54: _t->show_qslogview(); break;
        case 55: _t->tabWidgetClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 56: _t->conditional_update(); break;
        case 57: _t->load_editor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 58: _t->load_event_editor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 59: _t->load_qseqedit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 60: _t->load_qseqedit_ex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 61: _t->load_qperfedit((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->load_live_frame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 63: _t->load_session_frame(); break;
        case 64: _t->load_set_master(); break;
        case 65: _t->load_mute_master(); break;
        case 66: _t->toggle_time_format((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 67: _t->reset_sets(); break;
        case 68: _t->open_performance_edit(); break;
        case 69: _t->apply_song_transpose(); break;
        case 70: _t->reload_mute_groups(); break;
        case 71: _t->clear_mute_groups(); break;
        case 72: _t->set_song_mute_on(); break;
        case 73: _t->set_song_mute_off(); break;
        case 74: _t->set_song_mute_toggle(); break;
        case 75: _t->set_playscreen_copy(); break;
        case 76: _t->set_playscreen_paste(); break;
        case 77: _t->learn_toggle(); break;
        case 78: _t->tap(); break;
        case 79: _t->queue_it(); break;
        case 80: _t->slot_test(); break;
        case 81: _t->slot_close_externals(); break;
        case 82: _t->slot_show_hide(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (qsmainwnd::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qsmainwnd::signal_set_change)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (qsmainwnd::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qsmainwnd::signal_song_action)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qsmainwnd::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_seq66__qsmainwnd.data,
    qt_meta_data_seq66__qsmainwnd,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qsmainwnd::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qsmainwnd::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qsmainwnd.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int seq66::qsmainwnd::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 83)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 83;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 83)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 83;
    }
    return _id;
}

// SIGNAL 0
void seq66::qsmainwnd::signal_set_change(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void seq66::qsmainwnd::signal_song_action(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
