/********************************************************************************
** Form generated from reading UI file 'qsabout.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSABOUT_H
#define QSABOUT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_qsabout
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *topLabel;
    QFrame *separatorTop;
    QLabel *versionLabel;
    QLabel *descriptionLabel;
    QLabel *label_github_seq66;
    QLabel *label_gmail_ahlstromcj;
    QPushButton *extraButton;
    QLabel *originsLabel;
    QLabel *label_filter24_seq24;
    QLabel *label_github_seq32;
    QLabel *label_github_kepler34;

    void setupUi(QDialog *qsabout)
    {
        if (qsabout->objectName().isEmpty())
            qsabout->setObjectName(QString::fromUtf8("qsabout"));
        qsabout->resize(475, 362);
        verticalLayout = new QVBoxLayout(qsabout);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        topLabel = new QLabel(qsabout);
        topLabel->setObjectName(QString::fromUtf8("topLabel"));
        QFont font;
        font.setPointSize(18);
        font.setBold(true);
        topLabel->setFont(font);
        topLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(topLabel);

        separatorTop = new QFrame(qsabout);
        separatorTop->setObjectName(QString::fromUtf8("separatorTop"));
        separatorTop->setFrameShape(QFrame::HLine);
        separatorTop->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(separatorTop);

        versionLabel = new QLabel(qsabout);
        versionLabel->setObjectName(QString::fromUtf8("versionLabel"));
        versionLabel->setMinimumSize(QSize(0, 32));
        versionLabel->setMaximumSize(QSize(16777215, 42));
        QFont font1;
        font1.setPointSize(10);
        font1.setBold(true);
        versionLabel->setFont(font1);
        versionLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(versionLabel);

        descriptionLabel = new QLabel(qsabout);
        descriptionLabel->setObjectName(QString::fromUtf8("descriptionLabel"));
        descriptionLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        descriptionLabel->setWordWrap(true);

        verticalLayout->addWidget(descriptionLabel);

        label_github_seq66 = new QLabel(qsabout);
        label_github_seq66->setObjectName(QString::fromUtf8("label_github_seq66"));
        label_github_seq66->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_github_seq66);

        label_gmail_ahlstromcj = new QLabel(qsabout);
        label_gmail_ahlstromcj->setObjectName(QString::fromUtf8("label_gmail_ahlstromcj"));
        label_gmail_ahlstromcj->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_gmail_ahlstromcj);

        extraButton = new QPushButton(qsabout);
        extraButton->setObjectName(QString::fromUtf8("extraButton"));
        extraButton->setEnabled(false);
        extraButton->setCheckable(false);
        extraButton->setChecked(false);
        extraButton->setAutoDefault(false);
        extraButton->setFlat(false);

        verticalLayout->addWidget(extraButton);

        originsLabel = new QLabel(qsabout);
        originsLabel->setObjectName(QString::fromUtf8("originsLabel"));
        originsLabel->setMaximumSize(QSize(16777215, 16777215));
        originsLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        originsLabel->setWordWrap(true);

        verticalLayout->addWidget(originsLabel);

        label_filter24_seq24 = new QLabel(qsabout);
        label_filter24_seq24->setObjectName(QString::fromUtf8("label_filter24_seq24"));
        label_filter24_seq24->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_filter24_seq24);

        label_github_seq32 = new QLabel(qsabout);
        label_github_seq32->setObjectName(QString::fromUtf8("label_github_seq32"));
        label_github_seq32->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_github_seq32);

        label_github_kepler34 = new QLabel(qsabout);
        label_github_kepler34->setObjectName(QString::fromUtf8("label_github_kepler34"));
        label_github_kepler34->setAlignment(Qt::AlignmentFlag::AlignCenter);

        verticalLayout->addWidget(label_github_kepler34);


        retranslateUi(qsabout);

        QMetaObject::connectSlotsByName(qsabout);
    } // setupUi

    void retranslateUi(QDialog *qsabout)
    {
        qsabout->setWindowTitle(QCoreApplication::translate("qsabout", "About Seq66", nullptr));
        topLabel->setText(QCoreApplication::translate("qsabout", "Seq66 [qseq66]", nullptr));
        versionLabel->setText(QCoreApplication::translate("qsabout", "(replaced at run-time)", nullptr));
        descriptionLabel->setText(QCoreApplication::translate("qsabout", "<html><head/><body><p align=\"center\">Loop-based MIDI sequencer<br>Licensed under the GNU General Public License v2/v3</p></body></html>", nullptr));
        label_github_seq66->setText(QCoreApplication::translate("qsabout", "<a href=\"https://github.com/ahlstromcj/seq66\"><span style=\" text-decoration: underline; color:#0000ff;\">GitHub/ahlstromcj/seq66</span></a>", nullptr));
        label_gmail_ahlstromcj->setText(QCoreApplication::translate("qsabout", "<a href=\"mailto:ahlstromcj@gmail.com\"><span style=\" text-decoration: underline; color:#0000ff;\">ahlstromcj@gmail.com</span></a>", nullptr));
        extraButton->setText(QString());
        originsLabel->setText(QCoreApplication::translate("qsabout", "<html><head/><body><p align=\"center\">Based on Seq24, Seq32, Sequencer64, and Kepler34 </p></body></html>", nullptr));
        label_filter24_seq24->setText(QCoreApplication::translate("qsabout", "<a href=\"http://filter24.org/seq24\"><span style=\" text-decoration: underline; color:#0000ff;\">Filter24.org/seq24</span></a>", nullptr));
        label_github_seq32->setText(QCoreApplication::translate("qsabout", "<a href=\"https://github.com/Stazed/seq32\"><span style=\" text-decoration: underline; color:#0000ff;\">GitHub/Stazed/Seq32</span></a>", nullptr));
        label_github_kepler34->setText(QCoreApplication::translate("qsabout", "<a href=\"https://github.com/oli-kester/kepler34\"><span style=\" text-decoration: underline; color:#0000ff;\">GitHub/oli-kester/kepler34</span></a>", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsabout: public Ui_qsabout {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSABOUT_H
