/********************************************************************************
** Form generated from reading UI file 'qseqeventframe.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSEQEVENTFRAME_H
#define QSEQEVENTFRAME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qseqeventframe
{
public:
    QTableWidget *eventTableWidget;
    QPlainTextEdit *data_text_edit;
    QWidget *gridLayoutWidget;
    QGridLayout *gridLayout_2;
    QPushButton *button_modify;
    QPushButton *button_del;
    QPushButton *button_dump;
    QPushButton *button_save;
    QPushButton *button_ins;
    QPushButton *button_clear;
    QWidget *horizLayoutWidget;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_seq_number;
    QLineEdit *m_entry_name;
    QWidget *dataLayoutWidget;
    QGridLayout *gridLayout_4;
    QPushButton *button_link;
    QLabel *label_time_sig;
    QPushButton *button_blank;
    QLabel *label_measures_ev_count;
    QWidget *layoutWidget;
    QGridLayout *gridLayout_3;
    QLineEdit *entry_ev_data_1;
    QLabel *d1_label;
    QCheckBox *pulse_time_check_box;
    QCheckBox *hex_data_check_box;
    QLabel *d0_label;
    QLineEdit *entry_ev_data_0;
    QWidget *layoutWidget1;
    QGridLayout *gridLayout;
    QLabel *label_category;
    QComboBox *combo_ev_category;
    QLabel *time_label;
    QLineEdit *entry_ev_timestamp;
    QLabel *event_label;
    QComboBox *combo_ev_name;
    QPushButton *select_button;
    QLabel *channel_label;
    QComboBox *channel_combo_box;

    void setupUi(QFrame *qseqeventframe)
    {
        if (qseqeventframe->objectName().isEmpty())
            qseqeventframe->setObjectName(QString::fromUtf8("qseqeventframe"));
        qseqeventframe->resize(900, 416);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qseqeventframe->sizePolicy().hasHeightForWidth());
        qseqeventframe->setSizePolicy(sizePolicy);
        qseqeventframe->setMaximumSize(QSize(900, 500));
        qseqeventframe->setFrameShape(QFrame::Shape::NoFrame);
        qseqeventframe->setFrameShadow(QFrame::Shadow::Plain);
        qseqeventframe->setLineWidth(3);
        qseqeventframe->setMidLineWidth(2);
        eventTableWidget = new QTableWidget(qseqeventframe);
        if (eventTableWidget->columnCount() < 7)
            eventTableWidget->setColumnCount(7);
        if (eventTableWidget->rowCount() < 48)
            eventTableWidget->setRowCount(48);
        eventTableWidget->setObjectName(QString::fromUtf8("eventTableWidget"));
        eventTableWidget->setGeometry(QRect(9, 17, 491, 380));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(eventTableWidget->sizePolicy().hasHeightForWidth());
        eventTableWidget->setSizePolicy(sizePolicy1);
        eventTableWidget->setMinimumSize(QSize(440, 380));
        eventTableWidget->setMaximumSize(QSize(492, 999));
        eventTableWidget->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContents);
        eventTableWidget->setAlternatingRowColors(false);
        eventTableWidget->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
        eventTableWidget->setGridStyle(Qt::PenStyle::DashLine);
        eventTableWidget->setWordWrap(false);
        eventTableWidget->setRowCount(48);
        eventTableWidget->setColumnCount(7);
        eventTableWidget->verticalHeader()->setMinimumSectionSize(16);
        eventTableWidget->verticalHeader()->setDefaultSectionSize(18);
        data_text_edit = new QPlainTextEdit(qseqeventframe);
        data_text_edit->setObjectName(QString::fromUtf8("data_text_edit"));
        data_text_edit->setGeometry(QRect(510, 270, 311, 69));
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(data_text_edit->sizePolicy().hasHeightForWidth());
        data_text_edit->setSizePolicy(sizePolicy2);
        data_text_edit->setLineWidth(2);
        data_text_edit->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        gridLayoutWidget = new QWidget(qseqeventframe);
        gridLayoutWidget->setObjectName(QString::fromUtf8("gridLayoutWidget"));
        gridLayoutWidget->setGeometry(QRect(520, 348, 261, 52));
        gridLayout_2 = new QGridLayout(gridLayoutWidget);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        gridLayout_2->setHorizontalSpacing(20);
        gridLayout_2->setVerticalSpacing(2);
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        button_modify = new QPushButton(gridLayoutWidget);
        button_modify->setObjectName(QString::fromUtf8("button_modify"));
        button_modify->setEnabled(false);
        button_modify->setMinimumSize(QSize(64, 24));
        button_modify->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_modify, 0, 2, 1, 1);

        button_del = new QPushButton(gridLayoutWidget);
        button_del->setObjectName(QString::fromUtf8("button_del"));
        button_del->setEnabled(false);
        button_del->setMinimumSize(QSize(64, 24));
        button_del->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_del, 0, 0, 1, 1);

        button_dump = new QPushButton(gridLayoutWidget);
        button_dump->setObjectName(QString::fromUtf8("button_dump"));
        button_dump->setEnabled(false);
        button_dump->setMinimumSize(QSize(64, 24));
        button_dump->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_dump, 1, 2, 1, 1);

        button_save = new QPushButton(gridLayoutWidget);
        button_save->setObjectName(QString::fromUtf8("button_save"));
        button_save->setEnabled(false);
        button_save->setMinimumSize(QSize(64, 24));
        button_save->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_save, 1, 1, 1, 1);

        button_ins = new QPushButton(gridLayoutWidget);
        button_ins->setObjectName(QString::fromUtf8("button_ins"));
        button_ins->setEnabled(false);
        button_ins->setMinimumSize(QSize(64, 24));
        button_ins->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_ins, 0, 1, 1, 1);

        button_clear = new QPushButton(gridLayoutWidget);
        button_clear->setObjectName(QString::fromUtf8("button_clear"));
        button_clear->setEnabled(false);
        button_clear->setMinimumSize(QSize(64, 24));
        button_clear->setMaximumSize(QSize(90, 24));

        gridLayout_2->addWidget(button_clear, 1, 0, 1, 1);

        horizLayoutWidget = new QWidget(qseqeventframe);
        horizLayoutWidget->setObjectName(QString::fromUtf8("horizLayoutWidget"));
        horizLayoutWidget->setGeometry(QRect(500, 12, 331, 32));
        horizontalLayout_2 = new QHBoxLayout(horizLayoutWidget);
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setSizeConstraint(QLayout::SizeConstraint::SetMinimumSize);
        horizontalLayout_2->setContentsMargins(0, 0, 10, 0);
        label_seq_number = new QLabel(horizLayoutWidget);
        label_seq_number->setObjectName(QString::fromUtf8("label_seq_number"));
        label_seq_number->setMinimumSize(QSize(36, 0));
        label_seq_number->setMaximumSize(QSize(48, 16777215));
        QFont font;
        font.setBold(true);
        label_seq_number->setFont(font);
        label_seq_number->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout_2->addWidget(label_seq_number);

        m_entry_name = new QLineEdit(horizLayoutWidget);
        m_entry_name->setObjectName(QString::fromUtf8("m_entry_name"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(m_entry_name->sizePolicy().hasHeightForWidth());
        m_entry_name->setSizePolicy(sizePolicy3);
        m_entry_name->setMinimumSize(QSize(248, 24));
        m_entry_name->setMaximumSize(QSize(256, 24));
        QFont font1;
        font1.setFamily(QString::fromUtf8("Anonymous Pro"));
        font1.setPointSize(12);
        m_entry_name->setFont(font1);

        horizontalLayout_2->addWidget(m_entry_name);

        dataLayoutWidget = new QWidget(qseqeventframe);
        dataLayoutWidget->setObjectName(QString::fromUtf8("dataLayoutWidget"));
        dataLayoutWidget->setGeometry(QRect(499, 48, 331, 58));
        gridLayout_4 = new QGridLayout(dataLayoutWidget);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        gridLayout_4->setHorizontalSpacing(10);
        gridLayout_4->setVerticalSpacing(2);
        gridLayout_4->setContentsMargins(0, 0, 0, 0);
        button_link = new QPushButton(dataLayoutWidget);
        button_link->setObjectName(QString::fromUtf8("button_link"));
        button_link->setMinimumSize(QSize(64, 24));
        button_link->setMaximumSize(QSize(64, 24));
        QFont font2;
        font2.setPointSize(8);
        font2.setItalic(true);
        button_link->setFont(font2);
        button_link->setCheckable(true);
        button_link->setFlat(false);

        gridLayout_4->addWidget(button_link, 0, 0, 1, 1);

        label_time_sig = new QLabel(dataLayoutWidget);
        label_time_sig->setObjectName(QString::fromUtf8("label_time_sig"));
        QSizePolicy sizePolicy4(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(label_time_sig->sizePolicy().hasHeightForWidth());
        label_time_sig->setSizePolicy(sizePolicy4);
        label_time_sig->setMinimumSize(QSize(248, 24));
        label_time_sig->setMaximumSize(QSize(248, 24));
        label_time_sig->setFont(font);
        label_time_sig->setFrameShape(QFrame::Shape::NoFrame);
        label_time_sig->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);

        gridLayout_4->addWidget(label_time_sig, 0, 1, 1, 1);

        button_blank = new QPushButton(dataLayoutWidget);
        button_blank->setObjectName(QString::fromUtf8("button_blank"));
        button_blank->setMinimumSize(QSize(64, 24));
        button_blank->setMaximumSize(QSize(64, 16777215));
        button_blank->setFlat(true);

        gridLayout_4->addWidget(button_blank, 1, 0, 1, 1);

        label_measures_ev_count = new QLabel(dataLayoutWidget);
        label_measures_ev_count->setObjectName(QString::fromUtf8("label_measures_ev_count"));
        sizePolicy4.setHeightForWidth(label_measures_ev_count->sizePolicy().hasHeightForWidth());
        label_measures_ev_count->setSizePolicy(sizePolicy4);
        label_measures_ev_count->setMinimumSize(QSize(248, 24));
        label_measures_ev_count->setMaximumSize(QSize(248, 24));
        label_measures_ev_count->setFont(font);
        label_measures_ev_count->setFrameShape(QFrame::Shape::NoFrame);
        label_measures_ev_count->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);

        gridLayout_4->addWidget(label_measures_ev_count, 1, 1, 1, 1);

        gridLayout_4->setColumnStretch(0, 1);
        gridLayout_4->setColumnStretch(1, 6);
        layoutWidget = new QWidget(qseqeventframe);
        layoutWidget->setObjectName(QString::fromUtf8("layoutWidget"));
        layoutWidget->setGeometry(QRect(511, 240, 288, 28));
        gridLayout_3 = new QGridLayout(layoutWidget);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        gridLayout_3->setHorizontalSpacing(8);
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        entry_ev_data_1 = new QLineEdit(layoutWidget);
        entry_ev_data_1->setObjectName(QString::fromUtf8("entry_ev_data_1"));
        entry_ev_data_1->setMinimumSize(QSize(40, 24));
        entry_ev_data_1->setMaximumSize(QSize(40, 24));

        gridLayout_3->addWidget(entry_ev_data_1, 0, 3, 1, 1);

        d1_label = new QLabel(layoutWidget);
        d1_label->setObjectName(QString::fromUtf8("d1_label"));
        d1_label->setMinimumSize(QSize(24, 24));
        d1_label->setMaximumSize(QSize(24, 24));
        d1_label->setFont(font);
        d1_label->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout_3->addWidget(d1_label, 0, 2, 1, 1);

        pulse_time_check_box = new QCheckBox(layoutWidget);
        pulse_time_check_box->setObjectName(QString::fromUtf8("pulse_time_check_box"));
        pulse_time_check_box->setMinimumSize(QSize(64, 24));
        pulse_time_check_box->setMaximumSize(QSize(64, 24));

        gridLayout_3->addWidget(pulse_time_check_box, 0, 4, 1, 1);

        hex_data_check_box = new QCheckBox(layoutWidget);
        hex_data_check_box->setObjectName(QString::fromUtf8("hex_data_check_box"));
        hex_data_check_box->setMaximumSize(QSize(64, 16777215));

        gridLayout_3->addWidget(hex_data_check_box, 0, 5, 1, 1);

        d0_label = new QLabel(layoutWidget);
        d0_label->setObjectName(QString::fromUtf8("d0_label"));
        d0_label->setMinimumSize(QSize(24, 24));
        d0_label->setMaximumSize(QSize(24, 24));
        d0_label->setFont(font);
        d0_label->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout_3->addWidget(d0_label, 0, 0, 1, 1);

        entry_ev_data_0 = new QLineEdit(layoutWidget);
        entry_ev_data_0->setObjectName(QString::fromUtf8("entry_ev_data_0"));
        entry_ev_data_0->setMinimumSize(QSize(40, 24));
        entry_ev_data_0->setMaximumSize(QSize(40, 24));

        gridLayout_3->addWidget(entry_ev_data_0, 0, 1, 1, 1);

        layoutWidget1 = new QWidget(qseqeventframe);
        layoutWidget1->setObjectName(QString::fromUtf8("layoutWidget1"));
        layoutWidget1->setGeometry(QRect(502, 109, 321, 131));
        gridLayout = new QGridLayout(layoutWidget1);
        gridLayout->setSpacing(4);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        label_category = new QLabel(layoutWidget1);
        label_category->setObjectName(QString::fromUtf8("label_category"));
        label_category->setMinimumSize(QSize(72, 24));
        label_category->setMaximumSize(QSize(72, 24));
        label_category->setFont(font);
        label_category->setAlignment(Qt::AlignmentFlag::AlignLeading|Qt::AlignmentFlag::AlignLeft|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(label_category, 0, 0, 1, 1);

        combo_ev_category = new QComboBox(layoutWidget1);
        combo_ev_category->setObjectName(QString::fromUtf8("combo_ev_category"));
        combo_ev_category->setMinimumSize(QSize(160, 24));
        combo_ev_category->setMaximumSize(QSize(180, 24));
        combo_ev_category->setMaxVisibleItems(8);
        combo_ev_category->setMaxCount(8);
        combo_ev_category->setInsertPolicy(QComboBox::InsertPolicy::NoInsert);

        gridLayout->addWidget(combo_ev_category, 0, 1, 1, 1);

        time_label = new QLabel(layoutWidget1);
        time_label->setObjectName(QString::fromUtf8("time_label"));
        time_label->setMinimumSize(QSize(72, 24));
        time_label->setMaximumSize(QSize(72, 24));
        time_label->setFont(font);

        gridLayout->addWidget(time_label, 1, 0, 1, 1);

        entry_ev_timestamp = new QLineEdit(layoutWidget1);
        entry_ev_timestamp->setObjectName(QString::fromUtf8("entry_ev_timestamp"));
        QSizePolicy sizePolicy5(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(entry_ev_timestamp->sizePolicy().hasHeightForWidth());
        entry_ev_timestamp->setSizePolicy(sizePolicy5);
        entry_ev_timestamp->setMinimumSize(QSize(160, 28));
        entry_ev_timestamp->setMaximumSize(QSize(180, 28));

        gridLayout->addWidget(entry_ev_timestamp, 1, 1, 1, 1);

        event_label = new QLabel(layoutWidget1);
        event_label->setObjectName(QString::fromUtf8("event_label"));
        event_label->setMinimumSize(QSize(72, 24));
        event_label->setMaximumSize(QSize(72, 24));
        event_label->setFont(font);

        gridLayout->addWidget(event_label, 2, 0, 1, 1);

        combo_ev_name = new QComboBox(layoutWidget1);
        combo_ev_name->setObjectName(QString::fromUtf8("combo_ev_name"));
        combo_ev_name->setMinimumSize(QSize(160, 28));
        combo_ev_name->setMaximumSize(QSize(180, 28));
        combo_ev_name->setEditable(true);
        combo_ev_name->setMaxVisibleItems(12);
        combo_ev_name->setMaxCount(32);
        combo_ev_name->setInsertPolicy(QComboBox::InsertPolicy::NoInsert);
        combo_ev_name->setFrame(true);

        gridLayout->addWidget(combo_ev_name, 2, 1, 1, 1);

        select_button = new QPushButton(layoutWidget1);
        select_button->setObjectName(QString::fromUtf8("select_button"));
        select_button->setMinimumSize(QSize(48, 0));
        select_button->setMaximumSize(QSize(48, 16777215));

        gridLayout->addWidget(select_button, 2, 2, 1, 1);

        channel_label = new QLabel(layoutWidget1);
        channel_label->setObjectName(QString::fromUtf8("channel_label"));
        channel_label->setMinimumSize(QSize(72, 24));
        channel_label->setMaximumSize(QSize(72, 24));
        channel_label->setFont(font);

        gridLayout->addWidget(channel_label, 3, 0, 1, 1);

        channel_combo_box = new QComboBox(layoutWidget1);
        channel_combo_box->setObjectName(QString::fromUtf8("channel_combo_box"));
        channel_combo_box->setMinimumSize(QSize(160, 28));
        channel_combo_box->setMaximumSize(QSize(180, 28));
        channel_combo_box->setEditable(true);
        channel_combo_box->setMaxVisibleItems(12);
        channel_combo_box->setMaxCount(17);
        channel_combo_box->setInsertPolicy(QComboBox::InsertPolicy::NoInsert);
        channel_combo_box->setMinimumContentsLength(1);
        channel_combo_box->setFrame(true);

        gridLayout->addWidget(channel_combo_box, 3, 1, 1, 1);

        QWidget::setTabOrder(eventTableWidget, entry_ev_timestamp);
        QWidget::setTabOrder(entry_ev_timestamp, combo_ev_name);
        QWidget::setTabOrder(combo_ev_name, channel_combo_box);
        QWidget::setTabOrder(channel_combo_box, entry_ev_data_0);
        QWidget::setTabOrder(entry_ev_data_0, entry_ev_data_1);
        QWidget::setTabOrder(entry_ev_data_1, button_del);
        QWidget::setTabOrder(button_del, button_clear);
        QWidget::setTabOrder(button_clear, button_ins);
        QWidget::setTabOrder(button_ins, button_save);
        QWidget::setTabOrder(button_save, button_modify);
        QWidget::setTabOrder(button_modify, button_dump);

        retranslateUi(qseqeventframe);

        channel_combo_box->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(qseqeventframe);
    } // setupUi

    void retranslateUi(QFrame *qseqeventframe)
    {
        qseqeventframe->setWindowTitle(QCoreApplication::translate("qseqeventframe", "Events", nullptr));
#if QT_CONFIG(tooltip)
        eventTableWidget->setToolTip(QCoreApplication::translate("qseqeventframe", "Events in the pattern/sequence. Events can be edited only with the controls at the right.", nullptr));
#endif // QT_CONFIG(tooltip)
        data_text_edit->setPlaceholderText(QCoreApplication::translate("qseqeventframe", "(Enter Meta/SysEx event text here)", nullptr));
#if QT_CONFIG(tooltip)
        button_modify->setToolTip(QCoreApplication::translate("qseqeventframe", "Deletes the current event, then inserts a new event at the time shown above, with the data items shown above.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_modify->setText(QCoreApplication::translate("qseqeventframe", "Modif&y", nullptr));
#if QT_CONFIG(tooltip)
        button_del->setToolTip(QCoreApplication::translate("qseqeventframe", "Deletes the currently-selected MIDI event.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_del->setText(QCoreApplication::translate("qseqeventframe", "&Delete", nullptr));
#if QT_CONFIG(tooltip)
        button_dump->setToolTip(QCoreApplication::translate("qseqeventframe", "Appends the text of events to a file for trouble-shooting.\n"
"It is in the same directory as the MIDI file, and has a name\n"
"of the form \"filename-pattern-#.text\", where '#' is the number of\n"
"the pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_dump->setText(QCoreApplication::translate("qseqeventframe", "D&ump", nullptr));
#if QT_CONFIG(tooltip)
        button_save->setToolTip(QCoreApplication::translate("qseqeventframe", "Saves current changes to the pattern.  This step\n"
"must be done to preserve the changes.  Then the file must\n"
"be saved to make the changes permanent.  Tricky, but safer.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_save->setText(QCoreApplication::translate("qseqeventframe", "Store", nullptr));
#if QT_CONFIG(tooltip)
        button_ins->setToolTip(QCoreApplication::translate("qseqeventframe", "Inserts a new event at the time shown above, with the data items shown above.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_ins->setText(QCoreApplication::translate("qseqeventframe", "&Insert", nullptr));
#if QT_CONFIG(tooltip)
        button_clear->setToolTip(QCoreApplication::translate("qseqeventframe", "Deletes all of the events in this pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_clear->setText(QCoreApplication::translate("qseqeventframe", "Clear", nullptr));
        label_seq_number->setText(QCoreApplication::translate("qseqeventframe", "1024", nullptr));
        m_entry_name->setText(QCoreApplication::translate("qseqeventframe", "Loop Name", nullptr));
#if QT_CONFIG(tooltip)
        button_link->setToolTip(QCoreApplication::translate("qseqeventframe", "If enabled, this setting allows two linked\n"
"Note events to be selected at the\n"
"same time.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_link->setText(QCoreApplication::translate("qseqeventframe", "Sel Linked", nullptr));
        label_time_sig->setText(QCoreApplication::translate("qseqeventframe", "Time Signature and PPQN", nullptr));
        button_blank->setText(QString());
        label_measures_ev_count->setText(QCoreApplication::translate("qseqeventframe", "Measures and Events Count", nullptr));
#if QT_CONFIG(statustip)
        entry_ev_data_1->setStatusTip(QCoreApplication::translate("qseqeventframe", "The second data byte, if applicable.  Change using decimal or hex (0xnn) notation.", nullptr));
#endif // QT_CONFIG(statustip)
        entry_ev_data_1->setText(QCoreApplication::translate("qseqeventframe", "0x00", nullptr));
        d1_label->setText(QCoreApplication::translate("qseqeventframe", "D1", nullptr));
        pulse_time_check_box->setText(QCoreApplication::translate("qseqeventframe", "Ticks", nullptr));
        hex_data_check_box->setText(QCoreApplication::translate("qseqeventframe", "Hex", nullptr));
        d0_label->setText(QCoreApplication::translate("qseqeventframe", "D0", nullptr));
#if QT_CONFIG(statustip)
        entry_ev_data_0->setStatusTip(QCoreApplication::translate("qseqeventframe", "The first data byte, if applicable. Change using decimal or hex (0xnn) notation.", nullptr));
#endif // QT_CONFIG(statustip)
        entry_ev_data_0->setText(QCoreApplication::translate("qseqeventframe", "0x00", nullptr));
        label_category->setText(QCoreApplication::translate("qseqeventframe", "Category", nullptr));
        time_label->setText(QCoreApplication::translate("qseqeventframe", "Time  ", nullptr));
#if QT_CONFIG(tooltip)
        entry_ev_timestamp->setToolTip(QCoreApplication::translate("qseqeventframe", "Current time in bars:beats:pulses. Edit to insert or modify events.", nullptr));
#endif // QT_CONFIG(tooltip)
        entry_ev_timestamp->setText(QCoreApplication::translate("qseqeventframe", "001:1:000", nullptr));
        event_label->setText(QCoreApplication::translate("qseqeventframe", "Event", nullptr));
        select_button->setText(QString());
        channel_label->setText(QCoreApplication::translate("qseqeventframe", "Chan", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qseqeventframe: public Ui_qseqeventframe {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSEQEVENTFRAME_H
