/****************************************************************************
** Meta object code from reading C++ file 'qperfeditframe64.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qperfeditframe64.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qperfeditframe64.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qperfeditframe64_t {
    QByteArrayData data[25];
    char stringdata0[317];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qperfeditframe64_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qperfeditframe64_t qt_meta_stringdata_seq66__qperfeditframe64 = {
    {
QT_MOC_LITERAL(0, 0, 23), // "seq66::qperfeditframe64"
QT_MOC_LITERAL(1, 24, 16), // "update_grid_snap"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 9), // "snapindex"
QT_MOC_LITERAL(4, 52, 12), // "slot_zoom_in"
QT_MOC_LITERAL(5, 65, 13), // "slot_zoom_out"
QT_MOC_LITERAL(6, 79, 15), // "reset_transpose"
QT_MOC_LITERAL(7, 95, 16), // "update_transpose"
QT_MOC_LITERAL(8, 112, 5), // "index"
QT_MOC_LITERAL(9, 118, 15), // "marker_collapse"
QT_MOC_LITERAL(10, 134, 13), // "marker_expand"
QT_MOC_LITERAL(11, 148, 18), // "marker_expand_copy"
QT_MOC_LITERAL(12, 167, 11), // "marker_loop"
QT_MOC_LITERAL(13, 179, 4), // "loop"
QT_MOC_LITERAL(14, 184, 4), // "grow"
QT_MOC_LITERAL(15, 189, 6), // "follow"
QT_MOC_LITERAL(16, 196, 9), // "ischecked"
QT_MOC_LITERAL(17, 206, 10), // "entry_mode"
QT_MOC_LITERAL(18, 217, 13), // "slot_duration"
QT_MOC_LITERAL(19, 231, 23), // "reset_trigger_transpose"
QT_MOC_LITERAL(20, 255, 21), // "set_trigger_transpose"
QT_MOC_LITERAL(21, 277, 5), // "tpose"
QT_MOC_LITERAL(22, 283, 9), // "v_zoom_in"
QT_MOC_LITERAL(23, 293, 10), // "v_zoom_out"
QT_MOC_LITERAL(24, 304, 12) // "reset_v_zoom"

    },
    "seq66::qperfeditframe64\0update_grid_snap\0"
    "\0snapindex\0slot_zoom_in\0slot_zoom_out\0"
    "reset_transpose\0update_transpose\0index\0"
    "marker_collapse\0marker_expand\0"
    "marker_expand_copy\0marker_loop\0loop\0"
    "grow\0follow\0ischecked\0entry_mode\0"
    "slot_duration\0reset_trigger_transpose\0"
    "set_trigger_transpose\0tpose\0v_zoom_in\0"
    "v_zoom_out\0reset_v_zoom"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qperfeditframe64[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x08 /* Private */,
       4,    0,  107,    2, 0x08 /* Private */,
       5,    0,  108,    2, 0x08 /* Private */,
       6,    0,  109,    2, 0x08 /* Private */,
       7,    1,  110,    2, 0x08 /* Private */,
       9,    0,  113,    2, 0x08 /* Private */,
      10,    0,  114,    2, 0x08 /* Private */,
      11,    0,  115,    2, 0x08 /* Private */,
      12,    1,  116,    2, 0x08 /* Private */,
      14,    0,  119,    2, 0x08 /* Private */,
      15,    1,  120,    2, 0x08 /* Private */,
      17,    1,  123,    2, 0x08 /* Private */,
      18,    1,  126,    2, 0x08 /* Private */,
      19,    1,  129,    2, 0x08 /* Private */,
      20,    1,  132,    2, 0x08 /* Private */,
      22,    0,  135,    2, 0x08 /* Private */,
      23,    0,  136,    2, 0x08 /* Private */,
      24,    0,  137,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,

       0        // eod
};

void seq66::qperfeditframe64::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qperfeditframe64 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->update_grid_snap((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->slot_zoom_in(); break;
        case 2: _t->slot_zoom_out(); break;
        case 3: _t->reset_transpose(); break;
        case 4: _t->update_transpose((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->marker_collapse(); break;
        case 6: _t->marker_expand(); break;
        case 7: _t->marker_expand_copy(); break;
        case 8: _t->marker_loop((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->grow(); break;
        case 10: _t->follow((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->entry_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slot_duration((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->reset_trigger_transpose((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->set_trigger_transpose((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: { bool _r = _t->v_zoom_in();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->v_zoom_out();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->reset_v_zoom();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qperfeditframe64::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qperfeditframe64.data,
    qt_meta_data_seq66__qperfeditframe64,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qperfeditframe64::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qperfeditframe64::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qperfeditframe64.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qperfeditframe64::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
