/****************************************************************************
** Meta object code from reading C++ file 'qseqeventframe.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qseqeventframe.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qseqeventframe.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qseqeventframe_t {
    QByteArrayData data[27];
    char stringdata0[367];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qseqeventframe_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qseqeventframe_t qt_meta_stringdata_seq66__qseqeventframe = {
    {
QT_MOC_LITERAL(0, 0, 21), // "seq66::qseqeventframe"
QT_MOC_LITERAL(1, 22, 19), // "slot_table_click_ex"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 3), // "row"
QT_MOC_LITERAL(4, 47, 6), // "column"
QT_MOC_LITERAL(5, 54, 7), // "prevrow"
QT_MOC_LITERAL(6, 62, 7), // "prevcol"
QT_MOC_LITERAL(7, 70, 17), // "slot_row_selected"
QT_MOC_LITERAL(8, 88, 16), // "slot_link_status"
QT_MOC_LITERAL(9, 105, 11), // "slot_delete"
QT_MOC_LITERAL(10, 117, 11), // "slot_insert"
QT_MOC_LITERAL(11, 129, 11), // "slot_modify"
QT_MOC_LITERAL(12, 141, 9), // "slot_save"
QT_MOC_LITERAL(13, 151, 10), // "slot_clear"
QT_MOC_LITERAL(14, 162, 9), // "slot_dump"
QT_MOC_LITERAL(15, 172, 11), // "slot_cancel"
QT_MOC_LITERAL(16, 184, 15), // "update_seq_name"
QT_MOC_LITERAL(17, 200, 17), // "slot_midi_channel"
QT_MOC_LITERAL(18, 218, 5), // "index"
QT_MOC_LITERAL(19, 224, 15), // "slot_event_name"
QT_MOC_LITERAL(20, 240, 19), // "slot_event_category"
QT_MOC_LITERAL(21, 260, 19), // "slot_hex_data_state"
QT_MOC_LITERAL(22, 280, 5), // "state"
QT_MOC_LITERAL(23, 286, 21), // "slot_pulse_time_state"
QT_MOC_LITERAL(24, 308, 19), // "slot_ev_data_0_edit"
QT_MOC_LITERAL(25, 328, 21), // "slot_meta_text_change"
QT_MOC_LITERAL(26, 350, 16) // "slot_event_popup"

    },
    "seq66::qseqeventframe\0slot_table_click_ex\0"
    "\0row\0column\0prevrow\0prevcol\0"
    "slot_row_selected\0slot_link_status\0"
    "slot_delete\0slot_insert\0slot_modify\0"
    "slot_save\0slot_clear\0slot_dump\0"
    "slot_cancel\0update_seq_name\0"
    "slot_midi_channel\0index\0slot_event_name\0"
    "slot_event_category\0slot_hex_data_state\0"
    "state\0slot_pulse_time_state\0"
    "slot_ev_data_0_edit\0slot_meta_text_change\0"
    "slot_event_popup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qseqeventframe[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    4,  109,    2, 0x08 /* Private */,
       7,    0,  118,    2, 0x08 /* Private */,
       8,    0,  119,    2, 0x08 /* Private */,
       9,    0,  120,    2, 0x08 /* Private */,
      10,    0,  121,    2, 0x08 /* Private */,
      11,    0,  122,    2, 0x08 /* Private */,
      12,    0,  123,    2, 0x08 /* Private */,
      13,    0,  124,    2, 0x08 /* Private */,
      14,    0,  125,    2, 0x08 /* Private */,
      15,    0,  126,    2, 0x08 /* Private */,
      16,    0,  127,    2, 0x08 /* Private */,
      17,    1,  128,    2, 0x08 /* Private */,
      19,    1,  131,    2, 0x08 /* Private */,
      20,    1,  134,    2, 0x08 /* Private */,
      21,    1,  137,    2, 0x08 /* Private */,
      23,    1,  140,    2, 0x08 /* Private */,
      24,    1,  143,    2, 0x08 /* Private */,
      25,    0,  146,    2, 0x08 /* Private */,
      26,    0,  147,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    3,    4,    5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qseqeventframe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qseqeventframe *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slot_table_click_ex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->slot_row_selected(); break;
        case 2: _t->slot_link_status(); break;
        case 3: _t->slot_delete(); break;
        case 4: _t->slot_insert(); break;
        case 5: _t->slot_modify(); break;
        case 6: _t->slot_save(); break;
        case 7: _t->slot_clear(); break;
        case 8: _t->slot_dump(); break;
        case 9: _t->slot_cancel(); break;
        case 10: _t->update_seq_name(); break;
        case 11: _t->slot_midi_channel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slot_event_name((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slot_event_category((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->slot_hex_data_state((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->slot_pulse_time_state((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slot_ev_data_0_edit((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->slot_meta_text_change(); break;
        case 18: _t->slot_event_popup(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qseqeventframe::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qseqeventframe.data,
    qt_meta_data_seq66__qseqeventframe,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qseqeventframe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qseqeventframe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qseqeventframe.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qseqeventframe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
