/********************************************************************************
** Form generated from reading UI file 'qsetmaster.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSETMASTER_H
#define QSETMASTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsetmaster
{
public:
    QWidget *layoutWidgetSetMaster;
    QGridLayout *gridLayout_2;
    QGridLayout *setGridLayout;
    QTableWidget *m_set_table;
    QPlainTextEdit *m_set_contents_text;
    QGridLayout *gridLayout;
    QPushButton *m_button_show;
    QPushButton *m_button_toggle_trigger_mode;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *m_button_set_0;
    QLineEdit *m_set_number_text;
    QLineEdit *m_set_name_text;
    QPushButton *m_button_delete;
    QSpacerItem *horizontalSpacer;
    QPushButton *m_button_down;
    QPushButton *m_button_up;
    QLabel *label;
    QLabel *label_2;

    void setupUi(QFrame *qsetmaster)
    {
        if (qsetmaster->objectName().isEmpty())
            qsetmaster->setObjectName(QString::fromUtf8("qsetmaster"));
        qsetmaster->resize(841, 428);
        qsetmaster->setFrameShape(QFrame::Shape::NoFrame);
        qsetmaster->setFrameShadow(QFrame::Shadow::Plain);
        layoutWidgetSetMaster = new QWidget(qsetmaster);
        layoutWidgetSetMaster->setObjectName(QString::fromUtf8("layoutWidgetSetMaster"));
        layoutWidgetSetMaster->setGeometry(QRect(10, 10, 778, 412));
        gridLayout_2 = new QGridLayout(layoutWidgetSetMaster);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        setGridLayout = new QGridLayout();
        setGridLayout->setObjectName(QString::fromUtf8("setGridLayout"));

        gridLayout_2->addLayout(setGridLayout, 0, 0, 1, 1);

        m_set_table = new QTableWidget(layoutWidgetSetMaster);
        if (m_set_table->columnCount() < 3)
            m_set_table->setColumnCount(3);
        if (m_set_table->rowCount() < 32)
            m_set_table->setRowCount(32);
        m_set_table->setObjectName(QString::fromUtf8("m_set_table"));
        m_set_table->setEnabled(true);
        m_set_table->setMinimumSize(QSize(400, 248));
        m_set_table->setMaximumSize(QSize(448, 340));
        m_set_table->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        m_set_table->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContentsOnFirstShow);
        m_set_table->setAlternatingRowColors(false);
        m_set_table->setRowCount(32);
        m_set_table->setColumnCount(3);
        m_set_table->horizontalHeader()->setVisible(true);
        m_set_table->verticalHeader()->setVisible(false);
        m_set_table->verticalHeader()->setMinimumSectionSize(18);
        m_set_table->verticalHeader()->setDefaultSectionSize(21);

        gridLayout_2->addWidget(m_set_table, 0, 1, 1, 1);

        m_set_contents_text = new QPlainTextEdit(layoutWidgetSetMaster);
        m_set_contents_text->setObjectName(QString::fromUtf8("m_set_contents_text"));
        m_set_contents_text->setMinimumSize(QSize(300, 156));
        m_set_contents_text->setMaximumSize(QSize(16777215, 156));
        m_set_contents_text->setReadOnly(true);

        gridLayout_2->addWidget(m_set_contents_text, 1, 0, 1, 1);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        m_button_show = new QPushButton(layoutWidgetSetMaster);
        m_button_show->setObjectName(QString::fromUtf8("m_button_show"));

        gridLayout->addWidget(m_button_show, 2, 2, 1, 1);

        m_button_toggle_trigger_mode = new QPushButton(layoutWidgetSetMaster);
        m_button_toggle_trigger_mode->setObjectName(QString::fromUtf8("m_button_toggle_trigger_mode"));

        gridLayout->addWidget(m_button_toggle_trigger_mode, 1, 2, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_2, 1, 5, 1, 1);

        m_button_set_0 = new QPushButton(layoutWidgetSetMaster);
        m_button_set_0->setObjectName(QString::fromUtf8("m_button_set_0"));

        gridLayout->addWidget(m_button_set_0, 1, 3, 1, 1);

        m_set_number_text = new QLineEdit(layoutWidgetSetMaster);
        m_set_number_text->setObjectName(QString::fromUtf8("m_set_number_text"));
        m_set_number_text->setEnabled(false);

        gridLayout->addWidget(m_set_number_text, 0, 2, 1, 1);

        m_set_name_text = new QLineEdit(layoutWidgetSetMaster);
        m_set_name_text->setObjectName(QString::fromUtf8("m_set_name_text"));
        m_set_name_text->setEnabled(false);
        m_set_name_text->setReadOnly(true);

        gridLayout->addWidget(m_set_name_text, 0, 3, 1, 1);

        m_button_delete = new QPushButton(layoutWidgetSetMaster);
        m_button_delete->setObjectName(QString::fromUtf8("m_button_delete"));

        gridLayout->addWidget(m_button_delete, 2, 3, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 1, 0, 1, 1);

        m_button_down = new QPushButton(layoutWidgetSetMaster);
        m_button_down->setObjectName(QString::fromUtf8("m_button_down"));
        m_button_down->setEnabled(true);
        m_button_down->setMinimumSize(QSize(64, 0));

        gridLayout->addWidget(m_button_down, 1, 1, 1, 1);

        m_button_up = new QPushButton(layoutWidgetSetMaster);
        m_button_up->setObjectName(QString::fromUtf8("m_button_up"));
        m_button_up->setEnabled(true);
        m_button_up->setMinimumSize(QSize(64, 0));

        gridLayout->addWidget(m_button_up, 1, 4, 1, 1);

        label = new QLabel(layoutWidgetSetMaster);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        gridLayout->addWidget(label, 0, 1, 1, 1);

        label_2 = new QLabel(layoutWidgetSetMaster);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 4, 1, 1);


        gridLayout_2->addLayout(gridLayout, 1, 1, 1, 1);

        QWidget::setTabOrder(m_set_table, m_set_contents_text);
        QWidget::setTabOrder(m_set_contents_text, m_set_number_text);
        QWidget::setTabOrder(m_set_number_text, m_set_name_text);
        QWidget::setTabOrder(m_set_name_text, m_button_down);
        QWidget::setTabOrder(m_button_down, m_button_toggle_trigger_mode);
        QWidget::setTabOrder(m_button_toggle_trigger_mode, m_button_set_0);
        QWidget::setTabOrder(m_button_set_0, m_button_up);
        QWidget::setTabOrder(m_button_up, m_button_show);
        QWidget::setTabOrder(m_button_show, m_button_delete);

        retranslateUi(qsetmaster);

        QMetaObject::connectSlotsByName(qsetmaster);
    } // setupUi

    void retranslateUi(QFrame *qsetmaster)
    {
        qsetmaster->setWindowTitle(QCoreApplication::translate("qsetmaster", "Set Master", nullptr));
#if QT_CONFIG(tooltip)
        qsetmaster->setToolTip(QCoreApplication::translate("qsetmaster", "Set Master allows creation, viewing, deletion, and moving of sets.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        qsetmaster->setWhatsThis(QCoreApplication::translate("qsetmaster", "The Set Master", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        m_set_contents_text->setToolTip(QCoreApplication::translate("qsetmaster", "Shows the number and name of each pattern\n"
"in the screen-set.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_show->setToolTip(QCoreApplication::translate("qsetmaster", "Show a condensed list of sets and their patterns.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_show->setText(QCoreApplication::translate("qsetmaster", "Show Set Info", nullptr));
#if QT_CONFIG(tooltip)
        m_button_toggle_trigger_mode->setToolTip(QCoreApplication::translate("qsetmaster", "Move the selected set down in the list.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_toggle_trigger_mode->setText(QCoreApplication::translate("qsetmaster", "Triggers", nullptr));
#if QT_CONFIG(tooltip)
        m_button_set_0->setToolTip(QCoreApplication::translate("qsetmaster", "Move the selected set up in the list.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_set_0->setText(QCoreApplication::translate("qsetmaster", "Set 0", nullptr));
#if QT_CONFIG(tooltip)
        m_button_delete->setToolTip(QCoreApplication::translate("qsetmaster", "Clears the selected set. No undo! Careful!", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_delete->setText(QCoreApplication::translate("qsetmaster", "Clear Set", nullptr));
        m_button_down->setText(QCoreApplication::translate("qsetmaster", "Down", nullptr));
        m_button_up->setText(QCoreApplication::translate("qsetmaster", "Up", nullptr));
        label->setText(QCoreApplication::translate("qsetmaster", "Set No.", nullptr));
        label_2->setText(QCoreApplication::translate("qsetmaster", "Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsetmaster: public Ui_qsetmaster {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSETMASTER_H
