/********************************************************************************
** Form generated from reading UI file 'qseqeditframe64.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSEQEDITFRAME64_H
#define QSEQEDITFRAME64_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qscrollmaster.h"
#include "qscrollslave.h"

QT_BEGIN_NAMESPACE

class Ui_qseqeditframe64
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *verticalLayout_2;
    qscrollslave *keysScrollArea;
    QWidget *keysScrollAreaContents;
    QPushButton *tooltip_button;
    qscrollmaster *rollScrollArea;
    QWidget *rollScrollAreaContents;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QPushButton *m_toggle_transpose;
    QPushButton *m_map_notes;
    QPushButton *m_toggle_drum;
    QPushButton *m_button_chord;
    QPushButton *btn_spare_3;
    qscrollslave *dataScrollArea;
    QWidget *dataScrollAreaContents;
    QVBoxLayout *verticalLayout_3;
    QPushButton *btn_show_scale_or_chords;
    QPushButton *btn_filter_painted_notes;
    QPushButton *btn_show_hex;
    QPushButton *btn_level_numbers;
    QPushButton *btn_spare_0;
    QHBoxLayout *topBar2Layout;
    QPushButton *m_button_undo;
    QPushButton *m_button_redo;
    QPushButton *m_button_quantize;
    QPushButton *m_button_tools;
    QPushButton *m_toggle_follow;
    QPushButton *m_button_snap;
    QComboBox *m_combo_snap;
    QPushButton *m_button_note;
    QComboBox *m_combo_note;
    QPushButton *m_button_zoom;
    QComboBox *m_combo_zoom;
    QPushButton *m_button_key;
    QComboBox *m_combo_key;
    QPushButton *m_button_scale;
    QComboBox *m_combo_scale;
    QPushButton *m_button_sequence;
    QLineEdit *m_entry_sequence;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *bottomBarLayout;
    QPushButton *m_button_event;
    QPushButton *m_button_data;
    QLineEdit *m_entry_data;
    QPushButton *m_button_loop;
    QPushButton *m_button_lfo;
    QSpinBox *m_spin_loop_count;
    QPushButton *m_toggle_play;
    QPushButton *m_toggle_thru;
    QPushButton *m_toggle_record;
    QPushButton *m_toggle_qrecord;
    QComboBox *m_combo_rec_type;
    QPushButton *m_button_rec_vol;
    QComboBox *m_combo_rec_vol;
    QHBoxLayout *timeHorizLayout;
    QHBoxLayout *horizontalLayout_4;
    QPushButton *btnKeyVZoomOut;
    QPushButton *btnKeyVZoomReset;
    QPushButton *btnKeyVZoomIn;
    qscrollslave *timeScrollArea;
    QWidget *timeScrollAreaContents;
    QVBoxLayout *verticalLayout_6;
    QPushButton *spacer_button_1;
    QHBoxLayout *topBar1Layout;
    QLabel *m_label_seqnumber;
    QLineEdit *m_entry_name;
    QPushButton *m_button_log_timesig;
    QPushButton *m_button_bpm;
    QComboBox *m_combo_bpm;
    QPushButton *m_button_bw;
    QComboBox *m_combo_bw;
    QPushButton *m_button_length;
    QComboBox *m_combo_length;
    QComboBox *m_combo_chord;
    QPushButton *m_button_bus;
    QComboBox *m_combo_bus;
    QPushButton *m_button_channel;
    QComboBox *m_combo_channel;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *eventHorizLayout;
    QPushButton *m_button_note_entry;
    qscrollslave *eventScrollArea;
    QWidget *eventScrollAreaContents;
    QVBoxLayout *verticalLayout_4;
    QPushButton *btn_pattern_fix;

    void setupUi(QFrame *qseqeditframe64)
    {
        if (qseqeditframe64->objectName().isEmpty())
            qseqeditframe64->setObjectName(QString::fromUtf8("qseqeditframe64"));
        qseqeditframe64->resize(750, 486);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qseqeditframe64->sizePolicy().hasHeightForWidth());
        qseqeditframe64->setSizePolicy(sizePolicy);
        qseqeditframe64->setMinimumSize(QSize(700, 458));
        gridLayout = new QGridLayout(qseqeditframe64);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setSizeConstraint(QLayout::SetMinimumSize);
        gridLayout->setHorizontalSpacing(6);
        gridLayout->setVerticalSpacing(1);
        gridLayout->setContentsMargins(2, 2, 2, 2);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(5);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setSizeConstraint(QLayout::SetFixedSize);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setSizeConstraint(QLayout::SetMinimumSize);
        keysScrollArea = new qscrollslave(qseqeditframe64);
        keysScrollArea->setObjectName(QString::fromUtf8("keysScrollArea"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(keysScrollArea->sizePolicy().hasHeightForWidth());
        keysScrollArea->setSizePolicy(sizePolicy1);
        keysScrollArea->setMinimumSize(QSize(40, 64));
        keysScrollArea->setMaximumSize(QSize(48, 16777215));
        keysScrollArea->setFrameShape(QFrame::Box);
        keysScrollArea->setFrameShadow(QFrame::Raised);
        keysScrollArea->setLineWidth(1);
        keysScrollArea->setMidLineWidth(1);
        keysScrollArea->setWidgetResizable(true);
        keysScrollAreaContents = new QWidget();
        keysScrollAreaContents->setObjectName(QString::fromUtf8("keysScrollAreaContents"));
        keysScrollAreaContents->setGeometry(QRect(0, 0, 42, 134));
        keysScrollArea->setWidget(keysScrollAreaContents);

        verticalLayout_2->addWidget(keysScrollArea);

        tooltip_button = new QPushButton(qseqeditframe64);
        tooltip_button->setObjectName(QString::fromUtf8("tooltip_button"));
        tooltip_button->setMinimumSize(QSize(48, 10));
        tooltip_button->setMaximumSize(QSize(48, 10));
        QFont font;
        font.setPointSize(5);
        font.setBold(false);
        font.setItalic(true);
        tooltip_button->setFont(font);
        tooltip_button->setFlat(false);

        verticalLayout_2->addWidget(tooltip_button);


        horizontalLayout_2->addLayout(verticalLayout_2);

        rollScrollArea = new qscrollmaster(qseqeditframe64);
        rollScrollArea->setObjectName(QString::fromUtf8("rollScrollArea"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(rollScrollArea->sizePolicy().hasHeightForWidth());
        rollScrollArea->setSizePolicy(sizePolicy2);
        rollScrollArea->setMinimumSize(QSize(300, 64));
        rollScrollArea->setFrameShadow(QFrame::Sunken);
        rollScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        rollScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
        rollScrollArea->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        rollScrollArea->setWidgetResizable(true);
        rollScrollAreaContents = new QWidget();
        rollScrollAreaContents->setObjectName(QString::fromUtf8("rollScrollAreaContents"));
        rollScrollAreaContents->setGeometry(QRect(0, 0, 673, 140));
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(rollScrollAreaContents->sizePolicy().hasHeightForWidth());
        rollScrollAreaContents->setSizePolicy(sizePolicy3);
        rollScrollArea->setWidget(rollScrollAreaContents);

        horizontalLayout_2->addWidget(rollScrollArea);


        gridLayout->addLayout(horizontalLayout_2, 3, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        verticalLayout->setContentsMargins(-1, -1, 0, -1);
        m_toggle_transpose = new QPushButton(qseqeditframe64);
        m_toggle_transpose->setObjectName(QString::fromUtf8("m_toggle_transpose"));
        QSizePolicy sizePolicy4(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(m_toggle_transpose->sizePolicy().hasHeightForWidth());
        m_toggle_transpose->setSizePolicy(sizePolicy4);
        m_toggle_transpose->setMinimumSize(QSize(48, 28));
        m_toggle_transpose->setMaximumSize(QSize(48, 28));

        verticalLayout->addWidget(m_toggle_transpose);

        m_map_notes = new QPushButton(qseqeditframe64);
        m_map_notes->setObjectName(QString::fromUtf8("m_map_notes"));
        m_map_notes->setMinimumSize(QSize(48, 28));
        m_map_notes->setMaximumSize(QSize(48, 28));

        verticalLayout->addWidget(m_map_notes);

        m_toggle_drum = new QPushButton(qseqeditframe64);
        m_toggle_drum->setObjectName(QString::fromUtf8("m_toggle_drum"));
        m_toggle_drum->setEnabled(true);
        m_toggle_drum->setMinimumSize(QSize(48, 28));
        m_toggle_drum->setMaximumSize(QSize(48, 28));
        m_toggle_drum->setIconSize(QSize(20, 20));

        verticalLayout->addWidget(m_toggle_drum);

        m_button_chord = new QPushButton(qseqeditframe64);
        m_button_chord->setObjectName(QString::fromUtf8("m_button_chord"));
        m_button_chord->setMinimumSize(QSize(48, 28));
        m_button_chord->setMaximumSize(QSize(48, 28));

        verticalLayout->addWidget(m_button_chord);

        btn_spare_3 = new QPushButton(qseqeditframe64);
        btn_spare_3->setObjectName(QString::fromUtf8("btn_spare_3"));
        btn_spare_3->setMinimumSize(QSize(48, 24));
        btn_spare_3->setMaximumSize(QSize(48, 28));

        verticalLayout->addWidget(btn_spare_3);


        horizontalLayout->addLayout(verticalLayout);

        dataScrollArea = new qscrollslave(qseqeditframe64);
        dataScrollArea->setObjectName(QString::fromUtf8("dataScrollArea"));
        QSizePolicy sizePolicy5(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(dataScrollArea->sizePolicy().hasHeightForWidth());
        dataScrollArea->setSizePolicy(sizePolicy5);
        dataScrollArea->setMinimumSize(QSize(300, 64));
        dataScrollArea->setMaximumSize(QSize(16777215, 156));
        dataScrollArea->setFrameShape(QFrame::StyledPanel);
        dataScrollArea->setFrameShadow(QFrame::Sunken);
        dataScrollArea->setWidgetResizable(true);
        dataScrollAreaContents = new QWidget();
        dataScrollAreaContents->setObjectName(QString::fromUtf8("dataScrollAreaContents"));
        dataScrollAreaContents->setGeometry(QRect(0, 0, 662, 154));
        dataScrollArea->setWidget(dataScrollAreaContents);

        horizontalLayout->addWidget(dataScrollArea);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        btn_show_scale_or_chords = new QPushButton(qseqeditframe64);
        btn_show_scale_or_chords->setObjectName(QString::fromUtf8("btn_show_scale_or_chords"));
        btn_show_scale_or_chords->setEnabled(true);
        btn_show_scale_or_chords->setMinimumSize(QSize(16, 28));
        btn_show_scale_or_chords->setMaximumSize(QSize(16, 28));
        QFont font1;
        font1.setPointSize(12);
        font1.setBold(true);
        btn_show_scale_or_chords->setFont(font1);
        btn_show_scale_or_chords->setCheckable(true);
        btn_show_scale_or_chords->setFlat(false);

        verticalLayout_3->addWidget(btn_show_scale_or_chords);

        btn_filter_painted_notes = new QPushButton(qseqeditframe64);
        btn_filter_painted_notes->setObjectName(QString::fromUtf8("btn_filter_painted_notes"));
        btn_filter_painted_notes->setEnabled(true);
        QSizePolicy sizePolicy6(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy6.setHorizontalStretch(16);
        sizePolicy6.setVerticalStretch(28);
        sizePolicy6.setHeightForWidth(btn_filter_painted_notes->sizePolicy().hasHeightForWidth());
        btn_filter_painted_notes->setSizePolicy(sizePolicy6);
        btn_filter_painted_notes->setMinimumSize(QSize(16, 28));
        btn_filter_painted_notes->setMaximumSize(QSize(16, 28));
        btn_filter_painted_notes->setFont(font1);
        btn_filter_painted_notes->setCheckable(true);
        btn_filter_painted_notes->setFlat(false);

        verticalLayout_3->addWidget(btn_filter_painted_notes);

        btn_show_hex = new QPushButton(qseqeditframe64);
        btn_show_hex->setObjectName(QString::fromUtf8("btn_show_hex"));
        btn_show_hex->setMinimumSize(QSize(16, 28));
        btn_show_hex->setMaximumSize(QSize(16, 28));
        btn_show_hex->setCheckable(true);

        verticalLayout_3->addWidget(btn_show_hex);

        btn_level_numbers = new QPushButton(qseqeditframe64);
        btn_level_numbers->setObjectName(QString::fromUtf8("btn_level_numbers"));
        btn_level_numbers->setEnabled(true);
        btn_level_numbers->setMinimumSize(QSize(16, 28));
        btn_level_numbers->setMaximumSize(QSize(16, 28));
        btn_level_numbers->setCheckable(true);
        btn_level_numbers->setFlat(false);

        verticalLayout_3->addWidget(btn_level_numbers);

        btn_spare_0 = new QPushButton(qseqeditframe64);
        btn_spare_0->setObjectName(QString::fromUtf8("btn_spare_0"));
        btn_spare_0->setEnabled(false);
        btn_spare_0->setMinimumSize(QSize(16, 28));
        btn_spare_0->setMaximumSize(QSize(16, 28));
        QFont font2;
        font2.setBold(true);
        btn_spare_0->setFont(font2);
        btn_spare_0->setFlat(false);

        verticalLayout_3->addWidget(btn_spare_0);


        horizontalLayout->addLayout(verticalLayout_3);


        gridLayout->addLayout(horizontalLayout, 5, 0, 1, 1);

        topBar2Layout = new QHBoxLayout();
        topBar2Layout->setSpacing(0);
        topBar2Layout->setObjectName(QString::fromUtf8("topBar2Layout"));
        topBar2Layout->setSizeConstraint(QLayout::SetDefaultConstraint);
        m_button_undo = new QPushButton(qseqeditframe64);
        m_button_undo->setObjectName(QString::fromUtf8("m_button_undo"));
        m_button_undo->setMinimumSize(QSize(0, 28));
        m_button_undo->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_button_undo);

        m_button_redo = new QPushButton(qseqeditframe64);
        m_button_redo->setObjectName(QString::fromUtf8("m_button_redo"));
        m_button_redo->setMinimumSize(QSize(0, 28));
        m_button_redo->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_button_redo);

        m_button_quantize = new QPushButton(qseqeditframe64);
        m_button_quantize->setObjectName(QString::fromUtf8("m_button_quantize"));
        m_button_quantize->setMinimumSize(QSize(0, 28));
        m_button_quantize->setMaximumSize(QSize(80, 28));
        QFont font3;
        font3.setBold(true);
        font3.setItalic(false);
        m_button_quantize->setFont(font3);

        topBar2Layout->addWidget(m_button_quantize);

        m_button_tools = new QPushButton(qseqeditframe64);
        m_button_tools->setObjectName(QString::fromUtf8("m_button_tools"));
        m_button_tools->setMinimumSize(QSize(0, 28));
        m_button_tools->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_button_tools);

        m_toggle_follow = new QPushButton(qseqeditframe64);
        m_toggle_follow->setObjectName(QString::fromUtf8("m_toggle_follow"));
        m_toggle_follow->setMinimumSize(QSize(0, 28));
        m_toggle_follow->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_toggle_follow);

        m_button_snap = new QPushButton(qseqeditframe64);
        m_button_snap->setObjectName(QString::fromUtf8("m_button_snap"));
        m_button_snap->setMinimumSize(QSize(0, 28));
        m_button_snap->setMaximumSize(QSize(80, 28));
        m_button_snap->setIconSize(QSize(23, 12));

        topBar2Layout->addWidget(m_button_snap);

        m_combo_snap = new QComboBox(qseqeditframe64);
        m_combo_snap->setObjectName(QString::fromUtf8("m_combo_snap"));
        m_combo_snap->setMinimumSize(QSize(68, 28));
        m_combo_snap->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_combo_snap);

        m_button_note = new QPushButton(qseqeditframe64);
        m_button_note->setObjectName(QString::fromUtf8("m_button_note"));
        m_button_note->setMinimumSize(QSize(36, 28));
        m_button_note->setMaximumSize(QSize(80, 28));
        m_button_note->setIconSize(QSize(30, 16));

        topBar2Layout->addWidget(m_button_note);

        m_combo_note = new QComboBox(qseqeditframe64);
        m_combo_note->setObjectName(QString::fromUtf8("m_combo_note"));
        m_combo_note->setMinimumSize(QSize(68, 28));
        m_combo_note->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_combo_note);

        m_button_zoom = new QPushButton(qseqeditframe64);
        m_button_zoom->setObjectName(QString::fromUtf8("m_button_zoom"));
        m_button_zoom->setMinimumSize(QSize(28, 28));
        m_button_zoom->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_button_zoom);

        m_combo_zoom = new QComboBox(qseqeditframe64);
        m_combo_zoom->setObjectName(QString::fromUtf8("m_combo_zoom"));
        m_combo_zoom->setMinimumSize(QSize(64, 28));
        m_combo_zoom->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_combo_zoom);

        m_button_key = new QPushButton(qseqeditframe64);
        m_button_key->setObjectName(QString::fromUtf8("m_button_key"));
        m_button_key->setMinimumSize(QSize(28, 28));
        m_button_key->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_button_key);

        m_combo_key = new QComboBox(qseqeditframe64);
        m_combo_key->setObjectName(QString::fromUtf8("m_combo_key"));
        m_combo_key->setMinimumSize(QSize(50, 28));
        m_combo_key->setMaximumSize(QSize(80, 28));

        topBar2Layout->addWidget(m_combo_key);

        m_button_scale = new QPushButton(qseqeditframe64);
        m_button_scale->setObjectName(QString::fromUtf8("m_button_scale"));
        m_button_scale->setMinimumSize(QSize(20, 28));
        m_button_scale->setMaximumSize(QSize(80, 28));
        m_button_scale->setIconSize(QSize(26, 14));

        topBar2Layout->addWidget(m_button_scale);

        m_combo_scale = new QComboBox(qseqeditframe64);
        m_combo_scale->setObjectName(QString::fromUtf8("m_combo_scale"));
        m_combo_scale->setMinimumSize(QSize(0, 28));
        m_combo_scale->setMaximumSize(QSize(132, 28));

        topBar2Layout->addWidget(m_combo_scale);

        m_button_sequence = new QPushButton(qseqeditframe64);
        m_button_sequence->setObjectName(QString::fromUtf8("m_button_sequence"));
        m_button_sequence->setMinimumSize(QSize(30, 28));
        m_button_sequence->setMaximumSize(QSize(80, 28));
        m_button_sequence->setIconSize(QSize(26, 12));

        topBar2Layout->addWidget(m_button_sequence);

        m_entry_sequence = new QLineEdit(qseqeditframe64);
        m_entry_sequence->setObjectName(QString::fromUtf8("m_entry_sequence"));
        m_entry_sequence->setEnabled(true);
        QSizePolicy sizePolicy7(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(m_entry_sequence->sizePolicy().hasHeightForWidth());
        m_entry_sequence->setSizePolicy(sizePolicy7);
        m_entry_sequence->setMinimumSize(QSize(0, 28));
        m_entry_sequence->setMaximumSize(QSize(148, 28));
        m_entry_sequence->setReadOnly(true);

        topBar2Layout->addWidget(m_entry_sequence);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        topBar2Layout->addItem(horizontalSpacer);


        gridLayout->addLayout(topBar2Layout, 1, 0, 1, 1);

        bottomBarLayout = new QHBoxLayout();
        bottomBarLayout->setSpacing(0);
        bottomBarLayout->setObjectName(QString::fromUtf8("bottomBarLayout"));
        m_button_event = new QPushButton(qseqeditframe64);
        m_button_event->setObjectName(QString::fromUtf8("m_button_event"));
        m_button_event->setMinimumSize(QSize(48, 28));
        m_button_event->setMaximumSize(QSize(48, 28));

        bottomBarLayout->addWidget(m_button_event);

        m_button_data = new QPushButton(qseqeditframe64);
        m_button_data->setObjectName(QString::fromUtf8("m_button_data"));
        m_button_data->setMinimumSize(QSize(28, 28));
        m_button_data->setMaximumSize(QSize(28, 28));

        bottomBarLayout->addWidget(m_button_data);

        m_entry_data = new QLineEdit(qseqeditframe64);
        m_entry_data->setObjectName(QString::fromUtf8("m_entry_data"));
        m_entry_data->setEnabled(true);
        m_entry_data->setMinimumSize(QSize(156, 28));
        m_entry_data->setMaximumSize(QSize(172, 28));
        m_entry_data->setReadOnly(true);

        bottomBarLayout->addWidget(m_entry_data);

        m_button_loop = new QPushButton(qseqeditframe64);
        m_button_loop->setObjectName(QString::fromUtf8("m_button_loop"));
        m_button_loop->setMinimumSize(QSize(40, 28));
        m_button_loop->setMaximumSize(QSize(48, 28));
        m_button_loop->setIconSize(QSize(33, 12));
        m_button_loop->setCheckable(true);

        bottomBarLayout->addWidget(m_button_loop);

        m_button_lfo = new QPushButton(qseqeditframe64);
        m_button_lfo->setObjectName(QString::fromUtf8("m_button_lfo"));
        m_button_lfo->setEnabled(true);
        m_button_lfo->setMinimumSize(QSize(40, 28));
        m_button_lfo->setMaximumSize(QSize(48, 28));

        bottomBarLayout->addWidget(m_button_lfo);

        m_spin_loop_count = new QSpinBox(qseqeditframe64);
        m_spin_loop_count->setObjectName(QString::fromUtf8("m_spin_loop_count"));
        m_spin_loop_count->setMinimumSize(QSize(28, 28));
        m_spin_loop_count->setMaximumSize(QSize(64, 28));
        m_spin_loop_count->setMaximum(999);

        bottomBarLayout->addWidget(m_spin_loop_count);

        m_toggle_play = new QPushButton(qseqeditframe64);
        m_toggle_play->setObjectName(QString::fromUtf8("m_toggle_play"));
        m_toggle_play->setMinimumSize(QSize(42, 28));
        m_toggle_play->setMaximumSize(QSize(52, 28));
        m_toggle_play->setIconSize(QSize(36, 14));

        bottomBarLayout->addWidget(m_toggle_play);

        m_toggle_thru = new QPushButton(qseqeditframe64);
        m_toggle_thru->setObjectName(QString::fromUtf8("m_toggle_thru"));
        m_toggle_thru->setMinimumSize(QSize(64, 28));
        m_toggle_thru->setMaximumSize(QSize(84, 28));
        m_toggle_thru->setIconSize(QSize(60, 14));

        bottomBarLayout->addWidget(m_toggle_thru);

        m_toggle_record = new QPushButton(qseqeditframe64);
        m_toggle_record->setObjectName(QString::fromUtf8("m_toggle_record"));
        m_toggle_record->setMinimumSize(QSize(42, 28));
        m_toggle_record->setMaximumSize(QSize(52, 28));
        m_toggle_record->setIconSize(QSize(36, 14));

        bottomBarLayout->addWidget(m_toggle_record);

        m_toggle_qrecord = new QPushButton(qseqeditframe64);
        m_toggle_qrecord->setObjectName(QString::fromUtf8("m_toggle_qrecord"));
        m_toggle_qrecord->setMinimumSize(QSize(42, 28));
        m_toggle_qrecord->setMaximumSize(QSize(52, 28));
        m_toggle_qrecord->setIconSize(QSize(36, 14));

        bottomBarLayout->addWidget(m_toggle_qrecord);

        m_combo_rec_type = new QComboBox(qseqeditframe64);
        m_combo_rec_type->setObjectName(QString::fromUtf8("m_combo_rec_type"));
        m_combo_rec_type->setMinimumSize(QSize(100, 28));
        m_combo_rec_type->setMaximumSize(QSize(116, 28));
        m_combo_rec_type->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        bottomBarLayout->addWidget(m_combo_rec_type);

        m_button_rec_vol = new QPushButton(qseqeditframe64);
        m_button_rec_vol->setObjectName(QString::fromUtf8("m_button_rec_vol"));
        m_button_rec_vol->setMinimumSize(QSize(30, 28));
        m_button_rec_vol->setMaximumSize(QSize(48, 28));

        bottomBarLayout->addWidget(m_button_rec_vol);

        m_combo_rec_vol = new QComboBox(qseqeditframe64);
        m_combo_rec_vol->setObjectName(QString::fromUtf8("m_combo_rec_vol"));
        m_combo_rec_vol->setMinimumSize(QSize(72, 28));
        m_combo_rec_vol->setMaximumSize(QSize(84, 28));
        m_combo_rec_vol->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        bottomBarLayout->addWidget(m_combo_rec_vol);


        gridLayout->addLayout(bottomBarLayout, 6, 0, 1, 1);

        timeHorizLayout = new QHBoxLayout();
        timeHorizLayout->setSpacing(1);
        timeHorizLayout->setObjectName(QString::fromUtf8("timeHorizLayout"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(0);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        horizontalLayout_4->setSizeConstraint(QLayout::SetFixedSize);
        btnKeyVZoomOut = new QPushButton(qseqeditframe64);
        btnKeyVZoomOut->setObjectName(QString::fromUtf8("btnKeyVZoomOut"));
        sizePolicy4.setHeightForWidth(btnKeyVZoomOut->sizePolicy().hasHeightForWidth());
        btnKeyVZoomOut->setSizePolicy(sizePolicy4);
        btnKeyVZoomOut->setMinimumSize(QSize(16, 24));
        btnKeyVZoomOut->setMaximumSize(QSize(16, 24));

        horizontalLayout_4->addWidget(btnKeyVZoomOut);

        btnKeyVZoomReset = new QPushButton(qseqeditframe64);
        btnKeyVZoomReset->setObjectName(QString::fromUtf8("btnKeyVZoomReset"));
        btnKeyVZoomReset->setMinimumSize(QSize(18, 24));
        btnKeyVZoomReset->setMaximumSize(QSize(18, 24));

        horizontalLayout_4->addWidget(btnKeyVZoomReset);

        btnKeyVZoomIn = new QPushButton(qseqeditframe64);
        btnKeyVZoomIn->setObjectName(QString::fromUtf8("btnKeyVZoomIn"));
        btnKeyVZoomIn->setMinimumSize(QSize(16, 24));
        btnKeyVZoomIn->setMaximumSize(QSize(16, 24));

        horizontalLayout_4->addWidget(btnKeyVZoomIn);


        timeHorizLayout->addLayout(horizontalLayout_4);

        timeScrollArea = new qscrollslave(qseqeditframe64);
        timeScrollArea->setObjectName(QString::fromUtf8("timeScrollArea"));
        sizePolicy5.setHeightForWidth(timeScrollArea->sizePolicy().hasHeightForWidth());
        timeScrollArea->setSizePolicy(sizePolicy5);
        timeScrollArea->setMinimumSize(QSize(300, 28));
        timeScrollArea->setMaximumSize(QSize(16777215, 28));
        timeScrollArea->setFrameShape(QFrame::StyledPanel);
        timeScrollArea->setFrameShadow(QFrame::Sunken);
        timeScrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        timeScrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        timeScrollArea->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContents);
        timeScrollArea->setWidgetResizable(true);
        timeScrollAreaContents = new QWidget();
        timeScrollAreaContents->setObjectName(QString::fromUtf8("timeScrollAreaContents"));
        timeScrollAreaContents->setGeometry(QRect(0, 0, 666, 28));
        timeScrollAreaContents->setMinimumSize(QSize(0, 28));
        timeScrollArea->setWidget(timeScrollAreaContents);

        timeHorizLayout->addWidget(timeScrollArea);

        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        spacer_button_1 = new QPushButton(qseqeditframe64);
        spacer_button_1->setObjectName(QString::fromUtf8("spacer_button_1"));
        spacer_button_1->setEnabled(false);
        spacer_button_1->setMinimumSize(QSize(20, 28));
        spacer_button_1->setMaximumSize(QSize(20, 28));
        spacer_button_1->setFlat(true);

        verticalLayout_6->addWidget(spacer_button_1);


        timeHorizLayout->addLayout(verticalLayout_6);


        gridLayout->addLayout(timeHorizLayout, 2, 0, 1, 1);

        topBar1Layout = new QHBoxLayout();
        topBar1Layout->setSpacing(0);
        topBar1Layout->setObjectName(QString::fromUtf8("topBar1Layout"));
        topBar1Layout->setSizeConstraint(QLayout::SetDefaultConstraint);
        m_label_seqnumber = new QLabel(qseqeditframe64);
        m_label_seqnumber->setObjectName(QString::fromUtf8("m_label_seqnumber"));
        QSizePolicy sizePolicy8(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(m_label_seqnumber->sizePolicy().hasHeightForWidth());
        m_label_seqnumber->setSizePolicy(sizePolicy8);
        m_label_seqnumber->setMinimumSize(QSize(44, 28));
        m_label_seqnumber->setMaximumSize(QSize(64, 28));
        m_label_seqnumber->setFont(font2);
        m_label_seqnumber->setFrameShape(QFrame::StyledPanel);
        m_label_seqnumber->setFrameShadow(QFrame::Sunken);
        m_label_seqnumber->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        topBar1Layout->addWidget(m_label_seqnumber);

        m_entry_name = new QLineEdit(qseqeditframe64);
        m_entry_name->setObjectName(QString::fromUtf8("m_entry_name"));
        QSizePolicy sizePolicy9(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy9.setHorizontalStretch(0);
        sizePolicy9.setVerticalStretch(0);
        sizePolicy9.setHeightForWidth(m_entry_name->sizePolicy().hasHeightForWidth());
        m_entry_name->setSizePolicy(sizePolicy9);
        m_entry_name->setMinimumSize(QSize(24, 28));
        m_entry_name->setMaximumSize(QSize(200, 28));
        m_entry_name->setFont(font3);

        topBar1Layout->addWidget(m_entry_name);

        m_button_log_timesig = new QPushButton(qseqeditframe64);
        m_button_log_timesig->setObjectName(QString::fromUtf8("m_button_log_timesig"));
        m_button_log_timesig->setEnabled(false);
        m_button_log_timesig->setMinimumSize(QSize(36, 28));
        m_button_log_timesig->setMaximumSize(QSize(40, 28));

        topBar1Layout->addWidget(m_button_log_timesig);

        m_button_bpm = new QPushButton(qseqeditframe64);
        m_button_bpm->setObjectName(QString::fromUtf8("m_button_bpm"));
        sizePolicy9.setHeightForWidth(m_button_bpm->sizePolicy().hasHeightForWidth());
        m_button_bpm->setSizePolicy(sizePolicy9);
        m_button_bpm->setMinimumSize(QSize(20, 28));
        m_button_bpm->setMaximumSize(QSize(64, 28));

        topBar1Layout->addWidget(m_button_bpm);

        m_combo_bpm = new QComboBox(qseqeditframe64);
        m_combo_bpm->setObjectName(QString::fromUtf8("m_combo_bpm"));
        sizePolicy9.setHeightForWidth(m_combo_bpm->sizePolicy().hasHeightForWidth());
        m_combo_bpm->setSizePolicy(sizePolicy9);
        m_combo_bpm->setMinimumSize(QSize(50, 28));
        m_combo_bpm->setMaximumSize(QSize(80, 28));
        m_combo_bpm->setEditable(true);

        topBar1Layout->addWidget(m_combo_bpm);

        m_button_bw = new QPushButton(qseqeditframe64);
        m_button_bw->setObjectName(QString::fromUtf8("m_button_bw"));
        sizePolicy9.setHeightForWidth(m_button_bw->sizePolicy().hasHeightForWidth());
        m_button_bw->setSizePolicy(sizePolicy9);
        m_button_bw->setMinimumSize(QSize(20, 28));
        m_button_bw->setMaximumSize(QSize(64, 28));

        topBar1Layout->addWidget(m_button_bw);

        m_combo_bw = new QComboBox(qseqeditframe64);
        m_combo_bw->setObjectName(QString::fromUtf8("m_combo_bw"));
        sizePolicy9.setHeightForWidth(m_combo_bw->sizePolicy().hasHeightForWidth());
        m_combo_bw->setSizePolicy(sizePolicy9);
        m_combo_bw->setMinimumSize(QSize(50, 28));
        m_combo_bw->setMaximumSize(QSize(80, 28));
        m_combo_bw->setEditable(true);

        topBar1Layout->addWidget(m_combo_bw);

        m_button_length = new QPushButton(qseqeditframe64);
        m_button_length->setObjectName(QString::fromUtf8("m_button_length"));
        sizePolicy9.setHeightForWidth(m_button_length->sizePolicy().hasHeightForWidth());
        m_button_length->setSizePolicy(sizePolicy9);
        m_button_length->setMinimumSize(QSize(22, 28));
        m_button_length->setMaximumSize(QSize(64, 28));
        m_button_length->setIconSize(QSize(24, 20));

        topBar1Layout->addWidget(m_button_length);

        m_combo_length = new QComboBox(qseqeditframe64);
        m_combo_length->setObjectName(QString::fromUtf8("m_combo_length"));
        sizePolicy9.setHeightForWidth(m_combo_length->sizePolicy().hasHeightForWidth());
        m_combo_length->setSizePolicy(sizePolicy9);
        m_combo_length->setMinimumSize(QSize(64, 28));
        m_combo_length->setMaximumSize(QSize(80, 28));
        m_combo_length->setEditable(true);

        topBar1Layout->addWidget(m_combo_length);

        m_combo_chord = new QComboBox(qseqeditframe64);
        m_combo_chord->setObjectName(QString::fromUtf8("m_combo_chord"));
        m_combo_chord->setMinimumSize(QSize(96, 28));
        m_combo_chord->setMaximumSize(QSize(120, 28));

        topBar1Layout->addWidget(m_combo_chord);

        m_button_bus = new QPushButton(qseqeditframe64);
        m_button_bus->setObjectName(QString::fromUtf8("m_button_bus"));
        sizePolicy9.setHeightForWidth(m_button_bus->sizePolicy().hasHeightForWidth());
        m_button_bus->setSizePolicy(sizePolicy9);
        m_button_bus->setMinimumSize(QSize(20, 28));
        m_button_bus->setMaximumSize(QSize(80, 28));
        m_button_bus->setIconSize(QSize(30, 16));

        topBar1Layout->addWidget(m_button_bus);

        m_combo_bus = new QComboBox(qseqeditframe64);
        m_combo_bus->setObjectName(QString::fromUtf8("m_combo_bus"));
        sizePolicy9.setHeightForWidth(m_combo_bus->sizePolicy().hasHeightForWidth());
        m_combo_bus->setSizePolicy(sizePolicy9);
        m_combo_bus->setMinimumSize(QSize(120, 28));
        m_combo_bus->setMaximumSize(QSize(220, 28));

        topBar1Layout->addWidget(m_combo_bus);

        m_button_channel = new QPushButton(qseqeditframe64);
        m_button_channel->setObjectName(QString::fromUtf8("m_button_channel"));
        sizePolicy9.setHeightForWidth(m_button_channel->sizePolicy().hasHeightForWidth());
        m_button_channel->setSizePolicy(sizePolicy9);
        m_button_channel->setMinimumSize(QSize(32, 28));
        m_button_channel->setMaximumSize(QSize(80, 28));

        topBar1Layout->addWidget(m_button_channel);

        m_combo_channel = new QComboBox(qseqeditframe64);
        m_combo_channel->setObjectName(QString::fromUtf8("m_combo_channel"));
        sizePolicy9.setHeightForWidth(m_combo_channel->sizePolicy().hasHeightForWidth());
        m_combo_channel->setSizePolicy(sizePolicy9);
        m_combo_channel->setMinimumSize(QSize(56, 28));
        m_combo_channel->setMaximumSize(QSize(128, 28));

        topBar1Layout->addWidget(m_combo_channel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        topBar1Layout->addItem(horizontalSpacer_2);


        gridLayout->addLayout(topBar1Layout, 0, 0, 1, 1);

        eventHorizLayout = new QHBoxLayout();
        eventHorizLayout->setSpacing(3);
        eventHorizLayout->setObjectName(QString::fromUtf8("eventHorizLayout"));
        m_button_note_entry = new QPushButton(qseqeditframe64);
        m_button_note_entry->setObjectName(QString::fromUtf8("m_button_note_entry"));
        sizePolicy7.setHeightForWidth(m_button_note_entry->sizePolicy().hasHeightForWidth());
        m_button_note_entry->setSizePolicy(sizePolicy7);
        m_button_note_entry->setMinimumSize(QSize(48, 24));
        m_button_note_entry->setMaximumSize(QSize(48, 24));
        m_button_note_entry->setCheckable(true);

        eventHorizLayout->addWidget(m_button_note_entry);

        eventScrollArea = new qscrollslave(qseqeditframe64);
        eventScrollArea->setObjectName(QString::fromUtf8("eventScrollArea"));
        sizePolicy5.setHeightForWidth(eventScrollArea->sizePolicy().hasHeightForWidth());
        eventScrollArea->setSizePolicy(sizePolicy5);
        eventScrollArea->setMinimumSize(QSize(300, 24));
        eventScrollArea->setMaximumSize(QSize(16777215, 24));
        eventScrollArea->setFrameShape(QFrame::StyledPanel);
        eventScrollArea->setFrameShadow(QFrame::Sunken);
        eventScrollArea->setWidgetResizable(true);
        eventScrollAreaContents = new QWidget();
        eventScrollAreaContents->setObjectName(QString::fromUtf8("eventScrollAreaContents"));
        eventScrollAreaContents->setGeometry(QRect(0, 0, 670, 22));
        eventScrollArea->setWidget(eventScrollAreaContents);

        eventHorizLayout->addWidget(eventScrollArea);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        btn_pattern_fix = new QPushButton(qseqeditframe64);
        btn_pattern_fix->setObjectName(QString::fromUtf8("btn_pattern_fix"));
        btn_pattern_fix->setEnabled(true);
        btn_pattern_fix->setMinimumSize(QSize(16, 24));
        btn_pattern_fix->setMaximumSize(QSize(16, 24));
        btn_pattern_fix->setFlat(false);

        verticalLayout_4->addWidget(btn_pattern_fix);


        eventHorizLayout->addLayout(verticalLayout_4);


        gridLayout->addLayout(eventHorizLayout, 4, 0, 1, 1);


        retranslateUi(qseqeditframe64);

        QMetaObject::connectSlotsByName(qseqeditframe64);
    } // setupUi

    void retranslateUi(QFrame *qseqeditframe64)
    {
        qseqeditframe64->setWindowTitle(QCoreApplication::translate("qseqeditframe64", "Pattern", nullptr));
#if QT_CONFIG(tooltip)
        tooltip_button->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle display of note tool-tips\n"
"when hovering.", nullptr));
#endif // QT_CONFIG(tooltip)
        tooltip_button->setText(QString());
#if QT_CONFIG(tooltip)
        m_toggle_transpose->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggles transposability of the pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_transpose->setText(QCoreApplication::translate("qseqeditframe64", "Transp", nullptr));
#if QT_CONFIG(tooltip)
        m_map_notes->setToolTip(QCoreApplication::translate("qseqeditframe64", "If transposable, apply active note-map\n"
"to note events. [ c for selected notes ]", nullptr));
#endif // QT_CONFIG(tooltip)
        m_map_notes->setText(QCoreApplication::translate("qseqeditframe64", "Map", nullptr));
#if QT_CONFIG(tooltip)
        m_toggle_drum->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggles the drum mode of event entry.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_drum->setText(QCoreApplication::translate("qseqeditframe64", "Drum", nullptr));
#if QT_CONFIG(tooltip)
        m_button_chord->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets chord generation to Off.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_chord->setText(QCoreApplication::translate("qseqeditframe64", "Chord", nullptr));
        btn_spare_3->setText(QString());
#if QT_CONFIG(tooltip)
        btn_show_scale_or_chords->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle the showing of the active scale or chord.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_show_scale_or_chords->setText(QCoreApplication::translate("qseqeditframe64", "+", nullptr));
#if QT_CONFIG(tooltip)
        btn_filter_painted_notes->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle filtering out off-scale or off-chord notes.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_filter_painted_notes->setText(QCoreApplication::translate("qseqeditframe64", "-", nullptr));
#if QT_CONFIG(tooltip)
        btn_show_hex->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle showing some event values in hex\n"
"(0xnn) notation.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_show_hex->setText(QCoreApplication::translate("qseqeditframe64", "x", nullptr));
#if QT_CONFIG(tooltip)
        btn_level_numbers->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle the display of the horizontal\n"
"grid-line numbers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_level_numbers->setText(QCoreApplication::translate("qseqeditframe64", "#", nullptr));
#if QT_CONFIG(tooltip)
        m_button_undo->setToolTip(QCoreApplication::translate("qseqeditframe64", "Undo previous operation(s). [ Ctrl-Z ]", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_undo->setText(QCoreApplication::translate("qseqeditframe64", "U", nullptr));
#if QT_CONFIG(tooltip)
        m_button_redo->setToolTip(QCoreApplication::translate("qseqeditframe64", "Redo last operation(s). [ Shift-Ctrl-Z ]", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_redo->setText(QCoreApplication::translate("qseqeditframe64", "R", nullptr));
#if QT_CONFIG(tooltip)
        m_button_quantize->setToolTip(QCoreApplication::translate("qseqeditframe64", "Quantize the selected notes. [ q r t ]", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_quantize->setText(QCoreApplication::translate("qseqeditframe64", "Q", nullptr));
#if QT_CONFIG(shortcut)
        m_button_quantize->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+Q", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_button_tools->setToolTip(QCoreApplication::translate("qseqeditframe64", "Shows Tools popup menus.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_tools->setText(QCoreApplication::translate("qseqeditframe64", "T", nullptr));
#if QT_CONFIG(shortcut)
        m_button_tools->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_toggle_follow->setToolTip(QCoreApplication::translate("qseqeditframe64", "Set piano roll to follow progress bar.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_follow->setText(QCoreApplication::translate("qseqeditframe64", "F", nullptr));
#if QT_CONFIG(shortcut)
        m_toggle_follow->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+F", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_button_snap->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets to default drawing snap size.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_snap->setText(QCoreApplication::translate("qseqeditframe64", "S", nullptr));
#if QT_CONFIG(shortcut)
        m_button_snap->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_combo_snap->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects snap size for spacing drawn events.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_note->setToolTip(QCoreApplication::translate("qseqeditframe64", "Sets note length for drawing\n"
"to the default value, 1/16.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_note->setText(QCoreApplication::translate("qseqeditframe64", "Note", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_note->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects the note length for drawing notes.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_zoom->setToolTip(QCoreApplication::translate("qseqeditframe64", "Horizontal zoom level. Resets to default value, 1:2.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_zoom->setText(QCoreApplication::translate("qseqeditframe64", "Z", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_zoom->setToolTip(QCoreApplication::translate("qseqeditframe64", "Horizontal zoom in pixels:ticks, the number\n"
"of ticks in 1 pixel.  A lower value\n"
"zooms in [ z 0 Z ].", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_key->setToolTip(QCoreApplication::translate("qseqeditframe64", "Musical key selection. Resets key selection to C.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_key->setText(QCoreApplication::translate("qseqeditframe64", "K", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_key->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects the musical key.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_scale->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets musical scale to Off.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_scale->setText(QCoreApplication::translate("qseqeditframe64", "S", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_scale->setToolTip(QCoreApplication::translate("qseqeditframe64", "Musical scale to show on piano roll background.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_sequence->setToolTip(QCoreApplication::translate("qseqeditframe64", "Background sequence popup menu.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_sequence->setText(QCoreApplication::translate("qseqeditframe64", "Seq", nullptr));
#if QT_CONFIG(tooltip)
        m_entry_sequence->setToolTip(QCoreApplication::translate("qseqeditframe64", "Another pattern/sequence to show on piano roll background.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_entry_sequence->setText(QCoreApplication::translate("qseqeditframe64", "Off", nullptr));
#if QT_CONFIG(tooltip)
        m_button_event->setToolTip(QCoreApplication::translate("qseqeditframe64", "Event to show in data panel popup menu.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_event->setText(QCoreApplication::translate("qseqeditframe64", "Event", nullptr));
#if QT_CONFIG(tooltip)
        m_button_data->setToolTip(QCoreApplication::translate("qseqeditframe64", "Present event-to-show popup menu.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_data->setText(QCoreApplication::translate("qseqeditframe64", "Data", nullptr));
#if QT_CONFIG(tooltip)
        m_entry_data->setToolTip(QCoreApplication::translate("qseqeditframe64", "Displays name of event being shown.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_entry_data->setText(QCoreApplication::translate("qseqeditframe64", "Note On", nullptr));
#if QT_CONFIG(tooltip)
        m_button_loop->setToolTip(QCoreApplication::translate("qseqeditframe64", "Activate L/R looping.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_loop->setText(QCoreApplication::translate("qseqeditframe64", "Loop", nullptr));
#if QT_CONFIG(tooltip)
        m_button_lfo->setToolTip(QCoreApplication::translate("qseqeditframe64", "Brings up an LFO window to modulate data values.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_lfo->setText(QCoreApplication::translate("qseqeditframe64", "LFO", nullptr));
#if QT_CONFIG(tooltip)
        m_spin_loop_count->setToolTip(QCoreApplication::translate("qseqeditframe64", "The number of repetitions in Live mode.\n"
"0 (default) means unlimited repetitions.\n"
"A value of 1 is a \"one-shot\" loop.\n"
"Values up to 999 are supported by this control.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_toggle_play->setToolTip(QCoreApplication::translate("qseqeditframe64", "When active, the pattern is unmuted.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_play->setText(QCoreApplication::translate("qseqeditframe64", "Play", nullptr));
#if QT_CONFIG(tooltip)
        m_toggle_thru->setToolTip(QCoreApplication::translate("qseqeditframe64", "When active, incoming MIDI goes through to output.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_thru->setText(QCoreApplication::translate("qseqeditframe64", "Thru", nullptr));
#if QT_CONFIG(tooltip)
        m_toggle_record->setToolTip(QCoreApplication::translate("qseqeditframe64", "Record incoming MIDI. [ o ]", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_record->setText(QCoreApplication::translate("qseqeditframe64", "Record", nullptr));
#if QT_CONFIG(shortcut)
        m_toggle_record->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_toggle_qrecord->setToolTip(QCoreApplication::translate("qseqeditframe64", "Quantize-record incoming MIDI.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_qrecord->setText(QCoreApplication::translate("qseqeditframe64", "QRecord", nullptr));
#if QT_CONFIG(shortcut)
        m_toggle_qrecord->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_combo_rec_type->setToolTip(QCoreApplication::translate("qseqeditframe64", "Recording type for loop patterns: \n"
"Merge (overdub) events; Overwrite events;\n"
"Expand pattern length while recording;\n"
"Oneshot recording of a pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_combo_rec_type->setCurrentText(QString());
#if QT_CONFIG(tooltip)
        m_button_rec_vol->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets recording velocity to Free (incoming velocity).", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_rec_vol->setText(QCoreApplication::translate("qseqeditframe64", "Vel", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_rec_vol->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects Free mode (incoming velocity) of recording,\n"
"or a velocity to force upon input.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnKeyVZoomOut->setToolTip(QCoreApplication::translate("qseqeditframe64", "Vertical zoom out. [ v ].", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomOut->setText(QCoreApplication::translate("qseqeditframe64", "-", nullptr));
#if QT_CONFIG(tooltip)
        btnKeyVZoomReset->setToolTip(QCoreApplication::translate("qseqeditframe64", "Vertical zoom reset. [ 0 ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomReset->setText(QCoreApplication::translate("qseqeditframe64", "0", nullptr));
#if QT_CONFIG(tooltip)
        btnKeyVZoomIn->setToolTip(QCoreApplication::translate("qseqeditframe64", "Vertical zoom in. [ V ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomIn->setText(QCoreApplication::translate("qseqeditframe64", "+", nullptr));
        spacer_button_1->setText(QString());
#if QT_CONFIG(tooltip)
        m_label_seqnumber->setToolTip(QCoreApplication::translate("qseqeditframe64", "Pattern/sequence number.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_label_seqnumber->setText(QCoreApplication::translate("qseqeditframe64", "1023", nullptr));
#if QT_CONFIG(tooltip)
        m_entry_name->setToolTip(QCoreApplication::translate("qseqeditframe64", "Pattern/Sequence name/title.  Keep it short.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_entry_name->setText(QCoreApplication::translate("qseqeditframe64", "Untitled", nullptr));
#if QT_CONFIG(tooltip)
        m_button_log_timesig->setToolTip(QCoreApplication::translate("qseqeditframe64", "Add the current time signature as a\n"
"meta event at the L marker.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_log_timesig->setText(QCoreApplication::translate("qseqeditframe64", "3/4", nullptr));
#if QT_CONFIG(tooltip)
        m_button_bpm->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets beats/bar to 4.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_bpm->setText(QCoreApplication::translate("qseqeditframe64", "B", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_bpm->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selection of beats/bar settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_combo_bpm->setCurrentText(QCoreApplication::translate("qseqeditframe64", "4", nullptr));
#if QT_CONFIG(tooltip)
        m_button_bw->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets beat-width to 4.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_bw->setText(QCoreApplication::translate("qseqeditframe64", "W", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_bw->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects beat width (time signature denominator).", nullptr));
#endif // QT_CONFIG(tooltip)
        m_combo_bw->setCurrentText(QCoreApplication::translate("qseqeditframe64", "4", nullptr));
#if QT_CONFIG(tooltip)
        m_button_length->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets pattern length to 1.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_length->setText(QCoreApplication::translate("qseqeditframe64", "L", nullptr));
#if QT_CONFIG(shortcut)
        m_button_length->setShortcut(QCoreApplication::translate("qseqeditframe64", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_combo_length->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects pattern length. It can be\n"
"overridden by the \"Expand\" setting.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_combo_length->setCurrentText(QCoreApplication::translate("qseqeditframe64", "1", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_chord->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects a chord to create when a note is painted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_bus->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets output MIDI buss number to 0.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_bus->setText(QCoreApplication::translate("qseqeditframe64", "Buss", nullptr));
#if QT_CONFIG(shortcut)
        m_button_bus->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        m_combo_bus->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects MIDI output buss; stored with the pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_channel->setToolTip(QCoreApplication::translate("qseqeditframe64", "Resets output MIDI channel number to 1.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_channel->setText(QCoreApplication::translate("qseqeditframe64", "C", nullptr));
#if QT_CONFIG(tooltip)
        m_combo_channel->setToolTip(QCoreApplication::translate("qseqeditframe64", "Selects MIDI output channel; stored with the pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_button_note_entry->setToolTip(QCoreApplication::translate("qseqeditframe64", "Toggle between selection and note-entry modes\n"
"in the piano roll. [ p x ]\n"
"      ", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_note_entry->setText(QCoreApplication::translate("qseqeditframe64", "Note", nullptr));
#if QT_CONFIG(tooltip)
        btn_pattern_fix->setToolTip(QCoreApplication::translate("qseqeditframe64", "Show the pattern-fix dialog.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_pattern_fix->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class qseqeditframe64: public Ui_qseqeditframe64 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSEQEDITFRAME64_H
