/********************************************************************************
** Form generated from reading UI file 'qlfoframe.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QLFOFRAME_H
#define QLFOFRAME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qlfoframe
{
public:
    QWidget *horizontalLayoutWidget;
    QHBoxLayout *horizontalLayout;
    QLabel *m_lfo_main_label;
    QLabel *m_pattern_label;
    QCheckBox *m_measures_check_box;
    QCheckBox *m_multiply_check_box;
    QGroupBox *m_wave_type_group;
    QWidget *radioButtonLayoutWidget;
    QVBoxLayout *m_wave_layout;
    QRadioButton *m_radio_wave_none;
    QRadioButton *m_radio_wave_sine;
    QRadioButton *m_radio_wave_saw;
    QRadioButton *m_radio_wave_revsaw;
    QRadioButton *m_radio_wave_triangle;
    QRadioButton *m_radio_wave_exp;
    QRadioButton *m_radio_wave_revexp;
    QRadioButton *m_radio_wave_dc;
    QWidget *buttonLayoutWidget;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *m_button_reset;
    QPushButton *m_button_lock;
    QPushButton *m_button_close;
    QWidget *sliderLayoutWidget;
    QHBoxLayout *horizontalLayout_2;
    QVBoxLayout *v_value_layout;
    QLabel *m_value_label;
    QSlider *m_value_slider;
    QLineEdit *m_value_text;
    QVBoxLayout *v_range_layout;
    QLabel *m_range_label;
    QSlider *m_range_slider;
    QLineEdit *m_range_text;
    QVBoxLayout *v_speed_layout;
    QLabel *m_speed_label;
    QSlider *m_speed_slider;
    QLineEdit *m_speed_text;
    QVBoxLayout *v_phase_layout;
    QLabel *m_phase_label;
    QSlider *m_phase_slider;
    QLineEdit *m_phase_text;

    void setupUi(QFrame *qlfoframe)
    {
        if (qlfoframe->objectName().isEmpty())
            qlfoframe->setObjectName(QString::fromUtf8("qlfoframe"));
        qlfoframe->resize(663, 360);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qlfoframe->sizePolicy().hasHeightForWidth());
        qlfoframe->setSizePolicy(sizePolicy);
        horizontalLayoutWidget = new QWidget(qlfoframe);
        horizontalLayoutWidget->setObjectName(QString::fromUtf8("horizontalLayoutWidget"));
        horizontalLayoutWidget->setGeometry(QRect(10, 10, 601, 31));
        horizontalLayout = new QHBoxLayout(horizontalLayoutWidget);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        m_lfo_main_label = new QLabel(horizontalLayoutWidget);
        m_lfo_main_label->setObjectName(QString::fromUtf8("m_lfo_main_label"));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        m_lfo_main_label->setFont(font);

        horizontalLayout->addWidget(m_lfo_main_label);

        m_pattern_label = new QLabel(horizontalLayoutWidget);
        m_pattern_label->setObjectName(QString::fromUtf8("m_pattern_label"));
        QFont font1;
        font1.setItalic(true);
        m_pattern_label->setFont(font1);

        horizontalLayout->addWidget(m_pattern_label);

        m_measures_check_box = new QCheckBox(horizontalLayoutWidget);
        m_measures_check_box->setObjectName(QString::fromUtf8("m_measures_check_box"));

        horizontalLayout->addWidget(m_measures_check_box);

        m_multiply_check_box = new QCheckBox(horizontalLayoutWidget);
        m_multiply_check_box->setObjectName(QString::fromUtf8("m_multiply_check_box"));

        horizontalLayout->addWidget(m_multiply_check_box);

        m_wave_type_group = new QGroupBox(qlfoframe);
        m_wave_type_group->setObjectName(QString::fromUtf8("m_wave_type_group"));
        m_wave_type_group->setGeometry(QRect(350, 40, 291, 251));
        QFont font2;
        font2.setBold(true);
        m_wave_type_group->setFont(font2);
        radioButtonLayoutWidget = new QWidget(m_wave_type_group);
        radioButtonLayoutWidget->setObjectName(QString::fromUtf8("radioButtonLayoutWidget"));
        radioButtonLayoutWidget->setGeometry(QRect(10, 30, 251, 211));
        m_wave_layout = new QVBoxLayout(radioButtonLayoutWidget);
        m_wave_layout->setSpacing(1);
        m_wave_layout->setObjectName(QString::fromUtf8("m_wave_layout"));
        m_wave_layout->setContentsMargins(12, 0, 0, 0);
        m_radio_wave_none = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_none->setObjectName(QString::fromUtf8("m_radio_wave_none"));
        QFont font3;
        font3.setBold(false);
        m_radio_wave_none->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_none);

        m_radio_wave_sine = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_sine->setObjectName(QString::fromUtf8("m_radio_wave_sine"));
        m_radio_wave_sine->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_sine);

        m_radio_wave_saw = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_saw->setObjectName(QString::fromUtf8("m_radio_wave_saw"));
        m_radio_wave_saw->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_saw);

        m_radio_wave_revsaw = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_revsaw->setObjectName(QString::fromUtf8("m_radio_wave_revsaw"));
        m_radio_wave_revsaw->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_revsaw);

        m_radio_wave_triangle = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_triangle->setObjectName(QString::fromUtf8("m_radio_wave_triangle"));
        m_radio_wave_triangle->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_triangle);

        m_radio_wave_exp = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_exp->setObjectName(QString::fromUtf8("m_radio_wave_exp"));
        m_radio_wave_exp->setEnabled(true);
        m_radio_wave_exp->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_exp);

        m_radio_wave_revexp = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_revexp->setObjectName(QString::fromUtf8("m_radio_wave_revexp"));
        m_radio_wave_revexp->setEnabled(true);
        m_radio_wave_revexp->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_revexp);

        m_radio_wave_dc = new QRadioButton(radioButtonLayoutWidget);
        m_radio_wave_dc->setObjectName(QString::fromUtf8("m_radio_wave_dc"));
        m_radio_wave_dc->setFont(font3);

        m_wave_layout->addWidget(m_radio_wave_dc);

        buttonLayoutWidget = new QWidget(qlfoframe);
        buttonLayoutWidget->setObjectName(QString::fromUtf8("buttonLayoutWidget"));
        buttonLayoutWidget->setGeometry(QRect(350, 293, 296, 34));
        horizontalLayout_5 = new QHBoxLayout(buttonLayoutWidget);
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalLayout_5->setContentsMargins(0, 0, 0, 0);
        m_button_reset = new QPushButton(buttonLayoutWidget);
        m_button_reset->setObjectName(QString::fromUtf8("m_button_reset"));
        m_button_reset->setMinimumSize(QSize(0, 28));

        horizontalLayout_5->addWidget(m_button_reset);

        m_button_lock = new QPushButton(buttonLayoutWidget);
        m_button_lock->setObjectName(QString::fromUtf8("m_button_lock"));
        m_button_lock->setMinimumSize(QSize(0, 28));

        horizontalLayout_5->addWidget(m_button_lock);

        m_button_close = new QPushButton(buttonLayoutWidget);
        m_button_close->setObjectName(QString::fromUtf8("m_button_close"));
        m_button_close->setMinimumSize(QSize(0, 28));

        horizontalLayout_5->addWidget(m_button_close);

        sliderLayoutWidget = new QWidget(qlfoframe);
        sliderLayoutWidget->setObjectName(QString::fromUtf8("sliderLayoutWidget"));
        sliderLayoutWidget->setGeometry(QRect(18, 53, 323, 281));
        horizontalLayout_2 = new QHBoxLayout(sliderLayoutWidget);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        v_value_layout = new QVBoxLayout();
        v_value_layout->setSpacing(6);
        v_value_layout->setObjectName(QString::fromUtf8("v_value_layout"));
        m_value_label = new QLabel(sliderLayoutWidget);
        m_value_label->setObjectName(QString::fromUtf8("m_value_label"));
        m_value_label->setAlignment(Qt::AlignCenter);

        v_value_layout->addWidget(m_value_label);

        m_value_slider = new QSlider(sliderLayoutWidget);
        m_value_slider->setObjectName(QString::fromUtf8("m_value_slider"));
        m_value_slider->setMinimumSize(QSize(0, 0));
        m_value_slider->setMaximumSize(QSize(16, 16777215));
        m_value_slider->setOrientation(Qt::Vertical);
        m_value_slider->setTickPosition(QSlider::NoTicks);

        v_value_layout->addWidget(m_value_slider);

        m_value_text = new QLineEdit(sliderLayoutWidget);
        m_value_text->setObjectName(QString::fromUtf8("m_value_text"));
        m_value_text->setMinimumSize(QSize(72, 0));
        m_value_text->setAlignment(Qt::AlignCenter);

        v_value_layout->addWidget(m_value_text);


        horizontalLayout_2->addLayout(v_value_layout);

        v_range_layout = new QVBoxLayout();
        v_range_layout->setObjectName(QString::fromUtf8("v_range_layout"));
        m_range_label = new QLabel(sliderLayoutWidget);
        m_range_label->setObjectName(QString::fromUtf8("m_range_label"));
        m_range_label->setAlignment(Qt::AlignCenter);

        v_range_layout->addWidget(m_range_label);

        m_range_slider = new QSlider(sliderLayoutWidget);
        m_range_slider->setObjectName(QString::fromUtf8("m_range_slider"));
        m_range_slider->setMinimumSize(QSize(0, 0));
        m_range_slider->setMaximumSize(QSize(16, 16777215));
        m_range_slider->setOrientation(Qt::Vertical);
        m_range_slider->setTickPosition(QSlider::NoTicks);

        v_range_layout->addWidget(m_range_slider);

        m_range_text = new QLineEdit(sliderLayoutWidget);
        m_range_text->setObjectName(QString::fromUtf8("m_range_text"));
        m_range_text->setMinimumSize(QSize(72, 0));
        m_range_text->setAlignment(Qt::AlignCenter);

        v_range_layout->addWidget(m_range_text);


        horizontalLayout_2->addLayout(v_range_layout);

        v_speed_layout = new QVBoxLayout();
        v_speed_layout->setObjectName(QString::fromUtf8("v_speed_layout"));
        m_speed_label = new QLabel(sliderLayoutWidget);
        m_speed_label->setObjectName(QString::fromUtf8("m_speed_label"));
        m_speed_label->setAlignment(Qt::AlignCenter);

        v_speed_layout->addWidget(m_speed_label);

        m_speed_slider = new QSlider(sliderLayoutWidget);
        m_speed_slider->setObjectName(QString::fromUtf8("m_speed_slider"));
        m_speed_slider->setMinimumSize(QSize(0, 0));
        m_speed_slider->setMaximumSize(QSize(16, 16777215));
        m_speed_slider->setOrientation(Qt::Vertical);
        m_speed_slider->setTickPosition(QSlider::NoTicks);

        v_speed_layout->addWidget(m_speed_slider);

        m_speed_text = new QLineEdit(sliderLayoutWidget);
        m_speed_text->setObjectName(QString::fromUtf8("m_speed_text"));
        m_speed_text->setMinimumSize(QSize(72, 0));
        m_speed_text->setAlignment(Qt::AlignCenter);

        v_speed_layout->addWidget(m_speed_text);


        horizontalLayout_2->addLayout(v_speed_layout);

        v_phase_layout = new QVBoxLayout();
        v_phase_layout->setObjectName(QString::fromUtf8("v_phase_layout"));
        m_phase_label = new QLabel(sliderLayoutWidget);
        m_phase_label->setObjectName(QString::fromUtf8("m_phase_label"));
        m_phase_label->setAlignment(Qt::AlignCenter);

        v_phase_layout->addWidget(m_phase_label);

        m_phase_slider = new QSlider(sliderLayoutWidget);
        m_phase_slider->setObjectName(QString::fromUtf8("m_phase_slider"));
        m_phase_slider->setMinimumSize(QSize(0, 0));
        m_phase_slider->setMaximumSize(QSize(16, 16777215));
        m_phase_slider->setOrientation(Qt::Vertical);
        m_phase_slider->setInvertedAppearance(false);
        m_phase_slider->setInvertedControls(false);
        m_phase_slider->setTickPosition(QSlider::NoTicks);

        v_phase_layout->addWidget(m_phase_slider);

        m_phase_text = new QLineEdit(sliderLayoutWidget);
        m_phase_text->setObjectName(QString::fromUtf8("m_phase_text"));
        m_phase_text->setMinimumSize(QSize(72, 0));
        m_phase_text->setAlignment(Qt::AlignCenter);

        v_phase_layout->addWidget(m_phase_text);


        horizontalLayout_2->addLayout(v_phase_layout);

        QWidget::setTabOrder(m_measures_check_box, m_range_slider);
        QWidget::setTabOrder(m_range_slider, m_range_text);
        QWidget::setTabOrder(m_range_text, m_speed_slider);
        QWidget::setTabOrder(m_speed_slider, m_speed_text);
        QWidget::setTabOrder(m_speed_text, m_phase_slider);
        QWidget::setTabOrder(m_phase_slider, m_phase_text);
        QWidget::setTabOrder(m_phase_text, m_radio_wave_none);
        QWidget::setTabOrder(m_radio_wave_none, m_radio_wave_sine);
        QWidget::setTabOrder(m_radio_wave_sine, m_radio_wave_saw);
        QWidget::setTabOrder(m_radio_wave_saw, m_radio_wave_revsaw);
        QWidget::setTabOrder(m_radio_wave_revsaw, m_radio_wave_triangle);
        QWidget::setTabOrder(m_radio_wave_triangle, m_radio_wave_exp);
        QWidget::setTabOrder(m_radio_wave_exp, m_radio_wave_revexp);
        QWidget::setTabOrder(m_radio_wave_revexp, m_button_reset);
        QWidget::setTabOrder(m_button_reset, m_button_close);

        retranslateUi(qlfoframe);

        QMetaObject::connectSlotsByName(qlfoframe);
    } // setupUi

    void retranslateUi(QFrame *qlfoframe)
    {
        qlfoframe->setWindowTitle(QCoreApplication::translate("qlfoframe", "LFO", nullptr));
        m_lfo_main_label->setText(QCoreApplication::translate("qlfoframe", "LFO Data Editor", nullptr));
        m_pattern_label->setText(QCoreApplication::translate("qlfoframe", "Pattern #xxxx", nullptr));
#if QT_CONFIG(tooltip)
        m_measures_check_box->setToolTip(QCoreApplication::translate("qlfoframe", "Use the measure length, rather than pattern length,\n"
"as the length of the LFO period.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_measures_check_box->setText(QCoreApplication::translate("qlfoframe", "Use Measure (vs Length)", nullptr));
#if QT_CONFIG(tooltip)
        m_multiply_check_box->setToolTip(QCoreApplication::translate("qlfoframe", "If checked, multiply existing event values by the\n"
"waveform, rather than forcing the events to\n"
"match the waveform.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_multiply_check_box->setText(QCoreApplication::translate("qlfoframe", "Multiply", nullptr));
        m_wave_type_group->setTitle(QCoreApplication::translate("qlfoframe", "Wave Type", nullptr));
        m_radio_wave_none->setText(QCoreApplication::translate("qlfoframe", "&None", nullptr));
        m_radio_wave_sine->setText(QCoreApplication::translate("qlfoframe", "Sine", nullptr));
        m_radio_wave_saw->setText(QCoreApplication::translate("qlfoframe", "Sawtooth", nullptr));
        m_radio_wave_revsaw->setText(QCoreApplication::translate("qlfoframe", "Re&verse Sawtooth", nullptr));
        m_radio_wave_triangle->setText(QCoreApplication::translate("qlfoframe", "Triangle", nullptr));
        m_radio_wave_exp->setText(QCoreApplication::translate("qlfoframe", "E&xponential Rise", nullptr));
        m_radio_wave_revexp->setText(QCoreApplication::translate("qlfoframe", "Exponential Fall", nullptr));
        m_radio_wave_dc->setText(QCoreApplication::translate("qlfoframe", "DC Only", nullptr));
#if QT_CONFIG(tooltip)
        m_button_reset->setToolTip(QCoreApplication::translate("qlfoframe", "Resets the pattern events to the original set\n"
"or to the last \"locked\" set of events.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_reset->setText(QCoreApplication::translate("qlfoframe", "&Reset", nullptr));
#if QT_CONFIG(tooltip)
        m_button_lock->setToolTip(QCoreApplication::translate("qlfoframe", "Makes the current, modified events official and\n"
"non-resettable. Further modifications can be\n"
"applied.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_lock->setText(QCoreApplication::translate("qlfoframe", "&Lock", nullptr));
#if QT_CONFIG(tooltip)
        m_button_close->setToolTip(QCoreApplication::translate("qlfoframe", "Close the LFO frame. If the pattern is modified, raise\n"
"the modify flag for the MIDI tune.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_close->setText(QCoreApplication::translate("qlfoframe", "&Close", nullptr));
        m_value_label->setText(QCoreApplication::translate("qlfoframe", "DC Offset", nullptr));
#if QT_CONFIG(tooltip)
        m_value_slider->setToolTip(QCoreApplication::translate("qlfoframe", "DC offset to add to modulating waveform.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_value_text->setText(QCoreApplication::translate("qlfoframe", "100.0", nullptr));
        m_range_label->setText(QCoreApplication::translate("qlfoframe", "Depth", nullptr));
#if QT_CONFIG(tooltip)
        m_range_slider->setToolTip(QCoreApplication::translate("qlfoframe", "Depth of modulation.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_range_text->setText(QCoreApplication::translate("qlfoframe", "100.0", nullptr));
        m_speed_label->setText(QCoreApplication::translate("qlfoframe", "Periods", nullptr));
#if QT_CONFIG(tooltip)
        m_speed_slider->setToolTip(QCoreApplication::translate("qlfoframe", "Number of periods per pattern. Subject to anti-aliasing.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_speed_text->setText(QCoreApplication::translate("qlfoframe", "100.0", nullptr));
        m_phase_label->setText(QCoreApplication::translate("qlfoframe", "Phase", nullptr));
#if QT_CONFIG(tooltip)
        m_phase_slider->setToolTip(QCoreApplication::translate("qlfoframe", "Phase of the LFO, 0 to 1 is 0 to 360 degrees.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_phase_text->setText(QCoreApplication::translate("qlfoframe", "360.0", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qlfoframe: public Ui_qlfoframe {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QLFOFRAME_H
