/****************************************************************************
** Meta object code from reading C++ file 'qslivegrid.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qslivegrid.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qslivegrid.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qslivegrid_t {
    QByteArrayData data[32];
    char stringdata0[501];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qslivegrid_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qslivegrid_t qt_meta_stringdata_seq66__qslivegrid = {
    {
QT_MOC_LITERAL(0, 0, 17), // "seq66::qslivegrid"
QT_MOC_LITERAL(1, 18, 18), // "signal_call_editor"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 5), // "seqid"
QT_MOC_LITERAL(4, 44, 21), // "signal_call_editor_ex"
QT_MOC_LITERAL(5, 66, 23), // "signal_call_edit_events"
QT_MOC_LITERAL(6, 90, 17), // "signal_live_frame"
QT_MOC_LITERAL(7, 108, 5), // "ssnum"
QT_MOC_LITERAL(8, 114, 16), // "flatten_sequence"
QT_MOC_LITERAL(9, 131, 15), // "export_sequence"
QT_MOC_LITERAL(10, 147, 13), // "copy_sequence"
QT_MOC_LITERAL(11, 161, 12), // "cut_sequence"
QT_MOC_LITERAL(12, 174, 14), // "paste_sequence"
QT_MOC_LITERAL(13, 189, 14), // "merge_sequence"
QT_MOC_LITERAL(14, 204, 15), // "delete_sequence"
QT_MOC_LITERAL(15, 220, 14), // "clear_sequence"
QT_MOC_LITERAL(16, 235, 18), // "conditional_update"
QT_MOC_LITERAL(17, 254, 12), // "new_sequence"
QT_MOC_LITERAL(18, 267, 13), // "edit_sequence"
QT_MOC_LITERAL(19, 281, 16), // "edit_sequence_ex"
QT_MOC_LITERAL(20, 298, 11), // "edit_events"
QT_MOC_LITERAL(21, 310, 15), // "record_sequence"
QT_MOC_LITERAL(22, 326, 14), // "new_live_frame"
QT_MOC_LITERAL(23, 341, 18), // "slot_set_bank_name"
QT_MOC_LITERAL(24, 360, 18), // "slot_activate_bank"
QT_MOC_LITERAL(25, 379, 7), // "clicked"
QT_MOC_LITERAL(26, 387, 17), // "slot_record_style"
QT_MOC_LITERAL(27, 405, 22), // "slot_record_alteration"
QT_MOC_LITERAL(28, 428, 21), // "slot_toggle_metronome"
QT_MOC_LITERAL(29, 450, 29), // "slot_toggle_background_record"
QT_MOC_LITERAL(30, 480, 14), // "slot_grid_mode"
QT_MOC_LITERAL(31, 495, 5) // "index"

    },
    "seq66::qslivegrid\0signal_call_editor\0"
    "\0seqid\0signal_call_editor_ex\0"
    "signal_call_edit_events\0signal_live_frame\0"
    "ssnum\0flatten_sequence\0export_sequence\0"
    "copy_sequence\0cut_sequence\0paste_sequence\0"
    "merge_sequence\0delete_sequence\0"
    "clear_sequence\0conditional_update\0"
    "new_sequence\0edit_sequence\0edit_sequence_ex\0"
    "edit_events\0record_sequence\0new_live_frame\0"
    "slot_set_bank_name\0slot_activate_bank\0"
    "clicked\0slot_record_style\0"
    "slot_record_alteration\0slot_toggle_metronome\0"
    "slot_toggle_background_record\0"
    "slot_grid_mode\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qslivegrid[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      26,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  144,    2, 0x06 /* Public */,
       4,    1,  147,    2, 0x06 /* Public */,
       5,    1,  150,    2, 0x06 /* Public */,
       6,    1,  153,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0,  156,    2, 0x08 /* Private */,
       9,    0,  157,    2, 0x08 /* Private */,
      10,    0,  158,    2, 0x08 /* Private */,
      11,    0,  159,    2, 0x08 /* Private */,
      12,    0,  160,    2, 0x08 /* Private */,
      13,    0,  161,    2, 0x08 /* Private */,
      14,    0,  162,    2, 0x08 /* Private */,
      15,    0,  163,    2, 0x08 /* Private */,
      16,    0,  164,    2, 0x08 /* Private */,
      17,    0,  165,    2, 0x08 /* Private */,
      18,    0,  166,    2, 0x08 /* Private */,
      19,    0,  167,    2, 0x08 /* Private */,
      20,    0,  168,    2, 0x08 /* Private */,
      21,    0,  169,    2, 0x08 /* Private */,
      22,    0,  170,    2, 0x08 /* Private */,
      23,    0,  171,    2, 0x08 /* Private */,
      24,    1,  172,    2, 0x08 /* Private */,
      26,    1,  175,    2, 0x08 /* Private */,
      27,    1,  178,    2, 0x08 /* Private */,
      28,    1,  181,    2, 0x08 /* Private */,
      29,    1,  184,    2, 0x08 /* Private */,
      30,    1,  187,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    7,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::Int,   31,

       0        // eod
};

void seq66::qslivegrid::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qslivegrid *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->signal_call_editor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->signal_call_editor_ex((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->signal_call_edit_events((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->signal_live_frame((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->flatten_sequence(); break;
        case 5: _t->export_sequence(); break;
        case 6: _t->copy_sequence(); break;
        case 7: _t->cut_sequence(); break;
        case 8: _t->paste_sequence(); break;
        case 9: _t->merge_sequence(); break;
        case 10: _t->delete_sequence(); break;
        case 11: _t->clear_sequence(); break;
        case 12: _t->conditional_update(); break;
        case 13: _t->new_sequence(); break;
        case 14: _t->edit_sequence(); break;
        case 15: _t->edit_sequence_ex(); break;
        case 16: _t->edit_events(); break;
        case 17: _t->record_sequence(); break;
        case 18: _t->new_live_frame(); break;
        case 19: _t->slot_set_bank_name(); break;
        case 20: _t->slot_activate_bank((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->slot_record_style((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->slot_record_alteration((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->slot_toggle_metronome((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->slot_toggle_background_record((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->slot_grid_mode((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (qslivegrid::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qslivegrid::signal_call_editor)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (qslivegrid::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qslivegrid::signal_call_editor_ex)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (qslivegrid::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qslivegrid::signal_call_edit_events)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (qslivegrid::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&qslivegrid::signal_live_frame)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qslivegrid::staticMetaObject = { {
    QMetaObject::SuperData::link<qslivebase::staticMetaObject>(),
    qt_meta_stringdata_seq66__qslivegrid.data,
    qt_meta_data_seq66__qslivegrid,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qslivegrid::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qslivegrid::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qslivegrid.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return qslivebase::qt_metacast(_clname);
}

int seq66::qslivegrid::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qslivebase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 26;
    }
    return _id;
}

// SIGNAL 0
void seq66::qslivegrid::signal_call_editor(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void seq66::qslivegrid::signal_call_editor_ex(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void seq66::qslivegrid::signal_call_edit_events(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void seq66::qslivegrid::signal_live_frame(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
