/********************************************************************************
** Form generated from reading UI file 'qslogview.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSLOGVIEW_H
#define QSLOGVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_qslogview
{
public:
    QTextBrowser *textBrowser;
    QDialogButtonBox *buttonBox;
    QLabel *panelLabel;
    QPushButton *buttonRefresh;
    QLabel *label;

    void setupUi(QDialog *qslogview)
    {
        if (qslogview->objectName().isEmpty())
            qslogview->setObjectName(QString::fromUtf8("qslogview"));
        qslogview->resize(781, 497);
        textBrowser = new QTextBrowser(qslogview);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));
        textBrowser->setGeometry(QRect(33, 33, 701, 401));
        textBrowser->setOpenLinks(false);
        buttonBox = new QDialogButtonBox(qslogview);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setGeometry(QRect(460, 450, 111, 27));
        buttonBox->setMaximumSize(QSize(200, 16777215));
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Ok);
        buttonBox->setCenterButtons(true);
        panelLabel = new QLabel(qslogview);
        panelLabel->setObjectName(QString::fromUtf8("panelLabel"));
        panelLabel->setGeometry(QRect(30, 9, 101, 18));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        panelLabel->setFont(font);
        panelLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);
        buttonRefresh = new QPushButton(qslogview);
        buttonRefresh->setObjectName(QString::fromUtf8("buttonRefresh"));
        buttonRefresh->setGeometry(QRect(180, 450, 111, 27));
        buttonRefresh->setMaximumSize(QSize(200, 16777215));
        label = new QLabel(qslogview);
        label->setObjectName(QString::fromUtf8("label"));
        label->setGeometry(QRect(170, 8, 291, 19));
        QFont font1;
        font1.setPointSize(11);
        font1.setItalic(true);
        label->setFont(font1);

        retranslateUi(qslogview);
        QObject::connect(buttonBox, SIGNAL(accepted()), qslogview, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), qslogview, SLOT(reject()));

        QMetaObject::connectSlotsByName(qslogview);
    } // setupUi

    void retranslateUi(QDialog *qslogview)
    {
        qslogview->setWindowTitle(QCoreApplication::translate("qslogview", "Seq66 Log", nullptr));
        textBrowser->setHtml(QCoreApplication::translate("qslogview", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(Filled at runtime)</p></body></html>", nullptr));
        panelLabel->setText(QCoreApplication::translate("qslogview", "Seq66 Log", nullptr));
        buttonRefresh->setText(QCoreApplication::translate("qslogview", "&Refresh", nullptr));
        label->setText(QCoreApplication::translate("qslogview", "(latest entries at bottom)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qslogview: public Ui_qslogview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSLOGVIEW_H
