/****************************************************************************
** Meta object code from reading C++ file 'qseqeditframe64.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qseqeditframe64.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qseqeditframe64.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qseqeditframe64_t {
    QByteArrayData data[73];
    char stringdata0[1111];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qseqeditframe64_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qseqeditframe64_t qt_meta_stringdata_seq66__qseqeditframe64 = {
    {
QT_MOC_LITERAL(0, 0, 22), // "seq66::qseqeditframe64"
QT_MOC_LITERAL(1, 23, 18), // "conditional_update"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 15), // "slot_reset_zoom"
QT_MOC_LITERAL(4, 59, 16), // "slot_update_zoom"
QT_MOC_LITERAL(5, 76, 5), // "index"
QT_MOC_LITERAL(6, 82, 15), // "update_seq_name"
QT_MOC_LITERAL(7, 98, 16), // "slot_log_timesig"
QT_MOC_LITERAL(8, 115, 16), // "slot_pattern_fix"
QT_MOC_LITERAL(9, 132, 25), // "slot_show_scale_or_chords"
QT_MOC_LITERAL(10, 158, 25), // "slot_filter_painted_notes"
QT_MOC_LITERAL(11, 184, 13), // "slot_show_hex"
QT_MOC_LITERAL(12, 198, 23), // "slot_show_level_numbers"
QT_MOC_LITERAL(13, 222, 20), // "update_beats_per_bar"
QT_MOC_LITERAL(14, 243, 18), // "text_beats_per_bar"
QT_MOC_LITERAL(15, 262, 17), // "update_beat_width"
QT_MOC_LITERAL(16, 280, 15), // "text_beat_width"
QT_MOC_LITERAL(17, 296, 19), // "reset_beats_per_bar"
QT_MOC_LITERAL(18, 316, 16), // "reset_beat_width"
QT_MOC_LITERAL(19, 333, 15), // "update_measures"
QT_MOC_LITERAL(20, 349, 18), // "text_measures_edit"
QT_MOC_LITERAL(21, 368, 14), // "reset_measures"
QT_MOC_LITERAL(22, 383, 9), // "transpose"
QT_MOC_LITERAL(23, 393, 9), // "ischecked"
QT_MOC_LITERAL(24, 403, 12), // "update_chord"
QT_MOC_LITERAL(25, 416, 11), // "reset_chord"
QT_MOC_LITERAL(26, 428, 15), // "update_midi_bus"
QT_MOC_LITERAL(27, 444, 14), // "reset_midi_bus"
QT_MOC_LITERAL(28, 459, 19), // "update_midi_channel"
QT_MOC_LITERAL(29, 479, 18), // "reset_midi_channel"
QT_MOC_LITERAL(30, 498, 4), // "undo"
QT_MOC_LITERAL(31, 503, 4), // "redo"
QT_MOC_LITERAL(32, 508, 5), // "tools"
QT_MOC_LITERAL(33, 514, 16), // "select_all_notes"
QT_MOC_LITERAL(34, 531, 22), // "inverse_note_selection"
QT_MOC_LITERAL(35, 554, 20), // "note_pitch_selection"
QT_MOC_LITERAL(36, 575, 14), // "quantize_notes"
QT_MOC_LITERAL(37, 590, 13), // "tighten_notes"
QT_MOC_LITERAL(38, 604, 12), // "jitter_notes"
QT_MOC_LITERAL(39, 617, 25), // "randomize_note_velocities"
QT_MOC_LITERAL(40, 643, 15), // "transpose_notes"
QT_MOC_LITERAL(41, 659, 18), // "transpose_harmonic"
QT_MOC_LITERAL(42, 678, 11), // "remap_notes"
QT_MOC_LITERAL(43, 690, 12), // "tooltip_mode"
QT_MOC_LITERAL(44, 703, 10), // "note_entry"
QT_MOC_LITERAL(45, 714, 9), // "sequences"
QT_MOC_LITERAL(46, 724, 16), // "update_grid_snap"
QT_MOC_LITERAL(47, 741, 15), // "reset_grid_snap"
QT_MOC_LITERAL(48, 757, 18), // "update_note_length"
QT_MOC_LITERAL(49, 776, 17), // "reset_note_length"
QT_MOC_LITERAL(50, 794, 10), // "update_key"
QT_MOC_LITERAL(51, 805, 9), // "reset_key"
QT_MOC_LITERAL(52, 815, 12), // "update_scale"
QT_MOC_LITERAL(53, 828, 11), // "reset_scale"
QT_MOC_LITERAL(54, 840, 11), // "editor_mode"
QT_MOC_LITERAL(55, 852, 9), // "loop_mode"
QT_MOC_LITERAL(56, 862, 6), // "events"
QT_MOC_LITERAL(57, 869, 4), // "data"
QT_MOC_LITERAL(58, 874, 14), // "show_lfo_frame"
QT_MOC_LITERAL(59, 889, 16), // "show_pattern_fix"
QT_MOC_LITERAL(60, 906, 16), // "slot_play_change"
QT_MOC_LITERAL(61, 923, 16), // "slot_thru_change"
QT_MOC_LITERAL(62, 940, 18), // "slot_record_change"
QT_MOC_LITERAL(63, 959, 20), // "slot_q_record_change"
QT_MOC_LITERAL(64, 980, 17), // "slot_record_style"
QT_MOC_LITERAL(65, 998, 21), // "slot_recording_volume"
QT_MOC_LITERAL(66, 1020, 15), // "slot_loop_count"
QT_MOC_LITERAL(67, 1036, 5), // "value"
QT_MOC_LITERAL(68, 1042, 22), // "reset_recording_volume"
QT_MOC_LITERAL(69, 1065, 11), // "slot_follow"
QT_MOC_LITERAL(70, 1077, 9), // "v_zoom_in"
QT_MOC_LITERAL(71, 1087, 10), // "v_zoom_out"
QT_MOC_LITERAL(72, 1098, 12) // "reset_v_zoom"

    },
    "seq66::qseqeditframe64\0conditional_update\0"
    "\0slot_reset_zoom\0slot_update_zoom\0"
    "index\0update_seq_name\0slot_log_timesig\0"
    "slot_pattern_fix\0slot_show_scale_or_chords\0"
    "slot_filter_painted_notes\0slot_show_hex\0"
    "slot_show_level_numbers\0update_beats_per_bar\0"
    "text_beats_per_bar\0update_beat_width\0"
    "text_beat_width\0reset_beats_per_bar\0"
    "reset_beat_width\0update_measures\0"
    "text_measures_edit\0reset_measures\0"
    "transpose\0ischecked\0update_chord\0"
    "reset_chord\0update_midi_bus\0reset_midi_bus\0"
    "update_midi_channel\0reset_midi_channel\0"
    "undo\0redo\0tools\0select_all_notes\0"
    "inverse_note_selection\0note_pitch_selection\0"
    "quantize_notes\0tighten_notes\0jitter_notes\0"
    "randomize_note_velocities\0transpose_notes\0"
    "transpose_harmonic\0remap_notes\0"
    "tooltip_mode\0note_entry\0sequences\0"
    "update_grid_snap\0reset_grid_snap\0"
    "update_note_length\0reset_note_length\0"
    "update_key\0reset_key\0update_scale\0"
    "reset_scale\0editor_mode\0loop_mode\0"
    "events\0data\0show_lfo_frame\0show_pattern_fix\0"
    "slot_play_change\0slot_thru_change\0"
    "slot_record_change\0slot_q_record_change\0"
    "slot_record_style\0slot_recording_volume\0"
    "slot_loop_count\0value\0reset_recording_volume\0"
    "slot_follow\0v_zoom_in\0v_zoom_out\0"
    "reset_v_zoom"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qseqeditframe64[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      68,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  354,    2, 0x08 /* Private */,
       3,    0,  355,    2, 0x08 /* Private */,
       4,    1,  356,    2, 0x08 /* Private */,
       6,    0,  359,    2, 0x08 /* Private */,
       7,    0,  360,    2, 0x08 /* Private */,
       8,    0,  361,    2, 0x08 /* Private */,
       9,    0,  362,    2, 0x08 /* Private */,
      10,    0,  363,    2, 0x08 /* Private */,
      11,    0,  364,    2, 0x08 /* Private */,
      12,    0,  365,    2, 0x08 /* Private */,
      13,    1,  366,    2, 0x08 /* Private */,
      14,    0,  369,    2, 0x08 /* Private */,
      15,    1,  370,    2, 0x08 /* Private */,
      16,    0,  373,    2, 0x08 /* Private */,
      17,    0,  374,    2, 0x08 /* Private */,
      18,    0,  375,    2, 0x08 /* Private */,
      19,    1,  376,    2, 0x08 /* Private */,
      20,    0,  379,    2, 0x08 /* Private */,
      21,    0,  380,    2, 0x08 /* Private */,
      22,    1,  381,    2, 0x08 /* Private */,
      24,    1,  384,    2, 0x08 /* Private */,
      25,    0,  387,    2, 0x08 /* Private */,
      26,    1,  388,    2, 0x08 /* Private */,
      27,    0,  391,    2, 0x08 /* Private */,
      28,    1,  392,    2, 0x08 /* Private */,
      29,    0,  395,    2, 0x08 /* Private */,
      30,    0,  396,    2, 0x08 /* Private */,
      31,    0,  397,    2, 0x08 /* Private */,
      32,    0,  398,    2, 0x08 /* Private */,
      33,    0,  399,    2, 0x08 /* Private */,
      34,    0,  400,    2, 0x08 /* Private */,
      35,    0,  401,    2, 0x08 /* Private */,
      36,    0,  402,    2, 0x08 /* Private */,
      37,    0,  403,    2, 0x08 /* Private */,
      38,    0,  404,    2, 0x08 /* Private */,
      39,    0,  405,    2, 0x08 /* Private */,
      40,    0,  406,    2, 0x08 /* Private */,
      41,    0,  407,    2, 0x08 /* Private */,
      42,    0,  408,    2, 0x08 /* Private */,
      43,    1,  409,    2, 0x08 /* Private */,
      44,    1,  412,    2, 0x08 /* Private */,
      45,    0,  415,    2, 0x08 /* Private */,
      46,    1,  416,    2, 0x08 /* Private */,
      47,    0,  419,    2, 0x08 /* Private */,
      48,    1,  420,    2, 0x08 /* Private */,
      49,    0,  423,    2, 0x08 /* Private */,
      50,    1,  424,    2, 0x08 /* Private */,
      51,    0,  427,    2, 0x08 /* Private */,
      52,    1,  428,    2, 0x08 /* Private */,
      53,    0,  431,    2, 0x08 /* Private */,
      54,    1,  432,    2, 0x08 /* Private */,
      55,    1,  435,    2, 0x08 /* Private */,
      56,    0,  438,    2, 0x08 /* Private */,
      57,    0,  439,    2, 0x08 /* Private */,
      58,    0,  440,    2, 0x08 /* Private */,
      59,    0,  441,    2, 0x08 /* Private */,
      60,    1,  442,    2, 0x08 /* Private */,
      61,    1,  445,    2, 0x08 /* Private */,
      62,    1,  448,    2, 0x08 /* Private */,
      63,    1,  451,    2, 0x08 /* Private */,
      64,    1,  454,    2, 0x08 /* Private */,
      65,    1,  457,    2, 0x08 /* Private */,
      66,    1,  460,    2, 0x08 /* Private */,
      68,    0,  463,    2, 0x08 /* Private */,
      69,    1,  464,    2, 0x08 /* Private */,
      70,    0,  467,    2, 0x08 /* Private */,
      71,    0,  468,    2, 0x08 /* Private */,
      72,    0,  469,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,   67,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qseqeditframe64::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qseqeditframe64 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->conditional_update(); break;
        case 1: _t->slot_reset_zoom(); break;
        case 2: _t->slot_update_zoom((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->update_seq_name(); break;
        case 4: _t->slot_log_timesig(); break;
        case 5: _t->slot_pattern_fix(); break;
        case 6: _t->slot_show_scale_or_chords(); break;
        case 7: _t->slot_filter_painted_notes(); break;
        case 8: _t->slot_show_hex(); break;
        case 9: _t->slot_show_level_numbers(); break;
        case 10: _t->update_beats_per_bar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->text_beats_per_bar(); break;
        case 12: _t->update_beat_width((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->text_beat_width(); break;
        case 14: _t->reset_beats_per_bar(); break;
        case 15: _t->reset_beat_width(); break;
        case 16: _t->update_measures((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->text_measures_edit(); break;
        case 18: _t->reset_measures(); break;
        case 19: _t->transpose((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->update_chord((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->reset_chord(); break;
        case 22: _t->update_midi_bus((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->reset_midi_bus(); break;
        case 24: _t->update_midi_channel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 25: _t->reset_midi_channel(); break;
        case 26: _t->undo(); break;
        case 27: _t->redo(); break;
        case 28: _t->tools(); break;
        case 29: _t->select_all_notes(); break;
        case 30: _t->inverse_note_selection(); break;
        case 31: _t->note_pitch_selection(); break;
        case 32: _t->quantize_notes(); break;
        case 33: _t->tighten_notes(); break;
        case 34: _t->jitter_notes(); break;
        case 35: _t->randomize_note_velocities(); break;
        case 36: _t->transpose_notes(); break;
        case 37: _t->transpose_harmonic(); break;
        case 38: _t->remap_notes(); break;
        case 39: _t->tooltip_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 40: _t->note_entry((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 41: _t->sequences(); break;
        case 42: _t->update_grid_snap((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 43: _t->reset_grid_snap(); break;
        case 44: _t->update_note_length((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 45: _t->reset_note_length(); break;
        case 46: _t->update_key((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 47: _t->reset_key(); break;
        case 48: _t->update_scale((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 49: _t->reset_scale(); break;
        case 50: _t->editor_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 51: _t->loop_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 52: _t->events(); break;
        case 53: _t->data(); break;
        case 54: _t->show_lfo_frame(); break;
        case 55: _t->show_pattern_fix(); break;
        case 56: _t->slot_play_change((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 57: _t->slot_thru_change((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 58: _t->slot_record_change((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 59: _t->slot_q_record_change((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->slot_record_style((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 61: _t->slot_recording_volume((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 62: _t->slot_loop_count((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 63: _t->reset_recording_volume(); break;
        case 64: _t->slot_follow((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 65: _t->v_zoom_in(); break;
        case 66: _t->v_zoom_out(); break;
        case 67: _t->reset_v_zoom(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qseqeditframe64::staticMetaObject = { {
    QMetaObject::SuperData::link<qseqframe::staticMetaObject>(),
    qt_meta_stringdata_seq66__qseqeditframe64.data,
    qt_meta_data_seq66__qseqeditframe64,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qseqeditframe64::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qseqeditframe64::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qseqeditframe64.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return qseqframe::qt_metacast(_clname);
}

int seq66::qseqeditframe64::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qseqframe::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 68)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 68;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 68)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 68;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
