/****************************************************************************
** Meta object code from reading C++ file 'qplaylistframe.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qplaylistframe.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qplaylistframe.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qplaylistframe_t {
    QByteArrayData data[23];
    char stringdata0[457];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qplaylistframe_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qplaylistframe_t qt_meta_stringdata_seq66__qplaylistframe = {
    {
QT_MOC_LITERAL(0, 0, 21), // "seq66::qplaylistframe"
QT_MOC_LITERAL(1, 22, 18), // "slot_list_click_ex"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 18), // "slot_song_click_ex"
QT_MOC_LITERAL(4, 61, 22), // "slot_file_create_click"
QT_MOC_LITERAL(5, 84, 19), // "slot_list_dir_click"
QT_MOC_LITERAL(6, 104, 20), // "slot_list_load_click"
QT_MOC_LITERAL(7, 125, 19), // "slot_list_add_click"
QT_MOC_LITERAL(8, 145, 22), // "slot_list_modify_click"
QT_MOC_LITERAL(9, 168, 22), // "slot_list_remove_click"
QT_MOC_LITERAL(10, 191, 20), // "slot_list_save_click"
QT_MOC_LITERAL(11, 212, 20), // "slot_song_load_click"
QT_MOC_LITERAL(12, 233, 22), // "slot_song_select_click"
QT_MOC_LITERAL(13, 256, 19), // "slot_song_add_click"
QT_MOC_LITERAL(14, 276, 22), // "slot_song_modify_click"
QT_MOC_LITERAL(15, 299, 22), // "slot_song_remove_click"
QT_MOC_LITERAL(16, 322, 26), // "slot_playlist_active_click"
QT_MOC_LITERAL(17, 349, 19), // "slot_auto_arm_click"
QT_MOC_LITERAL(18, 369, 20), // "slot_auto_play_click"
QT_MOC_LITERAL(19, 390, 23), // "slot_auto_advance_click"
QT_MOC_LITERAL(20, 414, 18), // "conditional_update"
QT_MOC_LITERAL(21, 433, 11), // "list_modify"
QT_MOC_LITERAL(22, 445, 11) // "song_modify"

    },
    "seq66::qplaylistframe\0slot_list_click_ex\0"
    "\0slot_song_click_ex\0slot_file_create_click\0"
    "slot_list_dir_click\0slot_list_load_click\0"
    "slot_list_add_click\0slot_list_modify_click\0"
    "slot_list_remove_click\0slot_list_save_click\0"
    "slot_song_load_click\0slot_song_select_click\0"
    "slot_song_add_click\0slot_song_modify_click\0"
    "slot_song_remove_click\0"
    "slot_playlist_active_click\0"
    "slot_auto_arm_click\0slot_auto_play_click\0"
    "slot_auto_advance_click\0conditional_update\0"
    "list_modify\0song_modify"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qplaylistframe[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    4,  139,    2, 0x08 /* Private */,
       3,    4,  148,    2, 0x08 /* Private */,
       4,    0,  157,    2, 0x08 /* Private */,
       5,    0,  158,    2, 0x08 /* Private */,
       6,    0,  159,    2, 0x08 /* Private */,
       7,    0,  160,    2, 0x08 /* Private */,
       8,    0,  161,    2, 0x08 /* Private */,
       9,    0,  162,    2, 0x08 /* Private */,
      10,    0,  163,    2, 0x08 /* Private */,
      11,    0,  164,    2, 0x08 /* Private */,
      12,    0,  165,    2, 0x08 /* Private */,
      13,    0,  166,    2, 0x08 /* Private */,
      14,    0,  167,    2, 0x08 /* Private */,
      15,    0,  168,    2, 0x08 /* Private */,
      16,    0,  169,    2, 0x08 /* Private */,
      17,    0,  170,    2, 0x08 /* Private */,
      18,    0,  171,    2, 0x08 /* Private */,
      19,    0,  172,    2, 0x08 /* Private */,
      20,    0,  173,    2, 0x08 /* Private */,
      21,    0,  174,    2, 0x08 /* Private */,
      21,    1,  175,    2, 0x08 /* Private */,
      21,    1,  178,    2, 0x08 /* Private */,
      22,    0,  181,    2, 0x08 /* Private */,
      22,    1,  182,    2, 0x08 /* Private */,
      22,    1,  185,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void seq66::qplaylistframe::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qplaylistframe *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slot_list_click_ex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->slot_song_click_ex((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 2: _t->slot_file_create_click(); break;
        case 3: _t->slot_list_dir_click(); break;
        case 4: _t->slot_list_load_click(); break;
        case 5: _t->slot_list_add_click(); break;
        case 6: _t->slot_list_modify_click(); break;
        case 7: _t->slot_list_remove_click(); break;
        case 8: _t->slot_list_save_click(); break;
        case 9: _t->slot_song_load_click(); break;
        case 10: _t->slot_song_select_click(); break;
        case 11: _t->slot_song_add_click(); break;
        case 12: _t->slot_song_modify_click(); break;
        case 13: _t->slot_song_remove_click(); break;
        case 14: _t->slot_playlist_active_click(); break;
        case 15: _t->slot_auto_arm_click(); break;
        case 16: _t->slot_auto_play_click(); break;
        case 17: _t->slot_auto_advance_click(); break;
        case 18: _t->conditional_update(); break;
        case 19: _t->list_modify(); break;
        case 20: _t->list_modify((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->list_modify((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->song_modify(); break;
        case 23: _t->song_modify((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->song_modify((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qplaylistframe::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qplaylistframe.data,
    qt_meta_data_seq66__qplaylistframe,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qplaylistframe::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qplaylistframe::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qplaylistframe.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qplaylistframe::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
