/****************************************************************************
** Meta object code from reading C++ file 'qmutemaster.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qmutemaster.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmutemaster.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qmutemaster_t {
    QByteArrayData data[20];
    char stringdata0[283];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qmutemaster_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qmutemaster_t qt_meta_stringdata_seq66__qmutemaster = {
    {
QT_MOC_LITERAL(0, 0, 18), // "seq66::qmutemaster"
QT_MOC_LITERAL(1, 19, 18), // "conditional_update"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 16), // "slot_table_click"
QT_MOC_LITERAL(4, 56, 3), // "row"
QT_MOC_LITERAL(5, 60, 20), // "slot_clear_all_mutes"
QT_MOC_LITERAL(6, 81, 17), // "slot_cell_changed"
QT_MOC_LITERAL(7, 99, 6), // "column"
QT_MOC_LITERAL(8, 106, 13), // "slot_bin_mode"
QT_MOC_LITERAL(9, 120, 9), // "ischecked"
QT_MOC_LITERAL(10, 130, 13), // "slot_hex_mode"
QT_MOC_LITERAL(11, 144, 12), // "slot_trigger"
QT_MOC_LITERAL(12, 157, 18), // "slot_write_to_midi"
QT_MOC_LITERAL(13, 176, 19), // "slot_write_to_mutes"
QT_MOC_LITERAL(14, 196, 16), // "slot_strip_empty"
QT_MOC_LITERAL(15, 213, 15), // "slot_load_mutes"
QT_MOC_LITERAL(16, 229, 14), // "slot_load_midi"
QT_MOC_LITERAL(17, 244, 18), // "slot_toggle_active"
QT_MOC_LITERAL(18, 263, 9), // "slot_load"
QT_MOC_LITERAL(19, 273, 9) // "slot_save"

    },
    "seq66::qmutemaster\0conditional_update\0"
    "\0slot_table_click\0row\0slot_clear_all_mutes\0"
    "slot_cell_changed\0column\0slot_bin_mode\0"
    "ischecked\0slot_hex_mode\0slot_trigger\0"
    "slot_write_to_midi\0slot_write_to_mutes\0"
    "slot_strip_empty\0slot_load_mutes\0"
    "slot_load_midi\0slot_toggle_active\0"
    "slot_load\0slot_save"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qmutemaster[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   89,    2, 0x08 /* Private */,
       3,    4,   90,    2, 0x08 /* Private */,
       5,    0,   99,    2, 0x08 /* Private */,
       6,    2,  100,    2, 0x08 /* Private */,
       8,    1,  105,    2, 0x08 /* Private */,
      10,    1,  108,    2, 0x08 /* Private */,
      11,    0,  111,    2, 0x08 /* Private */,
      12,    0,  112,    2, 0x08 /* Private */,
      13,    0,  113,    2, 0x08 /* Private */,
      14,    0,  114,    2, 0x08 /* Private */,
      15,    0,  115,    2, 0x08 /* Private */,
      16,    0,  116,    2, 0x08 /* Private */,
      17,    0,  117,    2, 0x08 /* Private */,
      18,    0,  118,    2, 0x08 /* Private */,
      19,    0,  119,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int, QMetaType::Int,    4,    2,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    4,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qmutemaster::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qmutemaster *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->conditional_update(); break;
        case 1: _t->slot_table_click((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 2: _t->slot_clear_all_mutes(); break;
        case 3: _t->slot_cell_changed((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->slot_bin_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slot_hex_mode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->slot_trigger(); break;
        case 7: _t->slot_write_to_midi(); break;
        case 8: _t->slot_write_to_mutes(); break;
        case 9: _t->slot_strip_empty(); break;
        case 10: _t->slot_load_mutes(); break;
        case 11: _t->slot_load_midi(); break;
        case 12: _t->slot_toggle_active(); break;
        case 13: _t->slot_load(); break;
        case 14: _t->slot_save(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qmutemaster::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qmutemaster.data,
    qt_meta_data_seq66__qmutemaster,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qmutemaster::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qmutemaster::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qmutemaster.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "performer::callbacks"))
        return static_cast< performer::callbacks*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qmutemaster::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
