/********************************************************************************
** Form generated from reading UI file 'qsappinfo.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSAPPINFO_H
#define QSAPPINFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextBrowser>

QT_BEGIN_NAMESPACE

class Ui_qsappinfo
{
public:
    QGridLayout *gridLayout_2;
    QLabel *panelLabel;
    QLabel *appPanelLabel;
    QTextBrowser *textBrowser;
    QGridLayout *gridLayout;
    QPushButton *buttonCommonKeys;
    QPushButton *buttonAutomation;
    QPushButton *buttonSeqroll;
    QPushButton *buttonPerfroll;
    QPushButton *buttonHotKeys;
    QPushButton *buttonMutesKeys;
    QPushButton *buttonReserved_1;
    QPushButton *buttonReserved_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *qsappinfo)
    {
        if (qsappinfo->objectName().isEmpty())
            qsappinfo->setObjectName(QString::fromUtf8("qsappinfo"));
        qsappinfo->resize(763, 507);
        gridLayout_2 = new QGridLayout(qsappinfo);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        panelLabel = new QLabel(qsappinfo);
        panelLabel->setObjectName(QString::fromUtf8("panelLabel"));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        panelLabel->setFont(font);
        panelLabel->setAlignment(Qt::AlignmentFlag::AlignCenter);

        gridLayout_2->addWidget(panelLabel, 0, 0, 1, 1);

        appPanelLabel = new QLabel(qsappinfo);
        appPanelLabel->setObjectName(QString::fromUtf8("appPanelLabel"));

        gridLayout_2->addWidget(appPanelLabel, 0, 1, 1, 2);

        textBrowser = new QTextBrowser(qsappinfo);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));

        gridLayout_2->addWidget(textBrowser, 1, 0, 1, 3);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setVerticalSpacing(3);
        buttonCommonKeys = new QPushButton(qsappinfo);
        buttonCommonKeys->setObjectName(QString::fromUtf8("buttonCommonKeys"));
        buttonCommonKeys->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonCommonKeys, 0, 0, 1, 1);

        buttonAutomation = new QPushButton(qsappinfo);
        buttonAutomation->setObjectName(QString::fromUtf8("buttonAutomation"));
        buttonAutomation->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonAutomation, 0, 1, 1, 1);

        buttonSeqroll = new QPushButton(qsappinfo);
        buttonSeqroll->setObjectName(QString::fromUtf8("buttonSeqroll"));
        buttonSeqroll->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonSeqroll, 0, 2, 1, 1);

        buttonPerfroll = new QPushButton(qsappinfo);
        buttonPerfroll->setObjectName(QString::fromUtf8("buttonPerfroll"));
        buttonPerfroll->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonPerfroll, 0, 3, 1, 1);

        buttonHotKeys = new QPushButton(qsappinfo);
        buttonHotKeys->setObjectName(QString::fromUtf8("buttonHotKeys"));
        buttonHotKeys->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonHotKeys, 1, 0, 1, 1);

        buttonMutesKeys = new QPushButton(qsappinfo);
        buttonMutesKeys->setObjectName(QString::fromUtf8("buttonMutesKeys"));
        buttonMutesKeys->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonMutesKeys, 1, 1, 1, 1);

        buttonReserved_1 = new QPushButton(qsappinfo);
        buttonReserved_1->setObjectName(QString::fromUtf8("buttonReserved_1"));
        buttonReserved_1->setEnabled(false);
        buttonReserved_1->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonReserved_1, 1, 2, 1, 1);

        buttonReserved_2 = new QPushButton(qsappinfo);
        buttonReserved_2->setObjectName(QString::fromUtf8("buttonReserved_2"));
        buttonReserved_2->setEnabled(false);
        buttonReserved_2->setMinimumSize(QSize(110, 0));

        gridLayout->addWidget(buttonReserved_2, 1, 3, 1, 1);


        gridLayout_2->addLayout(gridLayout, 2, 0, 1, 2);

        buttonBox = new QDialogButtonBox(qsappinfo);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Orientation::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::StandardButton::Ok);

        gridLayout_2->addWidget(buttonBox, 2, 2, 1, 1);

        QWidget::setTabOrder(textBrowser, buttonCommonKeys);
        QWidget::setTabOrder(buttonCommonKeys, buttonAutomation);
        QWidget::setTabOrder(buttonAutomation, buttonSeqroll);
        QWidget::setTabOrder(buttonSeqroll, buttonPerfroll);
        QWidget::setTabOrder(buttonPerfroll, buttonHotKeys);

        retranslateUi(qsappinfo);
        QObject::connect(buttonBox, SIGNAL(accepted()), qsappinfo, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), qsappinfo, SLOT(reject()));

        QMetaObject::connectSlotsByName(qsappinfo);
    } // setupUi

    void retranslateUi(QDialog *qsappinfo)
    {
        qsappinfo->setWindowTitle(QCoreApplication::translate("qsappinfo", "Keystroke Summaries", nullptr));
        panelLabel->setText(QCoreApplication::translate("qsappinfo", "Keystroke Tables", nullptr));
        appPanelLabel->setText(QCoreApplication::translate("qsappinfo", "Active panel", nullptr));
        textBrowser->setHtml(QCoreApplication::translate("qsappinfo", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Noto Sans'; font-size:12pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">(Filled at runtime)</p></body></html>", nullptr));
        buttonCommonKeys->setText(QCoreApplication::translate("qsappinfo", "&Common", nullptr));
        buttonAutomation->setText(QCoreApplication::translate("qsappinfo", "&Automation", nullptr));
        buttonSeqroll->setText(QCoreApplication::translate("qsappinfo", "&Pattern Roll", nullptr));
        buttonPerfroll->setText(QCoreApplication::translate("qsappinfo", "&Song Roll", nullptr));
        buttonHotKeys->setText(QCoreApplication::translate("qsappinfo", "&Hot-Keys", nullptr));
        buttonMutesKeys->setText(QCoreApplication::translate("qsappinfo", "&Mute Keys", nullptr));
        buttonReserved_1->setText(QString());
        buttonReserved_2->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class qsappinfo: public Ui_qsappinfo {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSAPPINFO_H
