/********************************************************************************
** Form generated from reading UI file 'qperfeditframe64.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QPERFEDITFRAME64_H
#define QPERFEDITFRAME64_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>
#include "qscrollmaster.h"

QT_BEGIN_NAMESPACE

class Ui_qperfeditframe64
{
public:
    QWidget *layoutWidgetPerfedit;
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout;
    QHBoxLayout *rollHorizLayout;
    QScrollArea *namesScrollArea;
    QWidget *scrollAreaWidgetContents_2;
    qscrollmaster *rollScrollArea;
    QWidget *scrollAreaWidgetContents_3;
    QHBoxLayout *timeHorizLayout;
    QPushButton *btnKeyVZoomOut;
    QPushButton *btnKeyVZoomReset;
    QPushButton *btnKeyVZoomIn;
    QPushButton *dummyButton;
    QScrollArea *timeScrollArea;
    QWidget *scrollAreaWidgetContents;
    QPushButton *dummyButton2;
    QHBoxLayout *topBar1Layout;
    QHBoxLayout *horizontalLayout_8;
    QPushButton *btnUndo;
    QPushButton *btnRedo;
    QPushButton *m_toggle_follow;
    QHBoxLayout *horizontalLayout_5;
    QPushButton *btnZoomOut;
    QPushButton *btnZoomIn;
    QPushButton *btnEntryMode;
    QHBoxLayout *horizontalLayout_7;
    QPushButton *btnSnap;
    QComboBox *cmbGridSnap;
    QPushButton *btnTranspose;
    QComboBox *comboTranspose;
    QPushButton *btnLoop;
    QHBoxLayout *horizontalLayout_9;
    QFrame *lineExpand;
    QPushButton *btnCollapse;
    QPushButton *btnExpand;
    QPushButton *btnExpandCopy;
    QPushButton *btnGrow;
    QPushButton *btnResetTT;
    QSpinBox *spinBoxTT;
    QPushButton *btnDuration;
    QSpacerItem *horizontalSpacer;

    void setupUi(QFrame *qperfeditframe64)
    {
        if (qperfeditframe64->objectName().isEmpty())
            qperfeditframe64->setObjectName(QString::fromUtf8("qperfeditframe64"));
        qperfeditframe64->resize(859, 387);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qperfeditframe64->sizePolicy().hasHeightForWidth());
        qperfeditframe64->setSizePolicy(sizePolicy);
        qperfeditframe64->setMinimumSize(QSize(600, 320));
        qperfeditframe64->setFrameShape(QFrame::Shape::NoFrame);
        qperfeditframe64->setFrameShadow(QFrame::Shadow::Plain);
        layoutWidgetPerfedit = new QWidget(qperfeditframe64);
        layoutWidgetPerfedit->setObjectName(QString::fromUtf8("layoutWidgetPerfedit"));
        layoutWidgetPerfedit->setGeometry(QRect(0, 0, 2, 2));
        horizontalLayout = new QHBoxLayout(layoutWidgetPerfedit);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout(qperfeditframe64);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setVerticalSpacing(2);
        gridLayout->setContentsMargins(-1, 2, -1, 2);
        rollHorizLayout = new QHBoxLayout();
        rollHorizLayout->setSpacing(0);
        rollHorizLayout->setObjectName(QString::fromUtf8("rollHorizLayout"));
        namesScrollArea = new QScrollArea(qperfeditframe64);
        namesScrollArea->setObjectName(QString::fromUtf8("namesScrollArea"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(namesScrollArea->sizePolicy().hasHeightForWidth());
        namesScrollArea->setSizePolicy(sizePolicy1);
        namesScrollArea->setMinimumSize(QSize(148, 218));
        namesScrollArea->setMaximumSize(QSize(148, 16777215));
        namesScrollArea->setFrameShape(QFrame::Shape::Box);
        namesScrollArea->setFrameShadow(QFrame::Shadow::Raised);
        namesScrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_2"));
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 16, 307));
        sizePolicy1.setHeightForWidth(scrollAreaWidgetContents_2->sizePolicy().hasHeightForWidth());
        scrollAreaWidgetContents_2->setSizePolicy(sizePolicy1);
        namesScrollArea->setWidget(scrollAreaWidgetContents_2);

        rollHorizLayout->addWidget(namesScrollArea);

        rollScrollArea = new qscrollmaster(qperfeditframe64);
        rollScrollArea->setObjectName(QString::fromUtf8("rollScrollArea"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(rollScrollArea->sizePolicy().hasHeightForWidth());
        rollScrollArea->setSizePolicy(sizePolicy2);
        rollScrollArea->setMinimumSize(QSize(520, 240));
        rollScrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_3"));
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 691, 305));
        sizePolicy2.setHeightForWidth(scrollAreaWidgetContents_3->sizePolicy().hasHeightForWidth());
        scrollAreaWidgetContents_3->setSizePolicy(sizePolicy2);
        scrollAreaWidgetContents_3->setMinimumSize(QSize(580, 240));
        rollScrollArea->setWidget(scrollAreaWidgetContents_3);

        rollHorizLayout->addWidget(rollScrollArea);


        gridLayout->addLayout(rollHorizLayout, 2, 0, 1, 1);

        timeHorizLayout = new QHBoxLayout();
        timeHorizLayout->setSpacing(1);
        timeHorizLayout->setObjectName(QString::fromUtf8("timeHorizLayout"));
        btnKeyVZoomOut = new QPushButton(qperfeditframe64);
        btnKeyVZoomOut->setObjectName(QString::fromUtf8("btnKeyVZoomOut"));
        btnKeyVZoomOut->setMinimumSize(QSize(20, 0));
        btnKeyVZoomOut->setMaximumSize(QSize(20, 16777215));

        timeHorizLayout->addWidget(btnKeyVZoomOut);

        btnKeyVZoomReset = new QPushButton(qperfeditframe64);
        btnKeyVZoomReset->setObjectName(QString::fromUtf8("btnKeyVZoomReset"));
        btnKeyVZoomReset->setMinimumSize(QSize(20, 0));
        btnKeyVZoomReset->setMaximumSize(QSize(20, 16777215));

        timeHorizLayout->addWidget(btnKeyVZoomReset);

        btnKeyVZoomIn = new QPushButton(qperfeditframe64);
        btnKeyVZoomIn->setObjectName(QString::fromUtf8("btnKeyVZoomIn"));
        btnKeyVZoomIn->setMinimumSize(QSize(20, 0));
        btnKeyVZoomIn->setMaximumSize(QSize(20, 16777215));

        timeHorizLayout->addWidget(btnKeyVZoomIn);

        dummyButton = new QPushButton(qperfeditframe64);
        dummyButton->setObjectName(QString::fromUtf8("dummyButton"));
        dummyButton->setEnabled(false);
        dummyButton->setMinimumSize(QSize(84, 26));
        dummyButton->setMaximumSize(QSize(84, 26));
        dummyButton->setFlat(false);

        timeHorizLayout->addWidget(dummyButton);

        timeScrollArea = new QScrollArea(qperfeditframe64);
        timeScrollArea->setObjectName(QString::fromUtf8("timeScrollArea"));
        QSizePolicy sizePolicy3(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(timeScrollArea->sizePolicy().hasHeightForWidth());
        timeScrollArea->setSizePolicy(sizePolicy3);
        timeScrollArea->setMinimumSize(QSize(520, 26));
        timeScrollArea->setMaximumSize(QSize(16777215, 26));
        timeScrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 670, 16));
        sizePolicy3.setHeightForWidth(scrollAreaWidgetContents->sizePolicy().hasHeightForWidth());
        scrollAreaWidgetContents->setSizePolicy(sizePolicy3);
        timeScrollArea->setWidget(scrollAreaWidgetContents);

        timeHorizLayout->addWidget(timeScrollArea);

        dummyButton2 = new QPushButton(qperfeditframe64);
        dummyButton2->setObjectName(QString::fromUtf8("dummyButton2"));
        dummyButton2->setEnabled(false);
        dummyButton2->setMinimumSize(QSize(15, 24));
        dummyButton2->setMaximumSize(QSize(20, 24));
        dummyButton2->setFlat(false);

        timeHorizLayout->addWidget(dummyButton2);


        gridLayout->addLayout(timeHorizLayout, 1, 0, 1, 1);

        topBar1Layout = new QHBoxLayout();
        topBar1Layout->setSpacing(0);
        topBar1Layout->setObjectName(QString::fromUtf8("topBar1Layout"));
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        btnUndo = new QPushButton(qperfeditframe64);
        btnUndo->setObjectName(QString::fromUtf8("btnUndo"));
        btnUndo->setMinimumSize(QSize(28, 28));
        btnUndo->setMaximumSize(QSize(80, 28));
        btnUndo->setBaseSize(QSize(80, 28));
        btnUndo->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        horizontalLayout_8->addWidget(btnUndo);

        btnRedo = new QPushButton(qperfeditframe64);
        btnRedo->setObjectName(QString::fromUtf8("btnRedo"));
        btnRedo->setMinimumSize(QSize(28, 28));
        btnRedo->setMaximumSize(QSize(80, 28));
        btnRedo->setBaseSize(QSize(80, 28));
        btnRedo->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        horizontalLayout_8->addWidget(btnRedo);

        m_toggle_follow = new QPushButton(qperfeditframe64);
        m_toggle_follow->setObjectName(QString::fromUtf8("m_toggle_follow"));
        m_toggle_follow->setMinimumSize(QSize(28, 28));
        m_toggle_follow->setMaximumSize(QSize(80, 28));
        m_toggle_follow->setBaseSize(QSize(80, 28));

        horizontalLayout_8->addWidget(m_toggle_follow);


        topBar1Layout->addLayout(horizontalLayout_8);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        btnZoomOut = new QPushButton(qperfeditframe64);
        btnZoomOut->setObjectName(QString::fromUtf8("btnZoomOut"));
        btnZoomOut->setMinimumSize(QSize(28, 28));
        btnZoomOut->setMaximumSize(QSize(80, 28));
        btnZoomOut->setBaseSize(QSize(80, 28));
        btnZoomOut->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        horizontalLayout_5->addWidget(btnZoomOut);

        btnZoomIn = new QPushButton(qperfeditframe64);
        btnZoomIn->setObjectName(QString::fromUtf8("btnZoomIn"));
        btnZoomIn->setMinimumSize(QSize(28, 28));
        btnZoomIn->setMaximumSize(QSize(80, 28));
        btnZoomIn->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        horizontalLayout_5->addWidget(btnZoomIn);

        btnEntryMode = new QPushButton(qperfeditframe64);
        btnEntryMode->setObjectName(QString::fromUtf8("btnEntryMode"));
        btnEntryMode->setMinimumSize(QSize(36, 28));
        btnEntryMode->setMaximumSize(QSize(80, 28));
        btnEntryMode->setCheckable(true);

        horizontalLayout_5->addWidget(btnEntryMode);


        topBar1Layout->addLayout(horizontalLayout_5);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        btnSnap = new QPushButton(qperfeditframe64);
        btnSnap->setObjectName(QString::fromUtf8("btnSnap"));
        btnSnap->setMinimumSize(QSize(28, 28));
        btnSnap->setMaximumSize(QSize(80, 28));
        QFont font;
        font.setBold(true);
        btnSnap->setFont(font);
        btnSnap->setCheckable(true);

        horizontalLayout_7->addWidget(btnSnap);

        cmbGridSnap = new QComboBox(qperfeditframe64);
        cmbGridSnap->setObjectName(QString::fromUtf8("cmbGridSnap"));
        cmbGridSnap->setMinimumSize(QSize(70, 28));
        cmbGridSnap->setMaximumSize(QSize(80, 28));
        cmbGridSnap->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        horizontalLayout_7->addWidget(cmbGridSnap);


        topBar1Layout->addLayout(horizontalLayout_7);

        btnTranspose = new QPushButton(qperfeditframe64);
        btnTranspose->setObjectName(QString::fromUtf8("btnTranspose"));
        btnTranspose->setMinimumSize(QSize(28, 28));
        btnTranspose->setMaximumSize(QSize(80, 28));

        topBar1Layout->addWidget(btnTranspose);

        comboTranspose = new QComboBox(qperfeditframe64);
        comboTranspose->setObjectName(QString::fromUtf8("comboTranspose"));
        comboTranspose->setMinimumSize(QSize(96, 28));
        comboTranspose->setMaximumSize(QSize(128, 28));

        topBar1Layout->addWidget(comboTranspose);

        btnLoop = new QPushButton(qperfeditframe64);
        btnLoop->setObjectName(QString::fromUtf8("btnLoop"));
        btnLoop->setMinimumSize(QSize(36, 28));
        btnLoop->setMaximumSize(QSize(80, 28));
        btnLoop->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnLoop->setIconSize(QSize(33, 12));
        btnLoop->setCheckable(true);

        topBar1Layout->addWidget(btnLoop);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        lineExpand = new QFrame(qperfeditframe64);
        lineExpand->setObjectName(QString::fromUtf8("lineExpand"));
        QSizePolicy sizePolicy4(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(lineExpand->sizePolicy().hasHeightForWidth());
        lineExpand->setSizePolicy(sizePolicy4);
        lineExpand->setFrameShape(QFrame::VLine);
        lineExpand->setFrameShadow(QFrame::Sunken);

        horizontalLayout_9->addWidget(lineExpand);

        btnCollapse = new QPushButton(qperfeditframe64);
        btnCollapse->setObjectName(QString::fromUtf8("btnCollapse"));
        btnCollapse->setMinimumSize(QSize(36, 28));
        btnCollapse->setMaximumSize(QSize(80, 28));
        btnCollapse->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnCollapse->setIconSize(QSize(33, 12));

        horizontalLayout_9->addWidget(btnCollapse);

        btnExpand = new QPushButton(qperfeditframe64);
        btnExpand->setObjectName(QString::fromUtf8("btnExpand"));
        btnExpand->setMinimumSize(QSize(36, 28));
        btnExpand->setMaximumSize(QSize(80, 28));
        btnExpand->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnExpand->setIconSize(QSize(33, 12));

        horizontalLayout_9->addWidget(btnExpand);

        btnExpandCopy = new QPushButton(qperfeditframe64);
        btnExpandCopy->setObjectName(QString::fromUtf8("btnExpandCopy"));
        btnExpandCopy->setMinimumSize(QSize(36, 28));
        btnExpandCopy->setMaximumSize(QSize(80, 28));
        btnExpandCopy->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnExpandCopy->setIconSize(QSize(33, 12));

        horizontalLayout_9->addWidget(btnExpandCopy);

        btnGrow = new QPushButton(qperfeditframe64);
        btnGrow->setObjectName(QString::fromUtf8("btnGrow"));
        btnGrow->setMinimumSize(QSize(40, 28));
        btnGrow->setMaximumSize(QSize(80, 28));
        btnGrow->setIconSize(QSize(33, 22));

        horizontalLayout_9->addWidget(btnGrow);


        topBar1Layout->addLayout(horizontalLayout_9);

        btnResetTT = new QPushButton(qperfeditframe64);
        btnResetTT->setObjectName(QString::fromUtf8("btnResetTT"));
        btnResetTT->setMinimumSize(QSize(28, 28));
        btnResetTT->setMaximumSize(QSize(80, 28));
        btnResetTT->setFont(font);

        topBar1Layout->addWidget(btnResetTT);

        spinBoxTT = new QSpinBox(qperfeditframe64);
        spinBoxTT->setObjectName(QString::fromUtf8("spinBoxTT"));
        spinBoxTT->setMinimumSize(QSize(66, 28));
        spinBoxTT->setMaximumSize(QSize(80, 28));

        topBar1Layout->addWidget(spinBoxTT);

        btnDuration = new QPushButton(qperfeditframe64);
        btnDuration->setObjectName(QString::fromUtf8("btnDuration"));
        btnDuration->setMinimumSize(QSize(86, 28));
        btnDuration->setMaximumSize(QSize(128, 28));
        btnDuration->setFont(font);
        btnDuration->setCheckable(false);
        btnDuration->setFlat(false);

        topBar1Layout->addWidget(btnDuration);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        topBar1Layout->addItem(horizontalSpacer);


        gridLayout->addLayout(topBar1Layout, 0, 0, 1, 1);


        retranslateUi(qperfeditframe64);

        cmbGridSnap->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(qperfeditframe64);
    } // setupUi

    void retranslateUi(QFrame *qperfeditframe64)
    {
        qperfeditframe64->setWindowTitle(QCoreApplication::translate("qperfeditframe64", "Frame", nullptr));
#if QT_CONFIG(tooltip)
        btnKeyVZoomOut->setToolTip(QCoreApplication::translate("qperfeditframe64", "Show triggers at half height. [ v ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomOut->setText(QCoreApplication::translate("qperfeditframe64", "-", nullptr));
#if QT_CONFIG(tooltip)
        btnKeyVZoomReset->setToolTip(QCoreApplication::translate("qperfeditframe64", "Reset triggers to normal height. [ 0 ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomReset->setText(QCoreApplication::translate("qperfeditframe64", "0", nullptr));
#if QT_CONFIG(tooltip)
        btnKeyVZoomIn->setToolTip(QCoreApplication::translate("qperfeditframe64", "Double the height of triggers. [ V ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnKeyVZoomIn->setText(QCoreApplication::translate("qperfeditframe64", "+", nullptr));
#if QT_CONFIG(tooltip)
        dummyButton->setToolTip(QCoreApplication::translate("qperfeditframe64", "Time-line and L and R markers.", nullptr));
#endif // QT_CONFIG(tooltip)
        dummyButton->setText(QString());
        dummyButton2->setText(QString());
#if QT_CONFIG(tooltip)
        btnUndo->setToolTip(QCoreApplication::translate("qperfeditframe64", "Undo previous operation(s). [ Ctrl-Z ]", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        btnUndo->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        btnUndo->setText(QString());
#if QT_CONFIG(tooltip)
        btnRedo->setToolTip(QCoreApplication::translate("qperfeditframe64", "Redo last operations(s). [ Shift-Ctrl-Z ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnRedo->setText(QString());
#if QT_CONFIG(tooltip)
        m_toggle_follow->setToolTip(QCoreApplication::translate("qperfeditframe64", "Set piano roll to follow progress bar.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_toggle_follow->setText(QString());
#if QT_CONFIG(tooltip)
        btnZoomOut->setToolTip(QCoreApplication::translate("qperfeditframe64", "Zoom out horizontally in piano roll.\n"
"[ z ] The 0 key resets zoom.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        btnZoomOut->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        btnZoomOut->setText(QString());
#if QT_CONFIG(tooltip)
        btnZoomIn->setToolTip(QCoreApplication::translate("qperfeditframe64", "Zoom in horizontally in piano roll.\n"
"[ Z ] The 0 key resets zoom.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnZoomIn->setText(QString());
#if QT_CONFIG(tooltip)
        btnEntryMode->setToolTip(QCoreApplication::translate("qperfeditframe64", "Toggles painting mode.  Or hold the\n"
"right mouse button. [ p to enter, x\n"
"to leave ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnEntryMode->setText(QCoreApplication::translate("qperfeditframe64", "Enter", nullptr));
#if QT_CONFIG(tooltip)
        btnSnap->setToolTip(QCoreApplication::translate("qperfeditframe64", "Set \"song record\" snap. An added\n"
"trigger snaps to the length of the pattern\n"
"or snap setting. Growth occurs at snap length.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnSnap->setText(QCoreApplication::translate("qperfeditframe64", "Snap", nullptr));
#if QT_CONFIG(tooltip)
        cmbGridSnap->setToolTip(QCoreApplication::translate("qperfeditframe64", "Selects movement and sizing snap\n"
"value. \"L\" = full length. \"1/N\" is 1/Nth\n"
"of a measure. Affects L/R markers. Use\n"
"L only for short patterns.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnTranspose->setToolTip(QCoreApplication::translate("qperfeditframe64", "Resets transpose to 0 in the performance.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnTranspose->setText(QCoreApplication::translate("qperfeditframe64", "Trans", nullptr));
#if QT_CONFIG(tooltip)
        comboTranspose->setToolTip(QCoreApplication::translate("qperfeditframe64", "The number of semitones to transpose the performance, for transposable patterns.\n"
"Transposition is done during playback.", nullptr));
#endif // QT_CONFIG(tooltip)
        comboTranspose->setCurrentText(QString());
#if QT_CONFIG(tooltip)
        btnLoop->setToolTip(QCoreApplication::translate("qperfeditframe64", "Set looping between L/R markers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnLoop->setText(QCoreApplication::translate("qperfeditframe64", "Loop", nullptr));
#if QT_CONFIG(tooltip)
        btnCollapse->setToolTip(QCoreApplication::translate("qperfeditframe64", "Collapse pattern between L/R markers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnCollapse->setText(QString());
#if QT_CONFIG(tooltip)
        btnExpand->setToolTip(QCoreApplication::translate("qperfeditframe64", "Expand space between L/R markers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnExpand->setText(QString());
#if QT_CONFIG(tooltip)
        btnExpandCopy->setToolTip(QCoreApplication::translate("qperfeditframe64", "Expand and copy between L/R markers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnExpandCopy->setText(QString());
#if QT_CONFIG(tooltip)
        btnGrow->setToolTip(QCoreApplication::translate("qperfeditframe64", "Expand piano roll to the right\n"
"for more editing room.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnGrow->setText(QCoreApplication::translate("qperfeditframe64", "-->", nullptr));
#if QT_CONFIG(tooltip)
        btnResetTT->setToolTip(QCoreApplication::translate("qperfeditframe64", "Resets Trigger Transpose to 0.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnResetTT->setText(QCoreApplication::translate("qperfeditframe64", "TT", nullptr));
#if QT_CONFIG(tooltip)
        spinBoxTT->setToolTip(QCoreApplication::translate("qperfeditframe64", "Transposition value for Trigger Transpose.\n"
"Range: -60 to +60 (5 octaves each way).\n"
"Apply it to a trigger via Shift-Left click.\n"
"To reset, set to 0 and apply it.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnDuration->setToolTip(QCoreApplication::translate("qperfeditframe64", "Shows duration of the song based\n"
"on the longest track or Song layout.\n"
"Click to toggle between H:M:S and B:B:T\n"
"format.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnDuration->setText(QCoreApplication::translate("qperfeditframe64", "20:20.20", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qperfeditframe64: public Ui_qperfeditframe64 {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QPERFEDITFRAME64_H
