/****************************************************************************
** Meta object code from reading C++ file 'qseditoptions.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qseditoptions.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qseditoptions.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qseditoptions_t {
    QByteArrayData data[136];
    char stringdata0[2822];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qseditoptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qseditoptions_t qt_meta_stringdata_seq66__qseditoptions = {
    {
QT_MOC_LITERAL(0, 0, 20), // "seq66::qseditoptions"
QT_MOC_LITERAL(1, 21, 14), // "slot_sets_mode"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 8), // "buttonno"
QT_MOC_LITERAL(4, 46, 15), // "slot_start_mode"
QT_MOC_LITERAL(5, 62, 14), // "slot_jack_mode"
QT_MOC_LITERAL(6, 77, 17), // "slot_jack_connect"
QT_MOC_LITERAL(7, 95, 20), // "slot_jack_disconnect"
QT_MOC_LITERAL(8, 116, 16), // "slot_master_cond"
QT_MOC_LITERAL(9, 133, 16), // "slot_time_master"
QT_MOC_LITERAL(10, 150, 22), // "slot_transport_support"
QT_MOC_LITERAL(11, 173, 14), // "slot_jack_midi"
QT_MOC_LITERAL(12, 188, 22), // "slot_jack_auto_connect"
QT_MOC_LITERAL(13, 211, 12), // "slot_io_maps"
QT_MOC_LITERAL(14, 224, 21), // "slot_activate_io_maps"
QT_MOC_LITERAL(15, 246, 12), // "slot_session"
QT_MOC_LITERAL(16, 259, 12), // "slot_nsm_url"
QT_MOC_LITERAL(17, 272, 16), // "slot_note_resume"
QT_MOC_LITERAL(18, 289, 17), // "slot_ppqn_by_text"
QT_MOC_LITERAL(19, 307, 4), // "text"
QT_MOC_LITERAL(20, 312, 24), // "slot_buffer_size_by_text"
QT_MOC_LITERAL(21, 337, 18), // "slot_use_file_ppqn"
QT_MOC_LITERAL(22, 356, 21), // "slot_song_record_snap"
QT_MOC_LITERAL(23, 378, 17), // "slot_add_time_sig"
QT_MOC_LITERAL(24, 396, 15), // "slot_key_height"
QT_MOC_LITERAL(25, 412, 15), // "slot_ui_scaling"
QT_MOC_LITERAL(26, 428, 17), // "slot_grid_spacing"
QT_MOC_LITERAL(27, 446, 18), // "slot_set_size_rows"
QT_MOC_LITERAL(28, 465, 21), // "slot_set_size_columns"
QT_MOC_LITERAL(29, 487, 23), // "slot_progress_box_width"
QT_MOC_LITERAL(30, 511, 24), // "slot_progress_box_height"
QT_MOC_LITERAL(31, 536, 23), // "slot_progress_box_shown"
QT_MOC_LITERAL(32, 560, 21), // "slot_fingerprint_size"
QT_MOC_LITERAL(33, 582, 23), // "slot_quiet_active_click"
QT_MOC_LITERAL(34, 606, 27), // "slot_load_most_recent_click"
QT_MOC_LITERAL(35, 634, 26), // "slot_show_full_paths_click"
QT_MOC_LITERAL(36, 661, 26), // "slot_long_buss_names_click"
QT_MOC_LITERAL(37, 688, 26), // "slot_pair_buss_names_click"
QT_MOC_LITERAL(38, 715, 27), // "slot_lock_main_window_click"
QT_MOC_LITERAL(39, 743, 21), // "slot_dark_theme_click"
QT_MOC_LITERAL(40, 765, 27), // "slot_swap_coordinates_click"
QT_MOC_LITERAL(41, 793, 26), // "slot_bold_grid_slots_click"
QT_MOC_LITERAL(42, 820, 26), // "slot_gridlines_thick_click"
QT_MOC_LITERAL(43, 847, 21), // "slot_elliptical_click"
QT_MOC_LITERAL(44, 869, 26), // "slot_follow_progress_click"
QT_MOC_LITERAL(45, 896, 22), // "slot_double_click_edit"
QT_MOC_LITERAL(46, 919, 23), // "slot_global_seq_feature"
QT_MOC_LITERAL(47, 943, 18), // "slot_rc_save_click"
QT_MOC_LITERAL(48, 962, 19), // "slot_usr_save_click"
QT_MOC_LITERAL(49, 982, 17), // "slot_usr_filename"
QT_MOC_LITERAL(50, 1000, 22), // "slot_load_usr_filename"
QT_MOC_LITERAL(51, 1023, 21), // "slot_mutes_save_click"
QT_MOC_LITERAL(52, 1045, 23), // "slot_mutes_active_click"
QT_MOC_LITERAL(53, 1069, 19), // "slot_mutes_filename"
QT_MOC_LITERAL(54, 1089, 24), // "slot_load_mutes_filename"
QT_MOC_LITERAL(55, 1114, 24), // "slot_playlist_save_click"
QT_MOC_LITERAL(56, 1139, 26), // "slot_playlist_active_click"
QT_MOC_LITERAL(57, 1166, 22), // "slot_playlist_filename"
QT_MOC_LITERAL(58, 1189, 27), // "slot_load_playlist_filename"
QT_MOC_LITERAL(59, 1217, 22), // "slot_ctrl_active_click"
QT_MOC_LITERAL(60, 1240, 18), // "slot_ctrl_filename"
QT_MOC_LITERAL(61, 1259, 23), // "slot_load_ctrl_filename"
QT_MOC_LITERAL(62, 1283, 23), // "slot_drums_active_click"
QT_MOC_LITERAL(63, 1307, 19), // "slot_drums_filename"
QT_MOC_LITERAL(64, 1327, 24), // "slot_load_drums_filename"
QT_MOC_LITERAL(65, 1352, 28), // "slot_stylesheet_active_click"
QT_MOC_LITERAL(66, 1381, 24), // "slot_stylesheet_filename"
QT_MOC_LITERAL(67, 1406, 29), // "slot_load_stylesheet_filename"
QT_MOC_LITERAL(68, 1436, 23), // "slot_browser_executable"
QT_MOC_LITERAL(69, 1460, 28), // "slot_load_browser_executable"
QT_MOC_LITERAL(70, 1489, 19), // "slot_pdf_executable"
QT_MOC_LITERAL(71, 1509, 31), // "slot_load_pdf_viewer_executable"
QT_MOC_LITERAL(72, 1541, 27), // "slot_patches_save_now_click"
QT_MOC_LITERAL(73, 1569, 25), // "slot_patches_active_click"
QT_MOC_LITERAL(74, 1595, 21), // "slot_patches_filename"
QT_MOC_LITERAL(75, 1617, 26), // "slot_load_patches_filename"
QT_MOC_LITERAL(76, 1644, 27), // "slot_palette_save_now_click"
QT_MOC_LITERAL(77, 1672, 25), // "slot_palette_save_inverse"
QT_MOC_LITERAL(78, 1698, 25), // "slot_palette_active_click"
QT_MOC_LITERAL(79, 1724, 21), // "slot_palette_filename"
QT_MOC_LITERAL(80, 1746, 26), // "slot_load_palette_filename"
QT_MOC_LITERAL(81, 1773, 23), // "slot_clock_start_modulo"
QT_MOC_LITERAL(82, 1797, 3), // "arg"
QT_MOC_LITERAL(83, 1801, 15), // "slot_output_bus"
QT_MOC_LITERAL(84, 1817, 22), // "slot_output_bus_enable"
QT_MOC_LITERAL(85, 1840, 14), // "slot_input_bus"
QT_MOC_LITERAL(86, 1855, 21), // "slot_input_bus_enable"
QT_MOC_LITERAL(87, 1877, 18), // "slot_bpm_precision"
QT_MOC_LITERAL(88, 1896, 5), // "index"
QT_MOC_LITERAL(89, 1902, 16), // "slot_tempo_track"
QT_MOC_LITERAL(90, 1919, 20), // "slot_tempo_track_set"
QT_MOC_LITERAL(91, 1940, 18), // "slot_buss_override"
QT_MOC_LITERAL(92, 1959, 19), // "slot_record_by_buss"
QT_MOC_LITERAL(93, 1979, 22), // "slot_record_by_channel"
QT_MOC_LITERAL(94, 2002, 18), // "slot_virtual_ports"
QT_MOC_LITERAL(95, 2021, 25), // "slot_enable_virtual_ports"
QT_MOC_LITERAL(96, 2047, 22), // "slot_virtual_out_count"
QT_MOC_LITERAL(97, 2070, 21), // "slot_virtual_in_count"
QT_MOC_LITERAL(98, 2092, 24), // "slot_metro_beats_per_bar"
QT_MOC_LITERAL(99, 2117, 21), // "slot_metro_beat_width"
QT_MOC_LITERAL(100, 2139, 21), // "slot_metro_main_patch"
QT_MOC_LITERAL(101, 2161, 20), // "slot_metro_main_note"
QT_MOC_LITERAL(102, 2182, 24), // "slot_metro_main_velocity"
QT_MOC_LITERAL(103, 2207, 24), // "slot_metro_main_fraction"
QT_MOC_LITERAL(104, 2232, 20), // "slot_metro_sub_patch"
QT_MOC_LITERAL(105, 2253, 19), // "slot_metro_sub_note"
QT_MOC_LITERAL(106, 2273, 23), // "slot_metro_sub_velocity"
QT_MOC_LITERAL(107, 2297, 23), // "slot_metro_sub_fraction"
QT_MOC_LITERAL(108, 2321, 15), // "slot_metro_buss"
QT_MOC_LITERAL(109, 2337, 18), // "slot_metro_channel"
QT_MOC_LITERAL(110, 2356, 19), // "slot_metro_count_in"
QT_MOC_LITERAL(111, 2376, 28), // "slot_metro_count_in_measures"
QT_MOC_LITERAL(112, 2405, 20), // "slot_metro_recording"
QT_MOC_LITERAL(113, 2426, 29), // "slot_metro_recording_measures"
QT_MOC_LITERAL(114, 2456, 22), // "slot_metro_record_buss"
QT_MOC_LITERAL(115, 2479, 20), // "slot_metro_thru_buss"
QT_MOC_LITERAL(116, 2500, 23), // "slot_metro_thru_channel"
QT_MOC_LITERAL(117, 2524, 17), // "slot_metro_reload"
QT_MOC_LITERAL(118, 2542, 19), // "slot_escape_pattern"
QT_MOC_LITERAL(119, 2562, 21), // "slot_convert_to_smf_1"
QT_MOC_LITERAL(120, 2584, 16), // "slot_pattern_arm"
QT_MOC_LITERAL(121, 2601, 20), // "slot_pattern_tighten"
QT_MOC_LITERAL(122, 2622, 20), // "slot_pattern_qrecord"
QT_MOC_LITERAL(123, 2643, 20), // "slot_pattern_notemap"
QT_MOC_LITERAL(124, 2664, 21), // "slot_pattern_new_only"
QT_MOC_LITERAL(125, 2686, 19), // "slot_pattern_record"
QT_MOC_LITERAL(126, 2706, 17), // "slot_pattern_thru"
QT_MOC_LITERAL(127, 2724, 23), // "slot_pattern_wraparound"
QT_MOC_LITERAL(128, 2748, 17), // "slot_record_style"
QT_MOC_LITERAL(129, 2766, 11), // "slot_jitter"
QT_MOC_LITERAL(130, 2778, 4), // "jitr"
QT_MOC_LITERAL(131, 2783, 14), // "slot_amplitude"
QT_MOC_LITERAL(132, 2798, 4), // "okay"
QT_MOC_LITERAL(133, 2803, 6), // "cancel"
QT_MOC_LITERAL(134, 2810, 5), // "apply"
QT_MOC_LITERAL(135, 2816, 5) // "reset"

    },
    "seq66::qseditoptions\0slot_sets_mode\0"
    "\0buttonno\0slot_start_mode\0slot_jack_mode\0"
    "slot_jack_connect\0slot_jack_disconnect\0"
    "slot_master_cond\0slot_time_master\0"
    "slot_transport_support\0slot_jack_midi\0"
    "slot_jack_auto_connect\0slot_io_maps\0"
    "slot_activate_io_maps\0slot_session\0"
    "slot_nsm_url\0slot_note_resume\0"
    "slot_ppqn_by_text\0text\0slot_buffer_size_by_text\0"
    "slot_use_file_ppqn\0slot_song_record_snap\0"
    "slot_add_time_sig\0slot_key_height\0"
    "slot_ui_scaling\0slot_grid_spacing\0"
    "slot_set_size_rows\0slot_set_size_columns\0"
    "slot_progress_box_width\0"
    "slot_progress_box_height\0"
    "slot_progress_box_shown\0slot_fingerprint_size\0"
    "slot_quiet_active_click\0"
    "slot_load_most_recent_click\0"
    "slot_show_full_paths_click\0"
    "slot_long_buss_names_click\0"
    "slot_pair_buss_names_click\0"
    "slot_lock_main_window_click\0"
    "slot_dark_theme_click\0slot_swap_coordinates_click\0"
    "slot_bold_grid_slots_click\0"
    "slot_gridlines_thick_click\0"
    "slot_elliptical_click\0slot_follow_progress_click\0"
    "slot_double_click_edit\0slot_global_seq_feature\0"
    "slot_rc_save_click\0slot_usr_save_click\0"
    "slot_usr_filename\0slot_load_usr_filename\0"
    "slot_mutes_save_click\0slot_mutes_active_click\0"
    "slot_mutes_filename\0slot_load_mutes_filename\0"
    "slot_playlist_save_click\0"
    "slot_playlist_active_click\0"
    "slot_playlist_filename\0"
    "slot_load_playlist_filename\0"
    "slot_ctrl_active_click\0slot_ctrl_filename\0"
    "slot_load_ctrl_filename\0slot_drums_active_click\0"
    "slot_drums_filename\0slot_load_drums_filename\0"
    "slot_stylesheet_active_click\0"
    "slot_stylesheet_filename\0"
    "slot_load_stylesheet_filename\0"
    "slot_browser_executable\0"
    "slot_load_browser_executable\0"
    "slot_pdf_executable\0slot_load_pdf_viewer_executable\0"
    "slot_patches_save_now_click\0"
    "slot_patches_active_click\0"
    "slot_patches_filename\0slot_load_patches_filename\0"
    "slot_palette_save_now_click\0"
    "slot_palette_save_inverse\0"
    "slot_palette_active_click\0"
    "slot_palette_filename\0slot_load_palette_filename\0"
    "slot_clock_start_modulo\0arg\0slot_output_bus\0"
    "slot_output_bus_enable\0slot_input_bus\0"
    "slot_input_bus_enable\0slot_bpm_precision\0"
    "index\0slot_tempo_track\0slot_tempo_track_set\0"
    "slot_buss_override\0slot_record_by_buss\0"
    "slot_record_by_channel\0slot_virtual_ports\0"
    "slot_enable_virtual_ports\0"
    "slot_virtual_out_count\0slot_virtual_in_count\0"
    "slot_metro_beats_per_bar\0slot_metro_beat_width\0"
    "slot_metro_main_patch\0slot_metro_main_note\0"
    "slot_metro_main_velocity\0"
    "slot_metro_main_fraction\0slot_metro_sub_patch\0"
    "slot_metro_sub_note\0slot_metro_sub_velocity\0"
    "slot_metro_sub_fraction\0slot_metro_buss\0"
    "slot_metro_channel\0slot_metro_count_in\0"
    "slot_metro_count_in_measures\0"
    "slot_metro_recording\0slot_metro_recording_measures\0"
    "slot_metro_record_buss\0slot_metro_thru_buss\0"
    "slot_metro_thru_channel\0slot_metro_reload\0"
    "slot_escape_pattern\0slot_convert_to_smf_1\0"
    "slot_pattern_arm\0slot_pattern_tighten\0"
    "slot_pattern_qrecord\0slot_pattern_notemap\0"
    "slot_pattern_new_only\0slot_pattern_record\0"
    "slot_pattern_thru\0slot_pattern_wraparound\0"
    "slot_record_style\0slot_jitter\0jitr\0"
    "slot_amplitude\0okay\0cancel\0apply\0reset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qseditoptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     129,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  659,    2, 0x08 /* Private */,
       4,    1,  662,    2, 0x08 /* Private */,
       5,    1,  665,    2, 0x08 /* Private */,
       6,    0,  668,    2, 0x08 /* Private */,
       7,    0,  669,    2, 0x08 /* Private */,
       8,    0,  670,    2, 0x08 /* Private */,
       9,    0,  671,    2, 0x08 /* Private */,
      10,    0,  672,    2, 0x08 /* Private */,
      11,    0,  673,    2, 0x08 /* Private */,
      12,    0,  674,    2, 0x08 /* Private */,
      13,    0,  675,    2, 0x08 /* Private */,
      14,    0,  676,    2, 0x08 /* Private */,
      15,    1,  677,    2, 0x08 /* Private */,
      16,    0,  680,    2, 0x08 /* Private */,
      17,    0,  681,    2, 0x08 /* Private */,
      18,    1,  682,    2, 0x08 /* Private */,
      20,    1,  685,    2, 0x08 /* Private */,
      21,    0,  688,    2, 0x08 /* Private */,
      22,    0,  689,    2, 0x08 /* Private */,
      23,    0,  690,    2, 0x08 /* Private */,
      24,    0,  691,    2, 0x08 /* Private */,
      25,    0,  692,    2, 0x08 /* Private */,
      26,    0,  693,    2, 0x08 /* Private */,
      27,    0,  694,    2, 0x08 /* Private */,
      28,    0,  695,    2, 0x08 /* Private */,
      29,    0,  696,    2, 0x08 /* Private */,
      30,    0,  697,    2, 0x08 /* Private */,
      31,    0,  698,    2, 0x08 /* Private */,
      32,    0,  699,    2, 0x08 /* Private */,
      33,    0,  700,    2, 0x08 /* Private */,
      34,    0,  701,    2, 0x08 /* Private */,
      35,    0,  702,    2, 0x08 /* Private */,
      36,    0,  703,    2, 0x08 /* Private */,
      37,    0,  704,    2, 0x08 /* Private */,
      38,    0,  705,    2, 0x08 /* Private */,
      39,    0,  706,    2, 0x08 /* Private */,
      40,    0,  707,    2, 0x08 /* Private */,
      41,    0,  708,    2, 0x08 /* Private */,
      42,    0,  709,    2, 0x08 /* Private */,
      43,    0,  710,    2, 0x08 /* Private */,
      44,    0,  711,    2, 0x08 /* Private */,
      45,    0,  712,    2, 0x08 /* Private */,
      46,    0,  713,    2, 0x08 /* Private */,
      47,    0,  714,    2, 0x08 /* Private */,
      48,    0,  715,    2, 0x08 /* Private */,
      49,    0,  716,    2, 0x08 /* Private */,
      50,    0,  717,    2, 0x08 /* Private */,
      51,    0,  718,    2, 0x08 /* Private */,
      52,    0,  719,    2, 0x08 /* Private */,
      53,    0,  720,    2, 0x08 /* Private */,
      54,    0,  721,    2, 0x08 /* Private */,
      55,    0,  722,    2, 0x08 /* Private */,
      56,    0,  723,    2, 0x08 /* Private */,
      57,    0,  724,    2, 0x08 /* Private */,
      58,    0,  725,    2, 0x08 /* Private */,
      59,    0,  726,    2, 0x08 /* Private */,
      60,    0,  727,    2, 0x08 /* Private */,
      61,    0,  728,    2, 0x08 /* Private */,
      62,    0,  729,    2, 0x08 /* Private */,
      63,    0,  730,    2, 0x08 /* Private */,
      64,    0,  731,    2, 0x08 /* Private */,
      65,    0,  732,    2, 0x08 /* Private */,
      66,    0,  733,    2, 0x08 /* Private */,
      67,    0,  734,    2, 0x08 /* Private */,
      68,    0,  735,    2, 0x08 /* Private */,
      69,    0,  736,    2, 0x08 /* Private */,
      70,    0,  737,    2, 0x08 /* Private */,
      71,    0,  738,    2, 0x08 /* Private */,
      72,    0,  739,    2, 0x08 /* Private */,
      73,    0,  740,    2, 0x08 /* Private */,
      74,    0,  741,    2, 0x08 /* Private */,
      75,    0,  742,    2, 0x08 /* Private */,
      76,    0,  743,    2, 0x08 /* Private */,
      77,    0,  744,    2, 0x08 /* Private */,
      78,    0,  745,    2, 0x08 /* Private */,
      79,    0,  746,    2, 0x08 /* Private */,
      80,    0,  747,    2, 0x08 /* Private */,
      81,    1,  748,    2, 0x08 /* Private */,
      83,    1,  751,    2, 0x08 /* Private */,
      84,    0,  754,    2, 0x08 /* Private */,
      85,    1,  755,    2, 0x08 /* Private */,
      86,    0,  758,    2, 0x08 /* Private */,
      87,    1,  759,    2, 0x08 /* Private */,
      89,    0,  762,    2, 0x08 /* Private */,
      90,    0,  763,    2, 0x08 /* Private */,
      91,    0,  764,    2, 0x08 /* Private */,
      92,    0,  765,    2, 0x08 /* Private */,
      93,    0,  766,    2, 0x08 /* Private */,
      94,    0,  767,    2, 0x08 /* Private */,
      95,    0,  768,    2, 0x08 /* Private */,
      96,    0,  769,    2, 0x08 /* Private */,
      97,    0,  770,    2, 0x08 /* Private */,
      98,    0,  771,    2, 0x08 /* Private */,
      99,    0,  772,    2, 0x08 /* Private */,
     100,    0,  773,    2, 0x08 /* Private */,
     101,    0,  774,    2, 0x08 /* Private */,
     102,    0,  775,    2, 0x08 /* Private */,
     103,    0,  776,    2, 0x08 /* Private */,
     104,    0,  777,    2, 0x08 /* Private */,
     105,    0,  778,    2, 0x08 /* Private */,
     106,    0,  779,    2, 0x08 /* Private */,
     107,    0,  780,    2, 0x08 /* Private */,
     108,    1,  781,    2, 0x08 /* Private */,
     109,    1,  784,    2, 0x08 /* Private */,
     110,    0,  787,    2, 0x08 /* Private */,
     111,    0,  788,    2, 0x08 /* Private */,
     112,    0,  789,    2, 0x08 /* Private */,
     113,    0,  790,    2, 0x08 /* Private */,
     114,    1,  791,    2, 0x08 /* Private */,
     115,    1,  794,    2, 0x08 /* Private */,
     116,    1,  797,    2, 0x08 /* Private */,
     117,    0,  800,    2, 0x08 /* Private */,
     118,    0,  801,    2, 0x08 /* Private */,
     119,    0,  802,    2, 0x08 /* Private */,
     120,    0,  803,    2, 0x08 /* Private */,
     121,    0,  804,    2, 0x08 /* Private */,
     122,    0,  805,    2, 0x08 /* Private */,
     123,    0,  806,    2, 0x08 /* Private */,
     124,    0,  807,    2, 0x08 /* Private */,
     125,    0,  808,    2, 0x08 /* Private */,
     126,    0,  809,    2, 0x08 /* Private */,
     127,    0,  810,    2, 0x08 /* Private */,
     128,    1,  811,    2, 0x08 /* Private */,
     129,    1,  814,    2, 0x08 /* Private */,
     131,    1,  817,    2, 0x08 /* Private */,
     132,    0,  820,    2, 0x08 /* Private */,
     133,    0,  821,    2, 0x08 /* Private */,
     134,    0,  822,    2, 0x08 /* Private */,
     135,    0,  823,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   82,
    QMetaType::Void, QMetaType::Int,   82,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   82,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   88,
    QMetaType::Void, QMetaType::Int,  130,
    QMetaType::Void, QMetaType::Int,  130,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qseditoptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qseditoptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slot_sets_mode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->slot_start_mode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slot_jack_mode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->slot_jack_connect(); break;
        case 4: _t->slot_jack_disconnect(); break;
        case 5: _t->slot_master_cond(); break;
        case 6: _t->slot_time_master(); break;
        case 7: _t->slot_transport_support(); break;
        case 8: _t->slot_jack_midi(); break;
        case 9: _t->slot_jack_auto_connect(); break;
        case 10: _t->slot_io_maps(); break;
        case 11: _t->slot_activate_io_maps(); break;
        case 12: _t->slot_session((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slot_nsm_url(); break;
        case 14: _t->slot_note_resume(); break;
        case 15: _t->slot_ppqn_by_text((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->slot_buffer_size_by_text((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->slot_use_file_ppqn(); break;
        case 18: _t->slot_song_record_snap(); break;
        case 19: _t->slot_add_time_sig(); break;
        case 20: _t->slot_key_height(); break;
        case 21: _t->slot_ui_scaling(); break;
        case 22: _t->slot_grid_spacing(); break;
        case 23: _t->slot_set_size_rows(); break;
        case 24: _t->slot_set_size_columns(); break;
        case 25: _t->slot_progress_box_width(); break;
        case 26: _t->slot_progress_box_height(); break;
        case 27: _t->slot_progress_box_shown(); break;
        case 28: _t->slot_fingerprint_size(); break;
        case 29: _t->slot_quiet_active_click(); break;
        case 30: _t->slot_load_most_recent_click(); break;
        case 31: _t->slot_show_full_paths_click(); break;
        case 32: _t->slot_long_buss_names_click(); break;
        case 33: _t->slot_pair_buss_names_click(); break;
        case 34: _t->slot_lock_main_window_click(); break;
        case 35: _t->slot_dark_theme_click(); break;
        case 36: _t->slot_swap_coordinates_click(); break;
        case 37: _t->slot_bold_grid_slots_click(); break;
        case 38: _t->slot_gridlines_thick_click(); break;
        case 39: _t->slot_elliptical_click(); break;
        case 40: _t->slot_follow_progress_click(); break;
        case 41: _t->slot_double_click_edit(); break;
        case 42: _t->slot_global_seq_feature(); break;
        case 43: _t->slot_rc_save_click(); break;
        case 44: _t->slot_usr_save_click(); break;
        case 45: _t->slot_usr_filename(); break;
        case 46: _t->slot_load_usr_filename(); break;
        case 47: _t->slot_mutes_save_click(); break;
        case 48: _t->slot_mutes_active_click(); break;
        case 49: _t->slot_mutes_filename(); break;
        case 50: _t->slot_load_mutes_filename(); break;
        case 51: _t->slot_playlist_save_click(); break;
        case 52: _t->slot_playlist_active_click(); break;
        case 53: _t->slot_playlist_filename(); break;
        case 54: _t->slot_load_playlist_filename(); break;
        case 55: _t->slot_ctrl_active_click(); break;
        case 56: _t->slot_ctrl_filename(); break;
        case 57: _t->slot_load_ctrl_filename(); break;
        case 58: _t->slot_drums_active_click(); break;
        case 59: _t->slot_drums_filename(); break;
        case 60: _t->slot_load_drums_filename(); break;
        case 61: _t->slot_stylesheet_active_click(); break;
        case 62: _t->slot_stylesheet_filename(); break;
        case 63: _t->slot_load_stylesheet_filename(); break;
        case 64: _t->slot_browser_executable(); break;
        case 65: _t->slot_load_browser_executable(); break;
        case 66: _t->slot_pdf_executable(); break;
        case 67: _t->slot_load_pdf_viewer_executable(); break;
        case 68: _t->slot_patches_save_now_click(); break;
        case 69: _t->slot_patches_active_click(); break;
        case 70: _t->slot_patches_filename(); break;
        case 71: _t->slot_load_patches_filename(); break;
        case 72: _t->slot_palette_save_now_click(); break;
        case 73: _t->slot_palette_save_inverse(); break;
        case 74: _t->slot_palette_active_click(); break;
        case 75: _t->slot_palette_filename(); break;
        case 76: _t->slot_load_palette_filename(); break;
        case 77: _t->slot_clock_start_modulo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 78: _t->slot_output_bus((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 79: _t->slot_output_bus_enable(); break;
        case 80: _t->slot_input_bus((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 81: _t->slot_input_bus_enable(); break;
        case 82: _t->slot_bpm_precision((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 83: _t->slot_tempo_track(); break;
        case 84: _t->slot_tempo_track_set(); break;
        case 85: _t->slot_buss_override(); break;
        case 86: _t->slot_record_by_buss(); break;
        case 87: _t->slot_record_by_channel(); break;
        case 88: _t->slot_virtual_ports(); break;
        case 89: _t->slot_enable_virtual_ports(); break;
        case 90: _t->slot_virtual_out_count(); break;
        case 91: _t->slot_virtual_in_count(); break;
        case 92: _t->slot_metro_beats_per_bar(); break;
        case 93: _t->slot_metro_beat_width(); break;
        case 94: _t->slot_metro_main_patch(); break;
        case 95: _t->slot_metro_main_note(); break;
        case 96: _t->slot_metro_main_velocity(); break;
        case 97: _t->slot_metro_main_fraction(); break;
        case 98: _t->slot_metro_sub_patch(); break;
        case 99: _t->slot_metro_sub_note(); break;
        case 100: _t->slot_metro_sub_velocity(); break;
        case 101: _t->slot_metro_sub_fraction(); break;
        case 102: _t->slot_metro_buss((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 103: _t->slot_metro_channel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 104: _t->slot_metro_count_in(); break;
        case 105: _t->slot_metro_count_in_measures(); break;
        case 106: _t->slot_metro_recording(); break;
        case 107: _t->slot_metro_recording_measures(); break;
        case 108: _t->slot_metro_record_buss((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 109: _t->slot_metro_thru_buss((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 110: _t->slot_metro_thru_channel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 111: _t->slot_metro_reload(); break;
        case 112: _t->slot_escape_pattern(); break;
        case 113: _t->slot_convert_to_smf_1(); break;
        case 114: _t->slot_pattern_arm(); break;
        case 115: _t->slot_pattern_tighten(); break;
        case 116: _t->slot_pattern_qrecord(); break;
        case 117: _t->slot_pattern_notemap(); break;
        case 118: _t->slot_pattern_new_only(); break;
        case 119: _t->slot_pattern_record(); break;
        case 120: _t->slot_pattern_thru(); break;
        case 121: _t->slot_pattern_wraparound(); break;
        case 122: _t->slot_record_style((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 123: _t->slot_jitter((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 124: _t->slot_amplitude((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 125: _t->okay(); break;
        case 126: _t->cancel(); break;
        case 127: _t->apply(); break;
        case 128: _t->reset(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qseditoptions::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_seq66__qseditoptions.data,
    qt_meta_data_seq66__qseditoptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qseditoptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qseditoptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qseditoptions.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int seq66::qseditoptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 129)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 129;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 129)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 129;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
