/********************************************************************************
** Form generated from reading UI file 'qpatternfix.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QPATTERNFIX_H
#define QPATTERNFIX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qpatternfix
{
public:
    QWidget *horizontalLayoutWidget;
    QHBoxLayout *horizontalLayout;
    QLabel *label_fix_main;
    QSpacerItem *horizontalSpacer;
    QLabel *label_pattern_no;
    QLabel *label_pattern;
    QLabel *label;
    QLabel *label_pulses;
    QGroupBox *group_box_length;
    QWidget *layoutWidgetLength;
    QGridLayout *gridLayout;
    QRadioButton *btn_change_pick;
    QRadioButton *btn_change_scale;
    QRadioButton *btn_change_none;
    QLineEdit *line_edit_measures;
    QLineEdit *line_edit_none;
    QLineEdit *line_edit_scale;
    QGroupBox *group_box_effect;
    QWidget *layoutWidgetEffect;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *btn_effect_alteration;
    QCheckBox *btn_effect_shift;
    QCheckBox *btn_effect_reverse;
    QCheckBox *btn_effect_shrink;
    QCheckBox *btn_effect_expand;
    QCheckBox *btn_effect_time_sig;
    QGroupBox *group_box_other;
    QWidget *layoutWidgetOtherFixes;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *btn_align_left;
    QCheckBox *btn_align_right;
    QCheckBox *btn_reverse;
    QCheckBox *btn_reverse_in_place;
    QCheckBox *btn_save_note_length;
    QWidget *layoutWidgetAlignLeft;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *btn_set;
    QPushButton *btn_reset;
    QPushButton *btn_close;
    QGroupBox *group_box_quantize;
    QHBoxLayout *horizontalLayout_3;
    QGridLayout *gridLayout_2;
    QRadioButton *btn_alt_rev_notemap;
    QRadioButton *btn_alt_none;
    QLineEdit *line_edit_alt_tighten;
    QRadioButton *btn_alt_tighten;
    QRadioButton *btn_alt_full;
    QPushButton *btn_notemap_file;
    QRadioButton *btn_alt_jitter;
    QRadioButton *btn_alt_notemap;
    QRadioButton *btn_alt_random;
    QLineEdit *line_edit_alt_none;
    QLineEdit *line_edit_alt_jitter;
    QLineEdit *line_edit_alt_full;
    QLineEdit *line_edit_alt_random;
    QRadioButton *btn_alt_random_pitch;
    QLineEdit *line_edit_alt_random_pitch;

    void setupUi(QFrame *qpatternfix)
    {
        if (qpatternfix->objectName().isEmpty())
            qpatternfix->setObjectName(QString::fromUtf8("qpatternfix"));
        qpatternfix->resize(688, 476);
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qpatternfix->sizePolicy().hasHeightForWidth());
        qpatternfix->setSizePolicy(sizePolicy);
        horizontalLayoutWidget = new QWidget(qpatternfix);
        horizontalLayoutWidget->setObjectName(QString::fromUtf8("horizontalLayoutWidget"));
        horizontalLayoutWidget->setGeometry(QRect(12, 8, 631, 38));
        horizontalLayout = new QHBoxLayout(horizontalLayoutWidget);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        label_fix_main = new QLabel(horizontalLayoutWidget);
        label_fix_main->setObjectName(QString::fromUtf8("label_fix_main"));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label_fix_main->setFont(font);

        horizontalLayout->addWidget(label_fix_main);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        label_pattern_no = new QLabel(horizontalLayoutWidget);
        label_pattern_no->setObjectName(QString::fromUtf8("label_pattern_no"));
        label_pattern_no->setMinimumSize(QSize(100, 0));
        QFont font1;
        font1.setItalic(true);
        label_pattern_no->setFont(font1);
        label_pattern_no->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout->addWidget(label_pattern_no);

        label_pattern = new QLabel(horizontalLayoutWidget);
        label_pattern->setObjectName(QString::fromUtf8("label_pattern"));
        label_pattern->setEnabled(true);
        label_pattern->setMinimumSize(QSize(56, 0));
        label_pattern->setMaximumSize(QSize(56, 28));
        label_pattern->setFont(font1);
        label_pattern->setFrameShape(QFrame::Shape::Box);
        label_pattern->setFrameShadow(QFrame::Shadow::Sunken);
        label_pattern->setLineWidth(2);
        label_pattern->setMidLineWidth(0);

        horizontalLayout->addWidget(label_pattern);

        label = new QLabel(horizontalLayoutWidget);
        label->setObjectName(QString::fromUtf8("label"));
        label->setMinimumSize(QSize(108, 0));
        label->setMaximumSize(QSize(64, 16777215));
        label->setFont(font1);
        label->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout->addWidget(label);

        label_pulses = new QLabel(horizontalLayoutWidget);
        label_pulses->setObjectName(QString::fromUtf8("label_pulses"));
        label_pulses->setMinimumSize(QSize(96, 0));
        label_pulses->setMaximumSize(QSize(64, 28));
        label_pulses->setFont(font1);
        label_pulses->setFrameShape(QFrame::Shape::Box);
        label_pulses->setFrameShadow(QFrame::Shadow::Sunken);
        label_pulses->setLineWidth(2);

        horizontalLayout->addWidget(label_pulses);

        group_box_length = new QGroupBox(qpatternfix);
        group_box_length->setObjectName(QString::fromUtf8("group_box_length"));
        group_box_length->setGeometry(QRect(8, 56, 261, 151));
        QFont font2;
        font2.setBold(true);
        group_box_length->setFont(font2);
        group_box_length->setFlat(false);
        layoutWidgetLength = new QWidget(group_box_length);
        layoutWidgetLength->setObjectName(QString::fromUtf8("layoutWidgetLength"));
        layoutWidgetLength->setGeometry(QRect(9, 33, 259, 104));
        gridLayout = new QGridLayout(layoutWidgetLength);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(22);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        btn_change_pick = new QRadioButton(layoutWidgetLength);
        btn_change_pick->setObjectName(QString::fromUtf8("btn_change_pick"));
        QFont font3;
        font3.setBold(false);
        btn_change_pick->setFont(font3);

        gridLayout->addWidget(btn_change_pick, 1, 0, 1, 1);

        btn_change_scale = new QRadioButton(layoutWidgetLength);
        btn_change_scale->setObjectName(QString::fromUtf8("btn_change_scale"));
        btn_change_scale->setFont(font3);

        gridLayout->addWidget(btn_change_scale, 2, 0, 1, 1);

        btn_change_none = new QRadioButton(layoutWidgetLength);
        btn_change_none->setObjectName(QString::fromUtf8("btn_change_none"));
        btn_change_none->setFont(font3);

        gridLayout->addWidget(btn_change_none, 0, 0, 1, 1);

        line_edit_measures = new QLineEdit(layoutWidgetLength);
        line_edit_measures->setObjectName(QString::fromUtf8("line_edit_measures"));

        gridLayout->addWidget(line_edit_measures, 1, 1, 1, 1);

        line_edit_none = new QLineEdit(layoutWidgetLength);
        line_edit_none->setObjectName(QString::fromUtf8("line_edit_none"));
        line_edit_none->setEnabled(false);

        gridLayout->addWidget(line_edit_none, 0, 1, 1, 1);

        line_edit_scale = new QLineEdit(layoutWidgetLength);
        line_edit_scale->setObjectName(QString::fromUtf8("line_edit_scale"));

        gridLayout->addWidget(line_edit_scale, 2, 1, 1, 1);

        group_box_effect = new QGroupBox(qpatternfix);
        group_box_effect->setObjectName(QString::fromUtf8("group_box_effect"));
        group_box_effect->setEnabled(false);
        group_box_effect->setGeometry(QRect(530, 56, 141, 221));
        QFont font4;
        font4.setBold(true);
        font4.setItalic(true);
        group_box_effect->setFont(font4);
        group_box_effect->setFlat(false);
        group_box_effect->setCheckable(false);
        layoutWidgetEffect = new QWidget(group_box_effect);
        layoutWidgetEffect->setObjectName(QString::fromUtf8("layoutWidgetEffect"));
        layoutWidgetEffect->setGeometry(QRect(8, 35, 121, 181));
        verticalLayout_2 = new QVBoxLayout(layoutWidgetEffect);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        btn_effect_alteration = new QCheckBox(layoutWidgetEffect);
        btn_effect_alteration->setObjectName(QString::fromUtf8("btn_effect_alteration"));
        QFont font5;
        font5.setBold(false);
        font5.setItalic(true);
        btn_effect_alteration->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_alteration);

        btn_effect_shift = new QCheckBox(layoutWidgetEffect);
        btn_effect_shift->setObjectName(QString::fromUtf8("btn_effect_shift"));
        btn_effect_shift->setEnabled(false);
        btn_effect_shift->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_shift);

        btn_effect_reverse = new QCheckBox(layoutWidgetEffect);
        btn_effect_reverse->setObjectName(QString::fromUtf8("btn_effect_reverse"));
        btn_effect_reverse->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_reverse);

        btn_effect_shrink = new QCheckBox(layoutWidgetEffect);
        btn_effect_shrink->setObjectName(QString::fromUtf8("btn_effect_shrink"));
        btn_effect_shrink->setEnabled(false);
        btn_effect_shrink->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_shrink);

        btn_effect_expand = new QCheckBox(layoutWidgetEffect);
        btn_effect_expand->setObjectName(QString::fromUtf8("btn_effect_expand"));
        btn_effect_expand->setEnabled(false);
        btn_effect_expand->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_expand);

        btn_effect_time_sig = new QCheckBox(layoutWidgetEffect);
        btn_effect_time_sig->setObjectName(QString::fromUtf8("btn_effect_time_sig"));
        btn_effect_time_sig->setFont(font5);

        verticalLayout_2->addWidget(btn_effect_time_sig);

        group_box_other = new QGroupBox(qpatternfix);
        group_box_other->setObjectName(QString::fromUtf8("group_box_other"));
        group_box_other->setGeometry(QRect(8, 220, 221, 201));
        group_box_other->setFont(font2);
        layoutWidgetOtherFixes = new QWidget(group_box_other);
        layoutWidgetOtherFixes->setObjectName(QString::fromUtf8("layoutWidgetOtherFixes"));
        layoutWidgetOtherFixes->setGeometry(QRect(12, 24, 201, 161));
        verticalLayout_3 = new QVBoxLayout(layoutWidgetOtherFixes);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        btn_align_left = new QCheckBox(layoutWidgetOtherFixes);
        btn_align_left->setObjectName(QString::fromUtf8("btn_align_left"));
        btn_align_left->setFont(font3);

        verticalLayout_3->addWidget(btn_align_left);

        btn_align_right = new QCheckBox(layoutWidgetOtherFixes);
        btn_align_right->setObjectName(QString::fromUtf8("btn_align_right"));
        btn_align_right->setFont(font3);

        verticalLayout_3->addWidget(btn_align_right);

        btn_reverse = new QCheckBox(layoutWidgetOtherFixes);
        btn_reverse->setObjectName(QString::fromUtf8("btn_reverse"));
        btn_reverse->setFont(font3);

        verticalLayout_3->addWidget(btn_reverse);

        btn_reverse_in_place = new QCheckBox(layoutWidgetOtherFixes);
        btn_reverse_in_place->setObjectName(QString::fromUtf8("btn_reverse_in_place"));
        btn_reverse_in_place->setFont(font3);

        verticalLayout_3->addWidget(btn_reverse_in_place);

        btn_save_note_length = new QCheckBox(layoutWidgetOtherFixes);
        btn_save_note_length->setObjectName(QString::fromUtf8("btn_save_note_length"));
        btn_save_note_length->setFont(font3);

        verticalLayout_3->addWidget(btn_save_note_length);

        layoutWidgetAlignLeft = new QWidget(qpatternfix);
        layoutWidgetAlignLeft->setObjectName(QString::fromUtf8("layoutWidgetAlignLeft"));
        layoutWidgetAlignLeft->setGeometry(QRect(250, 380, 381, 38));
        horizontalLayout_2 = new QHBoxLayout(layoutWidgetAlignLeft);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        btn_set = new QPushButton(layoutWidgetAlignLeft);
        btn_set->setObjectName(QString::fromUtf8("btn_set"));
        btn_set->setMinimumSize(QSize(0, 28));

        horizontalLayout_2->addWidget(btn_set);

        btn_reset = new QPushButton(layoutWidgetAlignLeft);
        btn_reset->setObjectName(QString::fromUtf8("btn_reset"));
        btn_reset->setMinimumSize(QSize(0, 28));

        horizontalLayout_2->addWidget(btn_reset);

        btn_close = new QPushButton(layoutWidgetAlignLeft);
        btn_close->setObjectName(QString::fromUtf8("btn_close"));
        btn_close->setMinimumSize(QSize(28, 0));

        horizontalLayout_2->addWidget(btn_close);

        group_box_quantize = new QGroupBox(qpatternfix);
        group_box_quantize->setObjectName(QString::fromUtf8("group_box_quantize"));
        group_box_quantize->setGeometry(QRect(270, 56, 258, 321));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(group_box_quantize->sizePolicy().hasHeightForWidth());
        group_box_quantize->setSizePolicy(sizePolicy1);
        group_box_quantize->setFont(font2);
        horizontalLayout_3 = new QHBoxLayout(group_box_quantize);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        btn_alt_rev_notemap = new QRadioButton(group_box_quantize);
        btn_alt_rev_notemap->setObjectName(QString::fromUtf8("btn_alt_rev_notemap"));
        btn_alt_rev_notemap->setFont(font3);

        gridLayout_2->addWidget(btn_alt_rev_notemap, 7, 0, 1, 1);

        btn_alt_none = new QRadioButton(group_box_quantize);
        btn_alt_none->setObjectName(QString::fromUtf8("btn_alt_none"));
        btn_alt_none->setFont(font3);

        gridLayout_2->addWidget(btn_alt_none, 0, 0, 1, 1);

        line_edit_alt_tighten = new QLineEdit(group_box_quantize);
        line_edit_alt_tighten->setObjectName(QString::fromUtf8("line_edit_alt_tighten"));
        line_edit_alt_tighten->setEnabled(true);
        line_edit_alt_tighten->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_tighten, 1, 1, 1, 1);

        btn_alt_tighten = new QRadioButton(group_box_quantize);
        btn_alt_tighten->setObjectName(QString::fromUtf8("btn_alt_tighten"));
        btn_alt_tighten->setFont(font3);

        gridLayout_2->addWidget(btn_alt_tighten, 1, 0, 1, 1);

        btn_alt_full = new QRadioButton(group_box_quantize);
        btn_alt_full->setObjectName(QString::fromUtf8("btn_alt_full"));
        btn_alt_full->setFont(font3);

        gridLayout_2->addWidget(btn_alt_full, 2, 0, 1, 1);

        btn_notemap_file = new QPushButton(group_box_quantize);
        btn_notemap_file->setObjectName(QString::fromUtf8("btn_notemap_file"));
        btn_notemap_file->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(btn_notemap_file, 6, 1, 1, 1);

        btn_alt_jitter = new QRadioButton(group_box_quantize);
        btn_alt_jitter->setObjectName(QString::fromUtf8("btn_alt_jitter"));
        btn_alt_jitter->setFont(font3);

        gridLayout_2->addWidget(btn_alt_jitter, 5, 0, 1, 1);

        btn_alt_notemap = new QRadioButton(group_box_quantize);
        btn_alt_notemap->setObjectName(QString::fromUtf8("btn_alt_notemap"));
        btn_alt_notemap->setFont(font3);

        gridLayout_2->addWidget(btn_alt_notemap, 6, 0, 1, 1);

        btn_alt_random = new QRadioButton(group_box_quantize);
        btn_alt_random->setObjectName(QString::fromUtf8("btn_alt_random"));
        btn_alt_random->setFont(font3);

        gridLayout_2->addWidget(btn_alt_random, 3, 0, 1, 1);

        line_edit_alt_none = new QLineEdit(group_box_quantize);
        line_edit_alt_none->setObjectName(QString::fromUtf8("line_edit_alt_none"));
        line_edit_alt_none->setEnabled(false);
        line_edit_alt_none->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_none, 0, 1, 1, 1);

        line_edit_alt_jitter = new QLineEdit(group_box_quantize);
        line_edit_alt_jitter->setObjectName(QString::fromUtf8("line_edit_alt_jitter"));
        line_edit_alt_jitter->setEnabled(true);
        line_edit_alt_jitter->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_jitter, 5, 1, 1, 1);

        line_edit_alt_full = new QLineEdit(group_box_quantize);
        line_edit_alt_full->setObjectName(QString::fromUtf8("line_edit_alt_full"));
        line_edit_alt_full->setEnabled(true);
        line_edit_alt_full->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_full, 2, 1, 1, 1);

        line_edit_alt_random = new QLineEdit(group_box_quantize);
        line_edit_alt_random->setObjectName(QString::fromUtf8("line_edit_alt_random"));
        line_edit_alt_random->setEnabled(true);
        line_edit_alt_random->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_random, 3, 1, 1, 1);

        btn_alt_random_pitch = new QRadioButton(group_box_quantize);
        btn_alt_random_pitch->setObjectName(QString::fromUtf8("btn_alt_random_pitch"));
        btn_alt_random_pitch->setFont(font3);

        gridLayout_2->addWidget(btn_alt_random_pitch, 4, 0, 1, 1);

        line_edit_alt_random_pitch = new QLineEdit(group_box_quantize);
        line_edit_alt_random_pitch->setObjectName(QString::fromUtf8("line_edit_alt_random_pitch"));
        line_edit_alt_random_pitch->setMaximumSize(QSize(48, 16777215));

        gridLayout_2->addWidget(line_edit_alt_random_pitch, 4, 1, 1, 1);


        horizontalLayout_3->addLayout(gridLayout_2);

        QWidget::setTabOrder(btn_change_none, line_edit_none);
        QWidget::setTabOrder(line_edit_none, btn_change_pick);
        QWidget::setTabOrder(btn_change_pick, line_edit_measures);
        QWidget::setTabOrder(line_edit_measures, btn_change_scale);
        QWidget::setTabOrder(btn_change_scale, line_edit_scale);
        QWidget::setTabOrder(line_edit_scale, btn_align_left);
        QWidget::setTabOrder(btn_align_left, btn_set);
        QWidget::setTabOrder(btn_set, btn_reset);
        QWidget::setTabOrder(btn_reset, btn_close);
        QWidget::setTabOrder(btn_close, btn_effect_expand);
        QWidget::setTabOrder(btn_effect_expand, btn_effect_shrink);
        QWidget::setTabOrder(btn_effect_shrink, btn_effect_shift);

        retranslateUi(qpatternfix);

        QMetaObject::connectSlotsByName(qpatternfix);
    } // setupUi

    void retranslateUi(QFrame *qpatternfix)
    {
        qpatternfix->setWindowTitle(QCoreApplication::translate("qpatternfix", "Frame", nullptr));
        label_fix_main->setText(QCoreApplication::translate("qpatternfix", "Pattern-Fix Editor", nullptr));
        label_pattern_no->setText(QCoreApplication::translate("qpatternfix", "Pattern No. ", nullptr));
        label_pattern->setText(QString());
        label->setText(QCoreApplication::translate("qpatternfix", "Max. Tick", nullptr));
        label_pulses->setText(QString());
        group_box_length->setTitle(QCoreApplication::translate("qpatternfix", "Length Change", nullptr));
#if QT_CONFIG(tooltip)
        btn_change_pick->setToolTip(QCoreApplication::translate("qpatternfix", "If an integer, the number of measures is\n"
"changed; events are compressed or expanded\n"
"as needed.\n"
"\n"
"If a fraction (e.g. 3/4), the pattern is reduced to\n"
"one measure with the given time signature.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_change_pick->setText(QCoreApplication::translate("qpatternfix", "&Measures/Time Sig", nullptr));
#if QT_CONFIG(tooltip)
        btn_change_scale->setToolTip(QCoreApplication::translate("qpatternfix", "Shrink/expand the pattern by a factor,\n"
"accounting for the time of the last event.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_change_scale->setText(QCoreApplication::translate("qpatternfix", "Scale &Factor", nullptr));
#if QT_CONFIG(tooltip)
        btn_change_none->setToolTip(QCoreApplication::translate("qpatternfix", "No change in pattern length.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_change_none->setText(QCoreApplication::translate("qpatternfix", "&None", nullptr));
        line_edit_measures->setText(QString());
        group_box_effect->setTitle(QCoreApplication::translate("qpatternfix", "Applied Effects", nullptr));
        btn_effect_alteration->setText(QCoreApplication::translate("qpatternfix", "Alteration", nullptr));
#if QT_CONFIG(tooltip)
        btn_effect_shift->setToolTip(QCoreApplication::translate("qpatternfix", "Events will be shifted.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_effect_shift->setText(QCoreApplication::translate("qpatternfix", "Shift", nullptr));
#if QT_CONFIG(tooltip)
        btn_effect_reverse->setToolTip(QCoreApplication::translate("qpatternfix", "Events will be reversed.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_effect_reverse->setText(QCoreApplication::translate("qpatternfix", "Reverse", nullptr));
#if QT_CONFIG(tooltip)
        btn_effect_shrink->setToolTip(QCoreApplication::translate("qpatternfix", "Events will be compressed.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_effect_shrink->setText(QCoreApplication::translate("qpatternfix", "Shrin&k", nullptr));
#if QT_CONFIG(tooltip)
        btn_effect_expand->setToolTip(QCoreApplication::translate("qpatternfix", "Events will expand.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_effect_expand->setText(QCoreApplication::translate("qpatternfix", "E&xpand", nullptr));
#if QT_CONFIG(tooltip)
        btn_effect_time_sig->setToolTip(QCoreApplication::translate("qpatternfix", "The time signature will change.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_effect_time_sig->setText(QCoreApplication::translate("qpatternfix", "Time Sig", nullptr));
        group_box_other->setTitle(QCoreApplication::translate("qpatternfix", "Other Fixes", nullptr));
#if QT_CONFIG(tooltip)
        btn_align_left->setToolTip(QCoreApplication::translate("qpatternfix", "Move events left to remove delays.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_align_left->setText(QCoreApplication::translate("qpatternfix", "&Align left", nullptr));
        btn_align_right->setText(QCoreApplication::translate("qpatternfix", "Align right", nullptr));
#if QT_CONFIG(tooltip)
        btn_reverse->setToolTip(QCoreApplication::translate("qpatternfix", "Reverse the order of all events, flipping\n"
"the measures.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_reverse->setText(QCoreApplication::translate("qpatternfix", "Reverse measures", nullptr));
#if QT_CONFIG(tooltip)
        btn_reverse_in_place->setToolTip(QCoreApplication::translate("qpatternfix", "Reverse notes, preserving relative location.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_reverse_in_place->setText(QCoreApplication::translate("qpatternfix", "Reverse in place", nullptr));
#if QT_CONFIG(tooltip)
        btn_save_note_length->setToolTip(QCoreApplication::translate("qpatternfix", "When scaling, don't change note length.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_save_note_length->setText(QCoreApplication::translate("qpatternfix", "&Preserve note length", nullptr));
#if QT_CONFIG(tooltip)
        btn_set->setToolTip(QCoreApplication::translate("qpatternfix", "Apply the settings.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_set->setText(QCoreApplication::translate("qpatternfix", "&Set", nullptr));
#if QT_CONFIG(tooltip)
        btn_reset->setToolTip(QCoreApplication::translate("qpatternfix", "Go back to the original pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_reset->setText(QCoreApplication::translate("qpatternfix", "&Reset", nullptr));
#if QT_CONFIG(tooltip)
        btn_close->setToolTip(QCoreApplication::translate("qpatternfix", "Close this dialog.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_close->setText(QCoreApplication::translate("qpatternfix", "&Close", nullptr));
        group_box_quantize->setTitle(QCoreApplication::translate("qpatternfix", "Alteration", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_rev_notemap->setToolTip(QCoreApplication::translate("qpatternfix", "Remap notes in the opposite direction of \"Note-map\".", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_rev_notemap->setText(QCoreApplication::translate("qpatternfix", "Rev. Note-map", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_none->setToolTip(QCoreApplication::translate("qpatternfix", "No alteration operations will be done.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_none->setText(QCoreApplication::translate("qpatternfix", "None", nullptr));
#if QT_CONFIG(tooltip)
        line_edit_alt_tighten->setToolTip(QCoreApplication::translate("qpatternfix", "If not blank, replaces the snap/2 value.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btn_alt_tighten->setToolTip(QCoreApplication::translate("qpatternfix", "Do partial quantization.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_tighten->setText(QCoreApplication::translate("qpatternfix", "&Tight Q ticks", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_full->setToolTip(QCoreApplication::translate("qpatternfix", "Fully quantization to the snap value.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_full->setText(QCoreApplication::translate("qpatternfix", "F&ull Q ticks", nullptr));
#if QT_CONFIG(tooltip)
        btn_notemap_file->setToolTip(QCoreApplication::translate("qpatternfix", "Open the file dialog to select a .notemap/.drums file.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_notemap_file->setText(QCoreApplication::translate("qpatternfix", "...", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_jitter->setToolTip(QCoreApplication::translate("qpatternfix", "Jitter the timestamps of the events.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_jitter->setText(QCoreApplication::translate("qpatternfix", "Time/tick jitter", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_notemap->setToolTip(QCoreApplication::translate("qpatternfix", "Remap notes as per the loaded 'drum' file.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_notemap->setText(QCoreApplication::translate("qpatternfix", "Note-map", nullptr));
#if QT_CONFIG(tooltip)
        btn_alt_random->setToolTip(QCoreApplication::translate("qpatternfix", "Randomize event magnitude or note velocity.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_random->setText(QCoreApplication::translate("qpatternfix", "Random amplitude", nullptr));
#if QT_CONFIG(tooltip)
        line_edit_alt_jitter->setToolTip(QCoreApplication::translate("qpatternfix", "Plus/minus range of time jitter in ticks.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        line_edit_alt_full->setToolTip(QCoreApplication::translate("qpatternfix", "If not blank, replaces the snap value.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        line_edit_alt_random->setToolTip(QCoreApplication::translate("qpatternfix", "Plus/minus range of magnitude change in MIDI units.", nullptr));
#endif // QT_CONFIG(tooltip)
        btn_alt_random_pitch->setText(QCoreApplication::translate("qpatternfix", "Random pitch", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qpatternfix: public Ui_qpatternfix {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QPATTERNFIX_H
