/********************************************************************************
** Form generated from reading UI file 'qslivegrid.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSLIVEGRID_H
#define QSLIVEGRID_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_qslivegrid
{
public:
    QVBoxLayout *verticalLayout;
    QFrame *frame;
    QGridLayout *gridLayout;
    QGridLayout *loopGridLayout;
    QHBoxLayout *horizontalLayout_6;
    QLabel *setNameLabel;
    QLineEdit *txtBankName;
    QLabel *labelPlaylistSong;
    QLabel *labelArmMode;
    QPushButton *buttonBackgroundRecord;
    QPushButton *buttonMetronome;
    QComboBox *comboGridMode;
    QPushButton *buttonActivate;
    QPushButton *buttonLoopMode;
    QPushButton *buttonRecordMode;

    void setupUi(QFrame *qslivegrid)
    {
        if (qslivegrid->objectName().isEmpty())
            qslivegrid->setObjectName(QString::fromUtf8("qslivegrid"));
        qslivegrid->resize(691, 366);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qslivegrid->sizePolicy().hasHeightForWidth());
        qslivegrid->setSizePolicy(sizePolicy);
        qslivegrid->setMinimumSize(QSize(330, 100));
        qslivegrid->setFrameShape(QFrame::Shape::NoFrame);
        qslivegrid->setFrameShadow(QFrame::Shadow::Plain);
        verticalLayout = new QVBoxLayout(qslivegrid);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(qslivegrid);
        frame->setObjectName(QString::fromUtf8("frame"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy1);
        frame->setMinimumSize(QSize(660, 180));
        frame->setFrameShape(QFrame::Shape::NoFrame);
        frame->setFrameShadow(QFrame::Shadow::Plain);
        frame->setLineWidth(0);
        gridLayout = new QGridLayout(frame);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setVerticalSpacing(12);
        loopGridLayout = new QGridLayout();
        loopGridLayout->setObjectName(QString::fromUtf8("loopGridLayout"));

        gridLayout->addLayout(loopGridLayout, 0, 0, 1, 1);


        verticalLayout->addWidget(frame);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        horizontalLayout_6->setContentsMargins(10, 6, 10, 0);
        setNameLabel = new QLabel(qslivegrid);
        setNameLabel->setObjectName(QString::fromUtf8("setNameLabel"));
        setNameLabel->setEnabled(true);
        setNameLabel->setMinimumSize(QSize(38, 28));
        setNameLabel->setMaximumSize(QSize(324, 28));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        setNameLabel->setFont(font);
        setNameLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout_6->addWidget(setNameLabel);

        txtBankName = new QLineEdit(qslivegrid);
        txtBankName->setObjectName(QString::fromUtf8("txtBankName"));
        txtBankName->setMinimumSize(QSize(184, 28));
        txtBankName->setMaximumSize(QSize(999, 28));

        horizontalLayout_6->addWidget(txtBankName);

        labelPlaylistSong = new QLabel(qslivegrid);
        labelPlaylistSong->setObjectName(QString::fromUtf8("labelPlaylistSong"));
        labelPlaylistSong->setMinimumSize(QSize(400, 28));
        labelPlaylistSong->setMaximumSize(QSize(800, 28));
        QFont font1;
        font1.setPointSize(11);
        labelPlaylistSong->setFont(font1);
        labelPlaylistSong->setFrameShape(QFrame::Shape::StyledPanel);
        labelPlaylistSong->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout_6->addWidget(labelPlaylistSong);

        labelArmMode = new QLabel(qslivegrid);
        labelArmMode->setObjectName(QString::fromUtf8("labelArmMode"));
        labelArmMode->setMinimumSize(QSize(56, 0));
        labelArmMode->setMaximumSize(QSize(64, 16777215));
        QFont font2;
        font2.setPointSize(11);
        font2.setItalic(true);
        labelArmMode->setFont(font2);

        horizontalLayout_6->addWidget(labelArmMode);

        buttonBackgroundRecord = new QPushButton(qslivegrid);
        buttonBackgroundRecord->setObjectName(QString::fromUtf8("buttonBackgroundRecord"));
        buttonBackgroundRecord->setMinimumSize(QSize(36, 28));
        buttonBackgroundRecord->setMaximumSize(QSize(164, 28));
        buttonBackgroundRecord->setFont(font1);
        buttonBackgroundRecord->setIconSize(QSize(28, 16));
        buttonBackgroundRecord->setCheckable(true);

        horizontalLayout_6->addWidget(buttonBackgroundRecord);

        buttonMetronome = new QPushButton(qslivegrid);
        buttonMetronome->setObjectName(QString::fromUtf8("buttonMetronome"));
        buttonMetronome->setMinimumSize(QSize(36, 28));
        buttonMetronome->setMaximumSize(QSize(164, 28));
        buttonMetronome->setFont(font1);
        buttonMetronome->setIconSize(QSize(28, 20));
        buttonMetronome->setCheckable(true);

        horizontalLayout_6->addWidget(buttonMetronome);

        comboGridMode = new QComboBox(qslivegrid);
        comboGridMode->setObjectName(QString::fromUtf8("comboGridMode"));
        comboGridMode->setMinimumSize(QSize(100, 26));
        comboGridMode->setMaximumSize(QSize(164, 28));
        comboGridMode->setFont(font1);

        horizontalLayout_6->addWidget(comboGridMode);

        buttonActivate = new QPushButton(qslivegrid);
        buttonActivate->setObjectName(QString::fromUtf8("buttonActivate"));
        buttonActivate->setMinimumSize(QSize(80, 28));
        buttonActivate->setMaximumSize(QSize(164, 28));
        buttonActivate->setFont(font1);

        horizontalLayout_6->addWidget(buttonActivate);

        buttonLoopMode = new QPushButton(qslivegrid);
        buttonLoopMode->setObjectName(QString::fromUtf8("buttonLoopMode"));
        buttonLoopMode->setMinimumSize(QSize(80, 28));
        buttonLoopMode->setMaximumSize(QSize(164, 28));
        buttonLoopMode->setFont(font1);

        horizontalLayout_6->addWidget(buttonLoopMode);

        buttonRecordMode = new QPushButton(qslivegrid);
        buttonRecordMode->setObjectName(QString::fromUtf8("buttonRecordMode"));
        buttonRecordMode->setMinimumSize(QSize(80, 28));
        buttonRecordMode->setMaximumSize(QSize(164, 28));
        buttonRecordMode->setFont(font1);

        horizontalLayout_6->addWidget(buttonRecordMode);


        verticalLayout->addLayout(horizontalLayout_6);

        QWidget::setTabOrder(txtBankName, comboGridMode);
        QWidget::setTabOrder(comboGridMode, buttonActivate);
        QWidget::setTabOrder(buttonActivate, buttonLoopMode);
        QWidget::setTabOrder(buttonLoopMode, buttonRecordMode);

        retranslateUi(qslivegrid);

        QMetaObject::connectSlotsByName(qslivegrid);
    } // setupUi

    void retranslateUi(QFrame *qslivegrid)
    {
        qslivegrid->setWindowTitle(QCoreApplication::translate("qslivegrid", "Frame", nullptr));
        setNameLabel->setText(QCoreApplication::translate("qslivegrid", "Set  ", nullptr));
#if QT_CONFIG(tooltip)
        txtBankName->setToolTip(QCoreApplication::translate("qslivegrid", "Edit the Set/Bank name here.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelPlaylistSong->setText(QString());
        labelArmMode->setText(QCoreApplication::translate("qslivegrid", "Normal", nullptr));
#if QT_CONFIG(tooltip)
        buttonBackgroundRecord->setToolTip(QCoreApplication::translate("qslivegrid", "Activates/deactivates background recording.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonBackgroundRecord->setText(QCoreApplication::translate("qslivegrid", "BG", nullptr));
#if QT_CONFIG(tooltip)
        buttonMetronome->setToolTip(QCoreApplication::translate("qslivegrid", "Loads and activates/deactivates the metronome.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonMetronome->setText(QCoreApplication::translate("qslivegrid", "M", nullptr));
#if QT_CONFIG(tooltip)
        comboGridMode->setToolTip(QCoreApplication::translate("qslivegrid", "Determines the function of each slot, normally just loop control.", nullptr));
#endif // QT_CONFIG(tooltip)
        comboGridMode->setCurrentText(QString());
#if QT_CONFIG(tooltip)
        buttonActivate->setToolTip(QCoreApplication::translate("qslivegrid", "Makes this the active set.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonActivate->setText(QCoreApplication::translate("qslivegrid", "Activate", nullptr));
#if QT_CONFIG(tooltip)
        buttonLoopMode->setToolTip(QCoreApplication::translate("qslivegrid", "Cycles through recording styles.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonLoopMode->setText(QCoreApplication::translate("qslivegrid", "LMode", nullptr));
#if QT_CONFIG(tooltip)
        buttonRecordMode->setToolTip(QCoreApplication::translate("qslivegrid", "Cycles through quantized recording modes.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRecordMode->setText(QCoreApplication::translate("qslivegrid", "RMode", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qslivegrid: public Ui_qslivegrid {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSLIVEGRID_H
