/********************************************************************************
** Form generated from reading UI file 'qperfeditex.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QPERFEDITEX_H
#define QPERFEDITEX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qperfeditex
{
public:

    void setupUi(QWidget *qperfeditex)
    {
        if (qperfeditex->objectName().isEmpty())
            qperfeditex->setObjectName(QString::fromUtf8("qperfeditex"));
        qperfeditex->resize(861, 572);

        retranslateUi(qperfeditex);

        QMetaObject::connectSlotsByName(qperfeditex);
    } // setupUi

    void retranslateUi(QWidget *qperfeditex)
    {
        qperfeditex->setWindowTitle(QCoreApplication::translate("qperfeditex", "Song Editor", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qperfeditex: public Ui_qperfeditex {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QPERFEDITEX_H
