/********************************************************************************
** Form generated from reading UI file 'qplaylistframe.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QPLAYLISTFRAME_H
#define QPLAYLISTFRAME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qplaylistframe
{
public:
    QWidget *fileLayoutWidget;
    QHBoxLayout *horizPlaylistFileLayout;
    QLabel *label_playlist_file;
    QLineEdit *entry_playlist_file;
    QWidget *selectionLayoutWidget;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QPushButton *buttonSelectSongDir;
    QLabel *labelCurrentPlaylist;
    QLabel *labelSongReserved;
    QLineEdit *editPlaylistPath;
    QLabel *labelPlaylistDirectory;
    QSpinBox *spinPlaylistNumber;
    QLineEdit *editPlaylistNumber;
    QLabel *labelCurrentSong;
    QLabel *labelListSong;
    QLabel *labelListSection;
    QLineEdit *editSongNumber;
    QLineEdit *editPlaylistName;
    QLineEdit *editSongPath;
    QLineEdit *editSongFilename;
    QLabel *labelSongDirectory;
    QLabel *labelDirEmbedded;
    QSpinBox *spinSongNumber;
    QPushButton *buttonSelectPlaylist;
    QPushButton *buttonSelectSong;
    QPushButton *buttonSongReserved_1;
    QPushButton *buttonPlaylistReserved;
    QPushButton *buttonSongreserved_2;
    QHBoxLayout *horizontalLayout;
    QTableWidget *tablePlaylistSections;
    QVBoxLayout *verticalLayout;
    QPushButton *buttonPlaylistCreate;
    QPushButton *buttonPlaylistAdd;
    QPushButton *buttonPlaylistModify;
    QPushButton *buttonPlaylistRemove;
    QPushButton *buttonPlaylistSave;
    QTableWidget *tablePlaylistSongs;
    QVBoxLayout *verticalLayout_2;
    QPushButton *buttonSongLoad;
    QPushButton *buttonSongAdd;
    QPushButton *buttonSongModify;
    QPushButton *buttonSongRemove;
    QCheckBox *checkBoxPlaylistActive;
    QWidget *layoutWidgetPlay;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QLabel *midiBaseDirLabel;
    QLineEdit *midiBaseDirText;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_currentSongPath;
    QLineEdit *currentSongPath;
    QWidget *optionLayoutWidget;
    QGridLayout *gridLayout_2;
    QLabel *label_Auto;
    QCheckBox *checkBoxAutoArm;
    QLabel *label_Auto_Blank_1;
    QCheckBox *checkBoxAutoPlay;
    QLabel *label_Auto_Blank_2;
    QCheckBox *checkBoxAutoAdvance;

    void setupUi(QFrame *qplaylistframe)
    {
        if (qplaylistframe->objectName().isEmpty())
            qplaylistframe->setObjectName(QString::fromUtf8("qplaylistframe"));
        qplaylistframe->resize(825, 423);
        fileLayoutWidget = new QWidget(qplaylistframe);
        fileLayoutWidget->setObjectName(QString::fromUtf8("fileLayoutWidget"));
        fileLayoutWidget->setGeometry(QRect(6, 0, 781, 30));
        horizPlaylistFileLayout = new QHBoxLayout(fileLayoutWidget);
        horizPlaylistFileLayout->setObjectName(QString::fromUtf8("horizPlaylistFileLayout"));
        horizPlaylistFileLayout->setContentsMargins(0, 0, 0, 0);
        label_playlist_file = new QLabel(fileLayoutWidget);
        label_playlist_file->setObjectName(QString::fromUtf8("label_playlist_file"));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label_playlist_file->setFont(font);

        horizPlaylistFileLayout->addWidget(label_playlist_file);

        entry_playlist_file = new QLineEdit(fileLayoutWidget);
        entry_playlist_file->setObjectName(QString::fromUtf8("entry_playlist_file"));
        entry_playlist_file->setEnabled(false);
        entry_playlist_file->setMinimumSize(QSize(0, 24));
        entry_playlist_file->setMaximumSize(QSize(16777215, 24));
        entry_playlist_file->setReadOnly(true);

        horizPlaylistFileLayout->addWidget(entry_playlist_file);

        selectionLayoutWidget = new QWidget(qplaylistframe);
        selectionLayoutWidget->setObjectName(QString::fromUtf8("selectionLayoutWidget"));
        selectionLayoutWidget->setGeometry(QRect(6, 30, 824, 309));
        verticalLayout_3 = new QVBoxLayout(selectionLayoutWidget);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(2);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        buttonSelectSongDir = new QPushButton(selectionLayoutWidget);
        buttonSelectSongDir->setObjectName(QString::fromUtf8("buttonSelectSongDir"));
        buttonSelectSongDir->setMinimumSize(QSize(24, 24));
        buttonSelectSongDir->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonSelectSongDir, 1, 4, 1, 1);

        labelCurrentPlaylist = new QLabel(selectionLayoutWidget);
        labelCurrentPlaylist->setObjectName(QString::fromUtf8("labelCurrentPlaylist"));
        labelCurrentPlaylist->setMinimumSize(QSize(0, 24));
        labelCurrentPlaylist->setMaximumSize(QSize(16777215, 24));
        QFont font1;
        font1.setPointSize(11);
        font1.setItalic(true);
        labelCurrentPlaylist->setFont(font1);

        gridLayout->addWidget(labelCurrentPlaylist, 3, 0, 1, 1);

        labelSongReserved = new QLabel(selectionLayoutWidget);
        labelSongReserved->setObjectName(QString::fromUtf8("labelSongReserved"));

        gridLayout->addWidget(labelSongReserved, 2, 10, 2, 1);

        editPlaylistPath = new QLineEdit(selectionLayoutWidget);
        editPlaylistPath->setObjectName(QString::fromUtf8("editPlaylistPath"));
        editPlaylistPath->setMinimumSize(QSize(0, 24));
        editPlaylistPath->setMaximumSize(QSize(340, 24));
        QFont font2;
        font2.setPointSize(11);
        editPlaylistPath->setFont(font2);
        editPlaylistPath->setReadOnly(false);
        editPlaylistPath->setClearButtonEnabled(true);

        gridLayout->addWidget(editPlaylistPath, 1, 1, 2, 3);

        labelPlaylistDirectory = new QLabel(selectionLayoutWidget);
        labelPlaylistDirectory->setObjectName(QString::fromUtf8("labelPlaylistDirectory"));
        labelPlaylistDirectory->setMinimumSize(QSize(0, 24));
        labelPlaylistDirectory->setMaximumSize(QSize(16777215, 24));
        labelPlaylistDirectory->setFont(font1);

        gridLayout->addWidget(labelPlaylistDirectory, 1, 0, 1, 1);

        spinPlaylistNumber = new QSpinBox(selectionLayoutWidget);
        spinPlaylistNumber->setObjectName(QString::fromUtf8("spinPlaylistNumber"));
        spinPlaylistNumber->setMinimumSize(QSize(74, 24));
        spinPlaylistNumber->setMaximumSize(QSize(16777215, 24));
        spinPlaylistNumber->setFont(font2);
        spinPlaylistNumber->setMaximum(127);
        spinPlaylistNumber->setValue(127);

        gridLayout->addWidget(spinPlaylistNumber, 3, 2, 1, 1);

        editPlaylistNumber = new QLineEdit(selectionLayoutWidget);
        editPlaylistNumber->setObjectName(QString::fromUtf8("editPlaylistNumber"));
        editPlaylistNumber->setMinimumSize(QSize(24, 24));
        editPlaylistNumber->setMaximumSize(QSize(32, 24));
        editPlaylistNumber->setFont(font2);

        gridLayout->addWidget(editPlaylistNumber, 3, 1, 1, 1);

        labelCurrentSong = new QLabel(selectionLayoutWidget);
        labelCurrentSong->setObjectName(QString::fromUtf8("labelCurrentSong"));
        labelCurrentSong->setFont(font1);

        gridLayout->addWidget(labelCurrentSong, 3, 5, 1, 1);

        labelListSong = new QLabel(selectionLayoutWidget);
        labelListSong->setObjectName(QString::fromUtf8("labelListSong"));
        labelListSong->setMaximumSize(QSize(300, 16777215));
        labelListSong->setFont(font);

        gridLayout->addWidget(labelListSong, 0, 5, 1, 4);

        labelListSection = new QLabel(selectionLayoutWidget);
        labelListSection->setObjectName(QString::fromUtf8("labelListSection"));
        labelListSection->setMaximumSize(QSize(400, 16777215));
        labelListSection->setFont(font);

        gridLayout->addWidget(labelListSection, 0, 0, 1, 4);

        editSongNumber = new QLineEdit(selectionLayoutWidget);
        editSongNumber->setObjectName(QString::fromUtf8("editSongNumber"));
        editSongNumber->setMinimumSize(QSize(24, 24));
        editSongNumber->setMaximumSize(QSize(32, 24));
        editSongNumber->setFont(font2);

        gridLayout->addWidget(editSongNumber, 3, 6, 1, 1);

        editPlaylistName = new QLineEdit(selectionLayoutWidget);
        editPlaylistName->setObjectName(QString::fromUtf8("editPlaylistName"));
        editPlaylistName->setEnabled(true);
        editPlaylistName->setMinimumSize(QSize(228, 24));
        editPlaylistName->setMaximumSize(QSize(228, 24));
        editPlaylistName->setFont(font2);
        editPlaylistName->setClearButtonEnabled(true);

        gridLayout->addWidget(editPlaylistName, 3, 3, 1, 1);

        editSongPath = new QLineEdit(selectionLayoutWidget);
        editSongPath->setObjectName(QString::fromUtf8("editSongPath"));
        editSongPath->setEnabled(true);
        editSongPath->setMinimumSize(QSize(0, 24));
        editSongPath->setMaximumSize(QSize(300, 24));
        editSongPath->setFont(font2);
        editSongPath->setClearButtonEnabled(true);

        gridLayout->addWidget(editSongPath, 1, 6, 2, 3);

        editSongFilename = new QLineEdit(selectionLayoutWidget);
        editSongFilename->setObjectName(QString::fromUtf8("editSongFilename"));
        editSongFilename->setEnabled(true);
        editSongFilename->setMinimumSize(QSize(208, 24));
        editSongFilename->setMaximumSize(QSize(208, 24));
        editSongFilename->setFont(font2);
        editSongFilename->setReadOnly(true);
        editSongFilename->setClearButtonEnabled(true);

        gridLayout->addWidget(editSongFilename, 3, 8, 1, 1);

        labelSongDirectory = new QLabel(selectionLayoutWidget);
        labelSongDirectory->setObjectName(QString::fromUtf8("labelSongDirectory"));
        labelSongDirectory->setMinimumSize(QSize(0, 24));
        labelSongDirectory->setMaximumSize(QSize(16777215, 24));
        labelSongDirectory->setFont(font1);

        gridLayout->addWidget(labelSongDirectory, 1, 5, 1, 1);

        labelDirEmbedded = new QLabel(selectionLayoutWidget);
        labelDirEmbedded->setObjectName(QString::fromUtf8("labelDirEmbedded"));

        gridLayout->addWidget(labelDirEmbedded, 1, 10, 1, 1);

        spinSongNumber = new QSpinBox(selectionLayoutWidget);
        spinSongNumber->setObjectName(QString::fromUtf8("spinSongNumber"));
        spinSongNumber->setMinimumSize(QSize(0, 24));
        spinSongNumber->setMaximumSize(QSize(64, 24));
        spinSongNumber->setFont(font2);
        spinSongNumber->setMaximum(127);
        spinSongNumber->setValue(127);

        gridLayout->addWidget(spinSongNumber, 3, 7, 1, 1);

        buttonSelectPlaylist = new QPushButton(selectionLayoutWidget);
        buttonSelectPlaylist->setObjectName(QString::fromUtf8("buttonSelectPlaylist"));
        buttonSelectPlaylist->setMinimumSize(QSize(24, 24));
        buttonSelectPlaylist->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonSelectPlaylist, 0, 4, 1, 1);

        buttonSelectSong = new QPushButton(selectionLayoutWidget);
        buttonSelectSong->setObjectName(QString::fromUtf8("buttonSelectSong"));
        buttonSelectSong->setMinimumSize(QSize(24, 24));
        buttonSelectSong->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonSelectSong, 0, 9, 1, 1);

        buttonSongReserved_1 = new QPushButton(selectionLayoutWidget);
        buttonSongReserved_1->setObjectName(QString::fromUtf8("buttonSongReserved_1"));
        buttonSongReserved_1->setEnabled(false);
        buttonSongReserved_1->setMinimumSize(QSize(24, 24));
        buttonSongReserved_1->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonSongReserved_1, 1, 9, 1, 1);

        buttonPlaylistReserved = new QPushButton(selectionLayoutWidget);
        buttonPlaylistReserved->setObjectName(QString::fromUtf8("buttonPlaylistReserved"));
        buttonPlaylistReserved->setEnabled(false);
        buttonPlaylistReserved->setMinimumSize(QSize(24, 24));
        buttonPlaylistReserved->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonPlaylistReserved, 3, 4, 1, 1);

        buttonSongreserved_2 = new QPushButton(selectionLayoutWidget);
        buttonSongreserved_2->setObjectName(QString::fromUtf8("buttonSongreserved_2"));
        buttonSongreserved_2->setEnabled(false);
        buttonSongreserved_2->setMinimumSize(QSize(24, 24));
        buttonSongreserved_2->setMaximumSize(QSize(24, 24));

        gridLayout->addWidget(buttonSongreserved_2, 3, 9, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(2);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        tablePlaylistSections = new QTableWidget(selectionLayoutWidget);
        if (tablePlaylistSections->columnCount() < 2)
            tablePlaylistSections->setColumnCount(2);
        if (tablePlaylistSections->rowCount() < 16)
            tablePlaylistSections->setRowCount(16);
        tablePlaylistSections->setObjectName(QString::fromUtf8("tablePlaylistSections"));
        tablePlaylistSections->setMaximumSize(QSize(320, 16777215));
        tablePlaylistSections->setFont(font2);
        tablePlaylistSections->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        tablePlaylistSections->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContentsOnFirstShow);
        tablePlaylistSections->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        tablePlaylistSections->setAlternatingRowColors(true);
        tablePlaylistSections->setRowCount(16);
        tablePlaylistSections->setColumnCount(2);
        tablePlaylistSections->horizontalHeader()->setStretchLastSection(true);
        tablePlaylistSections->verticalHeader()->setMinimumSectionSize(18);
        tablePlaylistSections->verticalHeader()->setDefaultSectionSize(21);
        tablePlaylistSections->verticalHeader()->setStretchLastSection(false);

        horizontalLayout->addWidget(tablePlaylistSections);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(12);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        buttonPlaylistCreate = new QPushButton(selectionLayoutWidget);
        buttonPlaylistCreate->setObjectName(QString::fromUtf8("buttonPlaylistCreate"));
        buttonPlaylistCreate->setEnabled(true);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonPlaylistCreate->sizePolicy().hasHeightForWidth());
        buttonPlaylistCreate->setSizePolicy(sizePolicy);
        buttonPlaylistCreate->setMinimumSize(QSize(76, 25));
        buttonPlaylistCreate->setMaximumSize(QSize(76, 25));
        buttonPlaylistCreate->setFont(font2);

        verticalLayout->addWidget(buttonPlaylistCreate);

        buttonPlaylistAdd = new QPushButton(selectionLayoutWidget);
        buttonPlaylistAdd->setObjectName(QString::fromUtf8("buttonPlaylistAdd"));
        buttonPlaylistAdd->setEnabled(false);
        buttonPlaylistAdd->setMinimumSize(QSize(76, 25));
        buttonPlaylistAdd->setMaximumSize(QSize(76, 25));
        buttonPlaylistAdd->setFont(font2);

        verticalLayout->addWidget(buttonPlaylistAdd);

        buttonPlaylistModify = new QPushButton(selectionLayoutWidget);
        buttonPlaylistModify->setObjectName(QString::fromUtf8("buttonPlaylistModify"));
        buttonPlaylistModify->setEnabled(false);
        buttonPlaylistModify->setMinimumSize(QSize(76, 25));
        buttonPlaylistModify->setMaximumSize(QSize(76, 25));
        buttonPlaylistModify->setFont(font2);

        verticalLayout->addWidget(buttonPlaylistModify);

        buttonPlaylistRemove = new QPushButton(selectionLayoutWidget);
        buttonPlaylistRemove->setObjectName(QString::fromUtf8("buttonPlaylistRemove"));
        buttonPlaylistRemove->setMinimumSize(QSize(76, 25));
        buttonPlaylistRemove->setMaximumSize(QSize(76, 25));
        buttonPlaylistRemove->setFont(font2);

        verticalLayout->addWidget(buttonPlaylistRemove);

        buttonPlaylistSave = new QPushButton(selectionLayoutWidget);
        buttonPlaylistSave->setObjectName(QString::fromUtf8("buttonPlaylistSave"));
        buttonPlaylistSave->setEnabled(false);
        buttonPlaylistSave->setMinimumSize(QSize(76, 25));
        buttonPlaylistSave->setMaximumSize(QSize(76, 25));
        buttonPlaylistSave->setFont(font2);

        verticalLayout->addWidget(buttonPlaylistSave);


        horizontalLayout->addLayout(verticalLayout);

        tablePlaylistSongs = new QTableWidget(selectionLayoutWidget);
        if (tablePlaylistSongs->columnCount() < 2)
            tablePlaylistSongs->setColumnCount(2);
        if (tablePlaylistSongs->rowCount() < 16)
            tablePlaylistSongs->setRowCount(16);
        tablePlaylistSongs->setObjectName(QString::fromUtf8("tablePlaylistSongs"));
        tablePlaylistSongs->setMinimumSize(QSize(300, 0));
        tablePlaylistSongs->setMaximumSize(QSize(310, 16777215));
        tablePlaylistSongs->setFont(font2);
        tablePlaylistSongs->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);
        tablePlaylistSongs->setSizeAdjustPolicy(QAbstractScrollArea::SizeAdjustPolicy::AdjustToContentsOnFirstShow);
        tablePlaylistSongs->setEditTriggers(QAbstractItemView::EditTrigger::NoEditTriggers);
        tablePlaylistSongs->setAlternatingRowColors(true);
        tablePlaylistSongs->setRowCount(16);
        tablePlaylistSongs->setColumnCount(2);
        tablePlaylistSongs->horizontalHeader()->setStretchLastSection(true);
        tablePlaylistSongs->verticalHeader()->setMinimumSectionSize(18);
        tablePlaylistSongs->verticalHeader()->setDefaultSectionSize(21);
        tablePlaylistSongs->verticalHeader()->setStretchLastSection(false);

        horizontalLayout->addWidget(tablePlaylistSongs);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(12);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        buttonSongLoad = new QPushButton(selectionLayoutWidget);
        buttonSongLoad->setObjectName(QString::fromUtf8("buttonSongLoad"));
        buttonSongLoad->setEnabled(true);
        buttonSongLoad->setMinimumSize(QSize(76, 25));
        buttonSongLoad->setMaximumSize(QSize(76, 25));
        buttonSongLoad->setFont(font2);

        verticalLayout_2->addWidget(buttonSongLoad);

        buttonSongAdd = new QPushButton(selectionLayoutWidget);
        buttonSongAdd->setObjectName(QString::fromUtf8("buttonSongAdd"));
        buttonSongAdd->setEnabled(false);
        buttonSongAdd->setMinimumSize(QSize(76, 25));
        buttonSongAdd->setMaximumSize(QSize(76, 25));
        buttonSongAdd->setFont(font2);

        verticalLayout_2->addWidget(buttonSongAdd);

        buttonSongModify = new QPushButton(selectionLayoutWidget);
        buttonSongModify->setObjectName(QString::fromUtf8("buttonSongModify"));
        buttonSongModify->setEnabled(false);
        buttonSongModify->setMinimumSize(QSize(76, 25));
        buttonSongModify->setMaximumSize(QSize(76, 25));
        buttonSongModify->setFont(font2);

        verticalLayout_2->addWidget(buttonSongModify);

        buttonSongRemove = new QPushButton(selectionLayoutWidget);
        buttonSongRemove->setObjectName(QString::fromUtf8("buttonSongRemove"));
        buttonSongRemove->setEnabled(true);
        buttonSongRemove->setMinimumSize(QSize(76, 25));
        buttonSongRemove->setMaximumSize(QSize(76, 25));
        buttonSongRemove->setFont(font2);

        verticalLayout_2->addWidget(buttonSongRemove);

        checkBoxPlaylistActive = new QCheckBox(selectionLayoutWidget);
        checkBoxPlaylistActive->setObjectName(QString::fromUtf8("checkBoxPlaylistActive"));
        checkBoxPlaylistActive->setMinimumSize(QSize(76, 32));
        checkBoxPlaylistActive->setMaximumSize(QSize(76, 40));
        checkBoxPlaylistActive->setFont(font2);

        verticalLayout_2->addWidget(checkBoxPlaylistActive);


        horizontalLayout->addLayout(verticalLayout_2);

        horizontalLayout->setStretch(0, 4);
        horizontalLayout->setStretch(1, 1);
        horizontalLayout->setStretch(2, 4);
        horizontalLayout->setStretch(3, 1);

        verticalLayout_3->addLayout(horizontalLayout);

        layoutWidgetPlay = new QWidget(qplaylistframe);
        layoutWidgetPlay->setObjectName(QString::fromUtf8("layoutWidgetPlay"));
        layoutWidgetPlay->setGeometry(QRect(8, 344, 617, 69));
        verticalLayout_5 = new QVBoxLayout(layoutWidgetPlay);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        midiBaseDirLabel = new QLabel(layoutWidgetPlay);
        midiBaseDirLabel->setObjectName(QString::fromUtf8("midiBaseDirLabel"));
        midiBaseDirLabel->setMinimumSize(QSize(0, 24));
        midiBaseDirLabel->setMaximumSize(QSize(16777215, 24));
        midiBaseDirLabel->setFont(font);

        horizontalLayout_2->addWidget(midiBaseDirLabel);

        midiBaseDirText = new QLineEdit(layoutWidgetPlay);
        midiBaseDirText->setObjectName(QString::fromUtf8("midiBaseDirText"));
        midiBaseDirText->setEnabled(false);
        midiBaseDirText->setMinimumSize(QSize(0, 24));
        midiBaseDirText->setMaximumSize(QSize(16777215, 24));
        midiBaseDirText->setFont(font2);
        midiBaseDirText->setReadOnly(true);

        horizontalLayout_2->addWidget(midiBaseDirText);


        verticalLayout_5->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_currentSongPath = new QLabel(layoutWidgetPlay);
        label_currentSongPath->setObjectName(QString::fromUtf8("label_currentSongPath"));
        label_currentSongPath->setMinimumSize(QSize(0, 24));
        label_currentSongPath->setMaximumSize(QSize(16777215, 24));
        label_currentSongPath->setFont(font);

        horizontalLayout_3->addWidget(label_currentSongPath);

        currentSongPath = new QLineEdit(layoutWidgetPlay);
        currentSongPath->setObjectName(QString::fromUtf8("currentSongPath"));
        currentSongPath->setEnabled(false);
        currentSongPath->setMinimumSize(QSize(0, 24));
        currentSongPath->setMaximumSize(QSize(16777215, 24));
        currentSongPath->setFont(font2);
        currentSongPath->setReadOnly(true);

        horizontalLayout_3->addWidget(currentSongPath);


        verticalLayout_5->addLayout(horizontalLayout_3);

        optionLayoutWidget = new QWidget(qplaylistframe);
        optionLayoutWidget->setObjectName(QString::fromUtf8("optionLayoutWidget"));
        optionLayoutWidget->setGeometry(QRect(634, 344, 134, 83));
        gridLayout_2 = new QGridLayout(optionLayoutWidget);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        label_Auto = new QLabel(optionLayoutWidget);
        label_Auto->setObjectName(QString::fromUtf8("label_Auto"));
        label_Auto->setFont(font);

        gridLayout_2->addWidget(label_Auto, 0, 0, 1, 1);

        checkBoxAutoArm = new QCheckBox(optionLayoutWidget);
        checkBoxAutoArm->setObjectName(QString::fromUtf8("checkBoxAutoArm"));
        checkBoxAutoArm->setFont(font2);

        gridLayout_2->addWidget(checkBoxAutoArm, 0, 1, 1, 1);

        label_Auto_Blank_1 = new QLabel(optionLayoutWidget);
        label_Auto_Blank_1->setObjectName(QString::fromUtf8("label_Auto_Blank_1"));

        gridLayout_2->addWidget(label_Auto_Blank_1, 1, 0, 1, 1);

        checkBoxAutoPlay = new QCheckBox(optionLayoutWidget);
        checkBoxAutoPlay->setObjectName(QString::fromUtf8("checkBoxAutoPlay"));
        checkBoxAutoPlay->setFont(font2);

        gridLayout_2->addWidget(checkBoxAutoPlay, 1, 1, 2, 1);

        label_Auto_Blank_2 = new QLabel(optionLayoutWidget);
        label_Auto_Blank_2->setObjectName(QString::fromUtf8("label_Auto_Blank_2"));

        gridLayout_2->addWidget(label_Auto_Blank_2, 2, 0, 2, 1);

        checkBoxAutoAdvance = new QCheckBox(optionLayoutWidget);
        checkBoxAutoAdvance->setObjectName(QString::fromUtf8("checkBoxAutoAdvance"));
        checkBoxAutoAdvance->setFont(font2);

        gridLayout_2->addWidget(checkBoxAutoAdvance, 3, 1, 1, 1);


        retranslateUi(qplaylistframe);

        QMetaObject::connectSlotsByName(qplaylistframe);
    } // setupUi

    void retranslateUi(QFrame *qplaylistframe)
    {
        qplaylistframe->setWindowTitle(QCoreApplication::translate("qplaylistframe", "Frame", nullptr));
        label_playlist_file->setText(QCoreApplication::translate("qplaylistframe", "Playlist File", nullptr));
#if QT_CONFIG(tooltip)
        entry_playlist_file->setToolTip(QCoreApplication::translate("qplaylistframe", "Full path to the play-list file.  Read-only.\n"
"Click \"Load List\" to select a playlist file.", nullptr));
#endif // QT_CONFIG(tooltip)
        entry_playlist_file->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
#if QT_CONFIG(tooltip)
        buttonSelectSongDir->setToolTip(QCoreApplication::translate("qplaylistframe", "Allows the user to find a directory and\n"
"select it, instead of typing the full path\n"
"name in the edit control to the left.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSelectSongDir->setText(QCoreApplication::translate("qplaylistframe", "...", nullptr));
        labelCurrentPlaylist->setText(QCoreApplication::translate("qplaylistframe", "MIDI #:", nullptr));
        labelSongReserved->setText(QString());
#if QT_CONFIG(tooltip)
        editPlaylistPath->setToolTip(QCoreApplication::translate("qplaylistframe", "The base directory for the play-list selected\n"
"in the table below. Use the \"...\" to the right\n"
"to select the directory. The directory is not\n"
"used in the MIDI file contains a full path to\n"
"the file.", nullptr));
#endif // QT_CONFIG(tooltip)
        editPlaylistPath->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
        labelPlaylistDirectory->setText(QCoreApplication::translate("qplaylistframe", "Song Dir:", nullptr));
#if QT_CONFIG(tooltip)
        spinPlaylistNumber->setToolTip(QCoreApplication::translate("qplaylistframe", "Provides the ordering and control number\n"
"for this play-list, when added. Ranges from\n"
"0 to 127.", nullptr));
#endif // QT_CONFIG(tooltip)
        editPlaylistNumber->setText(QCoreApplication::translate("qplaylistframe", "000", nullptr));
        labelCurrentSong->setText(QCoreApplication::translate("qplaylistframe", "MIDI #:", nullptr));
        labelListSong->setText(QCoreApplication::translate("qplaylistframe", "Song Selection", nullptr));
        labelListSection->setText(QCoreApplication::translate("qplaylistframe", "Playlist File Selection", nullptr));
        editSongNumber->setText(QCoreApplication::translate("qplaylistframe", "000", nullptr));
#if QT_CONFIG(tooltip)
        editPlaylistName->setToolTip(QCoreApplication::translate("qplaylistframe", "Name of the currently-selected play-list section.\n"
"Edit to enable \"Add List\" and \"Modify List\".", nullptr));
#endif // QT_CONFIG(tooltip)
        editPlaylistName->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
#if QT_CONFIG(tooltip)
        editSongPath->setToolTip(QCoreApplication::translate("qplaylistframe", "Directory where the selected\n"
"song is stored. Use the \"Load Song\" button\n"
"to select an existing song.", nullptr));
#endif // QT_CONFIG(tooltip)
        editSongPath->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
#if QT_CONFIG(tooltip)
        editSongFilename->setToolTip(QCoreApplication::translate("qplaylistframe", "Base file-name of the currently selected song. \n"
"Read-only.  Use the Load Song button to modify this entry.", nullptr));
#endif // QT_CONFIG(tooltip)
        editSongFilename->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
        labelSongDirectory->setText(QCoreApplication::translate("qplaylistframe", "MIDI Dir:", nullptr));
#if QT_CONFIG(tooltip)
        labelDirEmbedded->setToolTip(QCoreApplication::translate("qplaylistframe", "If set to an asterisk, this directory is part of the song's file-name.", nullptr));
#endif // QT_CONFIG(tooltip)
        labelDirEmbedded->setText(QCoreApplication::translate("qplaylistframe", "*", nullptr));
#if QT_CONFIG(tooltip)
        spinSongNumber->setToolTip(QCoreApplication::translate("qplaylistframe", "Provides the ordering and control number\n"
"for this song, when added. Ranges from\n"
"0 to 127.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonSelectPlaylist->setToolTip(QCoreApplication::translate("qplaylistframe", "Look up a .playlist file and load it into\n"
"the play-list table.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSelectPlaylist->setText(QCoreApplication::translate("qplaylistframe", "...", nullptr));
#if QT_CONFIG(tooltip)
        buttonSelectSong->setToolTip(QCoreApplication::translate("qplaylistframe", "Look up a MIDI or WRK file and\n"
"add it to the song table.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSelectSong->setText(QCoreApplication::translate("qplaylistframe", "...", nullptr));
        buttonSongReserved_1->setText(QString());
#if QT_CONFIG(tooltip)
        buttonPlaylistReserved->setToolTip(QCoreApplication::translate("qplaylistframe", "Reserved for expansion", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistReserved->setText(QString());
        buttonSongreserved_2->setText(QString());
#if QT_CONFIG(tooltip)
        tablePlaylistSections->setToolTip(QCoreApplication::translate("qplaylistframe", "Shows existing play-lists read from the play-list file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        tablePlaylistSections->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(tooltip)
        buttonPlaylistCreate->setToolTip(QCoreApplication::translate("qplaylistframe", "Opens a dialog to set a directory and play-list file.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistCreate->setText(QCoreApplication::translate("qplaylistframe", "Create File", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlaylistAdd->setToolTip(QCoreApplication::translate("qplaylistframe", "Creates a new play-list file.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistAdd->setText(QCoreApplication::translate("qplaylistframe", "Add List", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlaylistModify->setToolTip(QCoreApplication::translate("qplaylistframe", "Update the current list with the 3 editable fields above.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistModify->setText(QCoreApplication::translate("qplaylistframe", "Modify", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlaylistRemove->setToolTip(QCoreApplication::translate("qplaylistframe", "To do: Removes the selected playlist from this play-list set.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistRemove->setText(QCoreApplication::translate("qplaylistframe", "Delete", nullptr));
#if QT_CONFIG(tooltip)
        buttonPlaylistSave->setToolTip(QCoreApplication::translate("qplaylistframe", "Saves the current state of the play-lists file, including song\n"
"information.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonPlaylistSave->setText(QCoreApplication::translate("qplaylistframe", "Save Lists", nullptr));
#if QT_CONFIG(tooltip)
        tablePlaylistSongs->setToolTip(QCoreApplication::translate("qplaylistframe", "Shows MIDI tunes in the currently-selected play-list.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonSongLoad->setToolTip(QCoreApplication::translate("qplaylistframe", "Opens a file dialog to select a MIDI song to add to the current\n"
"play-list; the Directory and Current values above are updated\n"
"automatically.  In Non Session Manager mode, the file is\n"
"imported.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSongLoad->setText(QCoreApplication::translate("qplaylistframe", "Load", nullptr));
#if QT_CONFIG(tooltip)
        buttonSongAdd->setToolTip(QCoreApplication::translate("qplaylistframe", "Adds the song information specified above\n"
"to the current play-list section; it is easier\n"
"to use \"Load Song\".", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSongAdd->setText(QCoreApplication::translate("qplaylistframe", "Add Song", nullptr));
#if QT_CONFIG(tooltip)
        buttonSongModify->setToolTip(QCoreApplication::translate("qplaylistframe", "Modifies the information about the currently-selected song\n"
"in the 3 entry fields above.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSongModify->setText(QCoreApplication::translate("qplaylistframe", "Modify", nullptr));
#if QT_CONFIG(tooltip)
        buttonSongRemove->setToolTip(QCoreApplication::translate("qplaylistframe", "Removes the current song from the current play-list;\n"
"the song itself is not deleted.", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonSongRemove->setText(QCoreApplication::translate("qplaylistframe", "Delete", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxPlaylistActive->setToolTip(QCoreApplication::translate("qplaylistframe", "If checked, the loaded play-list file is active.  The lists and \n"
"songs will be loaded when the arrow keys are pressed or \n"
"the configured MIDI control comes in.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxPlaylistActive->setText(QCoreApplication::translate("qplaylistframe", " Lists\n"
" active", nullptr));
        midiBaseDirLabel->setText(QCoreApplication::translate("qplaylistframe", "MIDI Base Directory", nullptr));
#if QT_CONFIG(tooltip)
        midiBaseDirText->setToolTip(QCoreApplication::translate("qplaylistframe", "Base directory holding all files in all playlists.\n"
"Read-only.", nullptr));
#endif // QT_CONFIG(tooltip)
        midiBaseDirText->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
        label_currentSongPath->setText(QCoreApplication::translate("qplaylistframe", "Current Song Path", nullptr));
#if QT_CONFIG(tooltip)
        currentSongPath->setToolTip(QCoreApplication::translate("qplaylistframe", "Full path to the currently-selected song.\n"
"Read-only.", nullptr));
#endif // QT_CONFIG(tooltip)
        currentSongPath->setText(QCoreApplication::translate("qplaylistframe", "None", nullptr));
        label_Auto->setText(QCoreApplication::translate("qplaylistframe", "Auto:", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxAutoArm->setToolTip(QCoreApplication::translate("qplaylistframe", "If checked, moving to the next song\n"
"automatically arms (unmutes) all\n"
"patterns in the song, if there is no\n"
"Song layout.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxAutoArm->setText(QCoreApplication::translate("qplaylistframe", "Arm", nullptr));
        label_Auto_Blank_1->setText(QString());
#if QT_CONFIG(tooltip)
        checkBoxAutoPlay->setToolTip(QCoreApplication::translate("qplaylistframe", "If checked, the next selected song starts\n"
"playing automatically. (Perhaps it should\n"
"also load the next song and keep going!)", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxAutoPlay->setText(QCoreApplication::translate("qplaylistframe", "Play", nullptr));
        label_Auto_Blank_2->setText(QString());
#if QT_CONFIG(tooltip)
        checkBoxAutoAdvance->setToolTip(QCoreApplication::translate("qplaylistframe", "If checked, auto-arm and auto-play are also\n"
"enabled, and once Play is pressed, the\n"
"whole play-list is played without a break.\n"
"\n"
"NOT YET ENABLED.", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxAutoAdvance->setText(QCoreApplication::translate("qplaylistframe", "Advance", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qplaylistframe: public Ui_qplaylistframe {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QPLAYLISTFRAME_H
