/********************************************************************************
** Form generated from reading UI file 'qmutemaster.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QMUTEMASTER_H
#define QMUTEMASTER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qmutemaster
{
public:
    QWidget *layoutWidgetMuteMaster;
    QHBoxLayout *horizontalLayout_3;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_4;
    QLabel *label_3;
    QGridLayout *setGridLayout;
    QLabel *label_5;
    QGridLayout *patternGridLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *m_mutes_file_label;
    QPlainTextEdit *m_mute_basename;
    QCheckBox *m_check_mutes_active;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QGridLayout *gridLayout;
    QCheckBox *m_check_to_midi;
    QCheckBox *m_check_to_mutes;
    QRadioButton *m_radio_hex;
    QRadioButton *m_radio_binary;
    QCheckBox *m_check_from_midi;
    QCheckBox *m_check_from_mutes;
    QHBoxLayout *horizontalLayout_5;
    QCheckBox *m_check_toggle_active;
    QCheckBox *m_check_strip_empty;
    QVBoxLayout *verticalLayout;
    QTableWidget *m_group_table;
    QGridLayout *gridLayout_2;
    QPushButton *m_button_trigger;
    QPushButton *m_button_clear_all;
    QPushButton *m_button_reserved;
    QPushButton *m_button_save;
    QPushButton *m_button_load;
    QPushButton *m_button_set_mutes;

    void setupUi(QFrame *qmutemaster)
    {
        if (qmutemaster->objectName().isEmpty())
            qmutemaster->setObjectName(QString::fromUtf8("qmutemaster"));
        qmutemaster->resize(848, 439);
        qmutemaster->setFrameShape(QFrame::Shape::NoFrame);
        qmutemaster->setFrameShadow(QFrame::Shadow::Plain);
        layoutWidgetMuteMaster = new QWidget(qmutemaster);
        layoutWidgetMuteMaster->setObjectName(QString::fromUtf8("layoutWidgetMuteMaster"));
        layoutWidgetMuteMaster->setGeometry(QRect(9, 9, 844, 391));
        horizontalLayout_3 = new QHBoxLayout(layoutWidgetMuteMaster);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        horizontalLayout_3->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(8);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setSizeConstraint(QLayout::SizeConstraint::SetMinimumSize);
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        label_4 = new QLabel(layoutWidgetMuteMaster);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setMinimumSize(QSize(120, 22));
        label_4->setMaximumSize(QSize(120, 28));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        label_4->setFont(font);

        horizontalLayout_7->addWidget(label_4);

        label_3 = new QLabel(layoutWidgetMuteMaster);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setMaximumSize(QSize(16777215, 28));
        QFont font1;
        font1.setPointSize(11);
        font1.setItalic(true);
        label_3->setFont(font1);
        label_3->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout_7->addWidget(label_3);


        verticalLayout_2->addLayout(horizontalLayout_7);

        setGridLayout = new QGridLayout();
        setGridLayout->setObjectName(QString::fromUtf8("setGridLayout"));
        setGridLayout->setSizeConstraint(QLayout::SizeConstraint::SetMinAndMaxSize);
        setGridLayout->setVerticalSpacing(2);

        verticalLayout_2->addLayout(setGridLayout);

        label_5 = new QLabel(layoutWidgetMuteMaster);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setMinimumSize(QSize(136, 22));
        label_5->setMaximumSize(QSize(152, 22));
        label_5->setFont(font);

        verticalLayout_2->addWidget(label_5);

        patternGridLayout = new QGridLayout();
        patternGridLayout->setObjectName(QString::fromUtf8("patternGridLayout"));
        patternGridLayout->setSizeConstraint(QLayout::SizeConstraint::SetMinimumSize);
        patternGridLayout->setVerticalSpacing(2);

        verticalLayout_2->addLayout(patternGridLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SizeConstraint::SetMinimumSize);
        m_mutes_file_label = new QLabel(layoutWidgetMuteMaster);
        m_mutes_file_label->setObjectName(QString::fromUtf8("m_mutes_file_label"));
        m_mutes_file_label->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_mutes_file_label->sizePolicy().hasHeightForWidth());
        m_mutes_file_label->setSizePolicy(sizePolicy);
        m_mutes_file_label->setMinimumSize(QSize(116, 28));
        m_mutes_file_label->setMaximumSize(QSize(120, 28));
        m_mutes_file_label->setFont(font);
        m_mutes_file_label->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        horizontalLayout->addWidget(m_mutes_file_label);

        m_mute_basename = new QPlainTextEdit(layoutWidgetMuteMaster);
        m_mute_basename->setObjectName(QString::fromUtf8("m_mute_basename"));
        m_mute_basename->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_mute_basename->sizePolicy().hasHeightForWidth());
        m_mute_basename->setSizePolicy(sizePolicy1);
        m_mute_basename->setMinimumSize(QSize(200, 28));
        m_mute_basename->setMaximumSize(QSize(200, 28));
        m_mute_basename->setVerticalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);

        horizontalLayout->addWidget(m_mute_basename);

        m_check_mutes_active = new QCheckBox(layoutWidgetMuteMaster);
        m_check_mutes_active->setObjectName(QString::fromUtf8("m_check_mutes_active"));
        m_check_mutes_active->setEnabled(false);
        m_check_mutes_active->setMinimumSize(QSize(0, 28));
        m_check_mutes_active->setMaximumSize(QSize(16777215, 28));

        horizontalLayout->addWidget(m_check_mutes_active);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_2 = new QLabel(layoutWidgetMuteMaster);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setFont(font);

        horizontalLayout_2->addWidget(label_2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        m_check_to_midi = new QCheckBox(layoutWidgetMuteMaster);
        m_check_to_midi->setObjectName(QString::fromUtf8("m_check_to_midi"));
        QFont font2;
        font2.setPointSize(11);
        m_check_to_midi->setFont(font2);

        gridLayout->addWidget(m_check_to_midi, 0, 2, 1, 1);

        m_check_to_mutes = new QCheckBox(layoutWidgetMuteMaster);
        m_check_to_mutes->setObjectName(QString::fromUtf8("m_check_to_mutes"));
        m_check_to_mutes->setFont(font2);

        gridLayout->addWidget(m_check_to_mutes, 1, 2, 1, 1);

        m_radio_hex = new QRadioButton(layoutWidgetMuteMaster);
        m_radio_hex->setObjectName(QString::fromUtf8("m_radio_hex"));
        m_radio_hex->setMaximumSize(QSize(80, 16777215));
        m_radio_hex->setFont(font2);

        gridLayout->addWidget(m_radio_hex, 1, 1, 1, 1);

        m_radio_binary = new QRadioButton(layoutWidgetMuteMaster);
        m_radio_binary->setObjectName(QString::fromUtf8("m_radio_binary"));
        m_radio_binary->setMaximumSize(QSize(80, 16777215));
        m_radio_binary->setFont(font2);

        gridLayout->addWidget(m_radio_binary, 0, 1, 1, 1);

        m_check_from_midi = new QCheckBox(layoutWidgetMuteMaster);
        m_check_from_midi->setObjectName(QString::fromUtf8("m_check_from_midi"));
        m_check_from_midi->setFont(font2);

        gridLayout->addWidget(m_check_from_midi, 0, 0, 1, 1);

        m_check_from_mutes = new QCheckBox(layoutWidgetMuteMaster);
        m_check_from_mutes->setObjectName(QString::fromUtf8("m_check_from_mutes"));
        m_check_from_mutes->setFont(font2);

        gridLayout->addWidget(m_check_from_mutes, 1, 0, 1, 1);


        horizontalLayout_2->addLayout(gridLayout);


        verticalLayout_2->addLayout(horizontalLayout_2);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        m_check_toggle_active = new QCheckBox(layoutWidgetMuteMaster);
        m_check_toggle_active->setObjectName(QString::fromUtf8("m_check_toggle_active"));
        m_check_toggle_active->setMaximumSize(QSize(160, 16777215));
        m_check_toggle_active->setFont(font2);

        horizontalLayout_5->addWidget(m_check_toggle_active);

        m_check_strip_empty = new QCheckBox(layoutWidgetMuteMaster);
        m_check_strip_empty->setObjectName(QString::fromUtf8("m_check_strip_empty"));
        m_check_strip_empty->setMaximumSize(QSize(170, 16777215));
        m_check_strip_empty->setFont(font2);

        horizontalLayout_5->addWidget(m_check_strip_empty);


        verticalLayout_2->addLayout(horizontalLayout_5);


        horizontalLayout_3->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(12);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        m_group_table = new QTableWidget(layoutWidgetMuteMaster);
        if (m_group_table->columnCount() < 4)
            m_group_table->setColumnCount(4);
        if (m_group_table->rowCount() < 32)
            m_group_table->setRowCount(32);
        m_group_table->setObjectName(QString::fromUtf8("m_group_table"));
        m_group_table->setEnabled(true);
        m_group_table->setMinimumSize(QSize(400, 280));
        m_group_table->setMaximumSize(QSize(400, 340));
        m_group_table->setFont(font2);
        m_group_table->setAlternatingRowColors(false);
        m_group_table->setRowCount(32);
        m_group_table->setColumnCount(4);
        m_group_table->horizontalHeader()->setVisible(true);
        m_group_table->verticalHeader()->setVisible(false);
        m_group_table->verticalHeader()->setMinimumSectionSize(16);
        m_group_table->verticalHeader()->setDefaultSectionSize(18);

        verticalLayout->addWidget(m_group_table);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        m_button_trigger = new QPushButton(layoutWidgetMuteMaster);
        m_button_trigger->setObjectName(QString::fromUtf8("m_button_trigger"));
        m_button_trigger->setEnabled(true);
        m_button_trigger->setMaximumSize(QSize(96, 16777215));
        m_button_trigger->setFont(font2);

        gridLayout_2->addWidget(m_button_trigger, 0, 0, 1, 1);

        m_button_clear_all = new QPushButton(layoutWidgetMuteMaster);
        m_button_clear_all->setObjectName(QString::fromUtf8("m_button_clear_all"));
        m_button_clear_all->setMaximumSize(QSize(96, 16777215));
        m_button_clear_all->setFont(font2);

        gridLayout_2->addWidget(m_button_clear_all, 0, 1, 1, 1);

        m_button_reserved = new QPushButton(layoutWidgetMuteMaster);
        m_button_reserved->setObjectName(QString::fromUtf8("m_button_reserved"));
        m_button_reserved->setEnabled(false);
        m_button_reserved->setMaximumSize(QSize(16, 16));

        gridLayout_2->addWidget(m_button_reserved, 0, 2, 1, 1);

        m_button_save = new QPushButton(layoutWidgetMuteMaster);
        m_button_save->setObjectName(QString::fromUtf8("m_button_save"));
        m_button_save->setEnabled(true);
        m_button_save->setMinimumSize(QSize(72, 0));
        m_button_save->setMaximumSize(QSize(96, 16777215));
        m_button_save->setFont(font2);

        gridLayout_2->addWidget(m_button_save, 1, 0, 1, 1);

        m_button_load = new QPushButton(layoutWidgetMuteMaster);
        m_button_load->setObjectName(QString::fromUtf8("m_button_load"));
        m_button_load->setMaximumSize(QSize(96, 16777215));
        m_button_load->setFont(font2);

        gridLayout_2->addWidget(m_button_load, 1, 1, 1, 1);

        m_button_set_mutes = new QPushButton(layoutWidgetMuteMaster);
        m_button_set_mutes->setObjectName(QString::fromUtf8("m_button_set_mutes"));
        m_button_set_mutes->setEnabled(false);
        m_button_set_mutes->setMinimumSize(QSize(0, 0));
        m_button_set_mutes->setMaximumSize(QSize(16, 16));

        gridLayout_2->addWidget(m_button_set_mutes, 1, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_2);


        horizontalLayout_3->addLayout(verticalLayout);

        QWidget::setTabOrder(m_group_table, m_check_from_midi);
        QWidget::setTabOrder(m_check_from_midi, m_radio_binary);
        QWidget::setTabOrder(m_radio_binary, m_check_to_midi);
        QWidget::setTabOrder(m_check_to_midi, m_check_from_mutes);
        QWidget::setTabOrder(m_check_from_mutes, m_radio_hex);
        QWidget::setTabOrder(m_radio_hex, m_check_to_mutes);
        QWidget::setTabOrder(m_check_to_mutes, m_check_toggle_active);
        QWidget::setTabOrder(m_check_toggle_active, m_check_strip_empty);
        QWidget::setTabOrder(m_check_strip_empty, m_button_trigger);
        QWidget::setTabOrder(m_button_trigger, m_button_clear_all);
        QWidget::setTabOrder(m_button_clear_all, m_button_save);
        QWidget::setTabOrder(m_button_save, m_button_load);
        QWidget::setTabOrder(m_button_load, m_button_reserved);
        QWidget::setTabOrder(m_button_reserved, m_button_set_mutes);
        QWidget::setTabOrder(m_button_set_mutes, m_check_mutes_active);
        QWidget::setTabOrder(m_check_mutes_active, m_mute_basename);

        retranslateUi(qmutemaster);

        QMetaObject::connectSlotsByName(qmutemaster);
    } // setupUi

    void retranslateUi(QFrame *qmutemaster)
    {
        qmutemaster->setWindowTitle(QCoreApplication::translate("qmutemaster", "Mute Master", nullptr));
#if QT_CONFIG(tooltip)
        qmutemaster->setToolTip(QCoreApplication::translate("qmutemaster", "Mute Master: viewing/editing of mute groups.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        qmutemaster->setWhatsThis(QCoreApplication::translate("qmutemaster", "The Mute Master", nullptr));
#endif // QT_CONFIG(whatsthis)
        label_4->setText(QCoreApplication::translate("qmutemaster", "Mute-Groups", nullptr));
        label_3->setText(QCoreApplication::translate("qmutemaster", "Click group row to show its layout ", nullptr));
        label_5->setText(QCoreApplication::translate("qmutemaster", "Group Patterns", nullptr));
        m_mutes_file_label->setText(QCoreApplication::translate("qmutemaster", "Mutes File", nullptr));
#if QT_CONFIG(tooltip)
        m_mute_basename->setToolTip(QCoreApplication::translate("qmutemaster", "For reference only. Go to Edit / Preferences / Session\n"
"to edit this name.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        m_check_mutes_active->setToolTip(QCoreApplication::translate("qmutemaster", "Indicates if the 'mutes' file is active. See\n"
"Edit / Preferences / Session / .mutes.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_mutes_active->setText(QCoreApplication::translate("qmutemaster", "Active", nullptr));
        label_2->setText(QCoreApplication::translate("qmutemaster", "Read/Write\n"
"and Format", nullptr));
#if QT_CONFIG(tooltip)
        m_check_to_midi->setToolTip(QCoreApplication::translate("qmutemaster", "Mute-groups are saved in the MIDI file.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_to_midi->setText(QCoreApplication::translate("qmutemaster", "To MIDI ", nullptr));
#if QT_CONFIG(tooltip)
        m_check_to_mutes->setToolTip(QCoreApplication::translate("qmutemaster", "Mute-groups are saved to the\n"
"\"mutes\" file at exit.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_to_mutes->setText(QCoreApplication::translate("qmutemaster", "To Mutes", nullptr));
#if QT_CONFIG(tooltip)
        m_radio_hex->setToolTip(QCoreApplication::translate("qmutemaster", "Mute statuses are saved as hexadecimal (e.g. 0xFF).\n"
"Useful if set-size is larger than 4 x 8.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_radio_hex->setText(QCoreApplication::translate("qmutemaster", "He&x", nullptr));
#if QT_CONFIG(tooltip)
        m_radio_binary->setToolTip(QCoreApplication::translate("qmutemaster", "Mute statuses are saved as stanzas of 1's\n"
"and 0's. Default setting.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_radio_binary->setText(QCoreApplication::translate("qmutemaster", "&Binary", nullptr));
#if QT_CONFIG(tooltip)
        m_check_from_midi->setToolTip(QCoreApplication::translate("qmutemaster", "If checked, read mute-groups from the MIDI\n"
"file. Takes precedence over the 'mutes' file.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_from_midi->setText(QCoreApplication::translate("qmutemaster", "From MIDI", nullptr));
#if QT_CONFIG(tooltip)
        m_check_from_mutes->setToolTip(QCoreApplication::translate("qmutemaster", "If checked, the mute-groups are read from\n"
"a common 'mutes' file. Superceded by\n"
"\"From MIDI\".", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_from_mutes->setText(QCoreApplication::translate("qmutemaster", "From M&utes", nullptr));
#if QT_CONFIG(tooltip)
        m_check_toggle_active->setToolTip(QCoreApplication::translate("qmutemaster", "If checked, toggling a mute-group changes only patterns\n"
"specified in that group; patterns armed by the user stay on.\n"
"Legacy behavior toggles mute-group and armed patterns.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_toggle_active->setText(QCoreApplication::translate("qmutemaster", "Toggle &Only Active", nullptr));
#if QT_CONFIG(tooltip)
        m_check_strip_empty->setToolTip(QCoreApplication::translate("qmutemaster", "If there are no non-zero mute-group values,\n"
"do not save them to the MIDI file.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_check_strip_empty->setText(QCoreApplication::translate("qmutemaster", "Strip &If No Mutes", nullptr));
#if QT_CONFIG(tooltip)
        m_button_trigger->setToolTip(QCoreApplication::translate("qmutemaster", "When active, the mute-group button here\n"
"can toggle the mute-groups.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_trigger->setText(QCoreApplication::translate("qmutemaster", "&Triggers", nullptr));
#if QT_CONFIG(tooltip)
        m_button_clear_all->setToolTip(QCoreApplication::translate("qmutemaster", "Removes all mute-groups from the mute-group setup.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_clear_all->setText(QCoreApplication::translate("qmutemaster", "&Zero Out", nullptr));
#if QT_CONFIG(tooltip)
        m_button_reserved->setToolTip(QCoreApplication::translate("qmutemaster", "Reserved for expansion.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_reserved->setText(QString());
#if QT_CONFIG(tooltip)
        m_button_save->setToolTip(QCoreApplication::translate("qmutemaster", "Saves mute groups to a file in the session\n"
"configuration.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_save->setText(QCoreApplication::translate("qmutemaster", "Save &File", nullptr));
#if QT_CONFIG(tooltip)
        m_button_load->setToolTip(QCoreApplication::translate("qmutemaster", "Present a dialog to load a 'mutes' file.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_load->setText(QCoreApplication::translate("qmutemaster", "&Load File", nullptr));
#if QT_CONFIG(tooltip)
        m_button_set_mutes->setToolTip(QCoreApplication::translate("qmutemaster", "A star if mute-group modifications affecting the\n"
"mutes file are present.", nullptr));
#endif // QT_CONFIG(tooltip)
        m_button_set_mutes->setText(QCoreApplication::translate("qmutemaster", " - ", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qmutemaster: public Ui_qmutemaster {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QMUTEMASTER_H
