/********************************************************************************
** Form generated from reading UI file 'qliveframeex.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QLIVEFRAMEEX_H
#define QLIVEFRAMEEX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qliveframeex
{
public:

    void setupUi(QWidget *qliveframeex)
    {
        if (qliveframeex->objectName().isEmpty())
            qliveframeex->setObjectName(QString::fromUtf8("qliveframeex"));
        qliveframeex->resize(774, 440);
        qliveframeex->setMinimumSize(QSize(680, 280));
        QFont font;
        font.setPointSize(11);
        qliveframeex->setFont(font);

        retranslateUi(qliveframeex);

        QMetaObject::connectSlotsByName(qliveframeex);
    } // setupUi

    void retranslateUi(QWidget *qliveframeex)
    {
        qliveframeex->setWindowTitle(QCoreApplication::translate("qliveframeex", "Live Grid", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qliveframeex: public Ui_qliveframeex {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QLIVEFRAMEEX_H
