/********************************************************************************
** Form generated from reading UI file 'qsmainwnd.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSMAINWND_H
#define QSMAINWND_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsmainwnd
{
public:
    QAction *actionNew;
    QAction *actionImportMIDI;
    QAction *actionImportProject;
    QAction *actionOpen;
    QAction *actionOpenPlaylist;
    QAction *actionRecent;
    QAction *actionPreferences;
    QAction *actionSave;
    QAction *actionSave_As;
    QAction *actionQuit;
    QAction *actionAbout;
    QAction *actionBuildInfo;
    QAction *actionAppInfo;
    QAction *actionAbout_Qt;
    QAction *actionTools;
    QAction *actionExportSong;
    QAction *actionExportMIDI;
    QAction *actionExportSMF0;
    QAction *actionSongEditor;
    QAction *actionSongTranspose;
    QAction *actionClearMuteGroups;
    QAction *actionReloadMuteGroups;
    QAction *actionMuteAllTracks;
    QAction *actionUnmuteAllTracks;
    QAction *actionToggleAllTracks;
    QAction *actionClose;
    QAction *actionSongSummary;
    QAction *actionCopyCurrentSet;
    QAction *actionPasteToCurrentSet;
    QAction *actionImportPlaylist;
    QAction *actionImportMIDIIntoSession;
    QAction *actionTutorial;
    QAction *actionUserManual;
    QAction *actionExportProject;
    QAction *actionLogView;
    QWidget *centralWidget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hLayoutTop;
    QPushButton *btnSeq66;
    QPushButton *btnShowHide;
    QPushButton *smf0Button;
    QPushButton *alsaJackButton;
    QPushButton *jackTransportButton;
    QComboBox *cmb_ppqn;
    QComboBox *cmb_global_bus;
    QComboBox *cmb_beat_measure;
    QLabel *label_ts_slash;
    QComboBox *cmb_beat_length;
    QPushButton *btnBBTHMS;
    QLabel *label_test;
    QWidget *verticalWidget;
    QVBoxLayout *layout_beat_ind;
    QTabWidget *tabWidget;
    QWidget *LiveTab;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *LiveTabLayout;
    QWidget *SongTab;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *SongTabLayout;
    QWidget *EditTab;
    QVBoxLayout *verticalLayout_4;
    QVBoxLayout *EditTabLayout;
    QWidget *EventTab;
    QWidget *verticalLayoutWidget;
    QVBoxLayout *EventTabLayout;
    QWidget *PlaylistTab;
    QWidget *verticalLayoutWidget_2;
    QVBoxLayout *PlaylistTabLayout;
    QWidget *SetMasterTab;
    QWidget *verticalLayoutWidget_3;
    QVBoxLayout *SetsTabLayout;
    QWidget *MuteMasterTab;
    QWidget *verticalLayoutWidget_4;
    QVBoxLayout *MutesTabLayout;
    QWidget *SessionTab;
    QWidget *verticalLayoutWidget_5;
    QVBoxLayout *SessionTabLayout;
    QHBoxLayout *hLayoutBottom_1;
    QLabel *setNameLabel;
    QLineEdit *txtBankName;
    QLabel *label;
    QLineEdit *txtMuteName;
    QLineEdit *txtUnderrun;
    QLabel *labelActiveSet;
    QLineEdit *entry_active_set;
    QPushButton *setHomeButton;
    QPushButton *setResetButton;
    QLabel *setNumberLabel;
    QSpinBox *spinBank;
    QHBoxLayout *hLayoutBottom_2;
    QHBoxLayout *lay_transport;
    QPushButton *btnPanic;
    QPushButton *btnStop;
    QPushButton *btnPause;
    QPushButton *btnPlay;
    QPushButton *btnRecordEx;
    QPushButton *btnLoop;
    QPushButton *btnRecord;
    QPushButton *button_keep_queue;
    QPushButton *button_learn;
    QPushButton *testButton;
    QPushButton *btnMute;
    QPushButton *btnPerfEdit;
    QPushButton *btnSongPlay;
    QLineEdit *lineEditPpqn;
    QHBoxLayout *lay_bpm;
    QPushButton *button_tap_bpm;
    QLabel *labelBPM;
    QDoubleSpinBox *spinBpm;
    QMenuBar *menuBar;
    QMenu *menuFile;
    QMenu *menuExport;
    QMenu *menuImport;
    QMenu *menuAbout;
    QMenu *menuEdit;

    void setupUi(QMainWindow *qsmainwnd)
    {
        if (qsmainwnd->objectName().isEmpty())
            qsmainwnd->setObjectName(QString::fromUtf8("qsmainwnd"));
        qsmainwnd->resize(900, 618);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qsmainwnd->sizePolicy().hasHeightForWidth());
        qsmainwnd->setSizePolicy(sizePolicy);
        qsmainwnd->setMinimumSize(QSize(720, 480));
        QFont font;
        font.setPointSize(12);
        font.setStyleStrategy(QFont::PreferDefault);
        qsmainwnd->setFont(font);
        qsmainwnd->setCursor(QCursor(Qt::ArrowCursor));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/pixmaps/logo.xpm"), QSize(), QIcon::Normal, QIcon::Off);
        qsmainwnd->setWindowIcon(icon);
        actionNew = new QAction(qsmainwnd);
        actionNew->setObjectName(QString::fromUtf8("actionNew"));
        actionImportMIDI = new QAction(qsmainwnd);
        actionImportMIDI->setObjectName(QString::fromUtf8("actionImportMIDI"));
        actionImportProject = new QAction(qsmainwnd);
        actionImportProject->setObjectName(QString::fromUtf8("actionImportProject"));
        actionOpen = new QAction(qsmainwnd);
        actionOpen->setObjectName(QString::fromUtf8("actionOpen"));
        actionOpenPlaylist = new QAction(qsmainwnd);
        actionOpenPlaylist->setObjectName(QString::fromUtf8("actionOpenPlaylist"));
        actionRecent = new QAction(qsmainwnd);
        actionRecent->setObjectName(QString::fromUtf8("actionRecent"));
        actionPreferences = new QAction(qsmainwnd);
        actionPreferences->setObjectName(QString::fromUtf8("actionPreferences"));
        actionPreferences->setShortcutContext(Qt::ShortcutContext::ApplicationShortcut);
        actionSave = new QAction(qsmainwnd);
        actionSave->setObjectName(QString::fromUtf8("actionSave"));
        actionSave_As = new QAction(qsmainwnd);
        actionSave_As->setObjectName(QString::fromUtf8("actionSave_As"));
        actionQuit = new QAction(qsmainwnd);
        actionQuit->setObjectName(QString::fromUtf8("actionQuit"));
        actionAbout = new QAction(qsmainwnd);
        actionAbout->setObjectName(QString::fromUtf8("actionAbout"));
        actionBuildInfo = new QAction(qsmainwnd);
        actionBuildInfo->setObjectName(QString::fromUtf8("actionBuildInfo"));
        actionAppInfo = new QAction(qsmainwnd);
        actionAppInfo->setObjectName(QString::fromUtf8("actionAppInfo"));
        actionAbout_Qt = new QAction(qsmainwnd);
        actionAbout_Qt->setObjectName(QString::fromUtf8("actionAbout_Qt"));
        actionTools = new QAction(qsmainwnd);
        actionTools->setObjectName(QString::fromUtf8("actionTools"));
        actionExportSong = new QAction(qsmainwnd);
        actionExportSong->setObjectName(QString::fromUtf8("actionExportSong"));
        actionExportMIDI = new QAction(qsmainwnd);
        actionExportMIDI->setObjectName(QString::fromUtf8("actionExportMIDI"));
        actionExportSMF0 = new QAction(qsmainwnd);
        actionExportSMF0->setObjectName(QString::fromUtf8("actionExportSMF0"));
        actionSongEditor = new QAction(qsmainwnd);
        actionSongEditor->setObjectName(QString::fromUtf8("actionSongEditor"));
        actionSongTranspose = new QAction(qsmainwnd);
        actionSongTranspose->setObjectName(QString::fromUtf8("actionSongTranspose"));
        actionClearMuteGroups = new QAction(qsmainwnd);
        actionClearMuteGroups->setObjectName(QString::fromUtf8("actionClearMuteGroups"));
        actionReloadMuteGroups = new QAction(qsmainwnd);
        actionReloadMuteGroups->setObjectName(QString::fromUtf8("actionReloadMuteGroups"));
        actionMuteAllTracks = new QAction(qsmainwnd);
        actionMuteAllTracks->setObjectName(QString::fromUtf8("actionMuteAllTracks"));
        actionUnmuteAllTracks = new QAction(qsmainwnd);
        actionUnmuteAllTracks->setObjectName(QString::fromUtf8("actionUnmuteAllTracks"));
        actionToggleAllTracks = new QAction(qsmainwnd);
        actionToggleAllTracks->setObjectName(QString::fromUtf8("actionToggleAllTracks"));
        actionClose = new QAction(qsmainwnd);
        actionClose->setObjectName(QString::fromUtf8("actionClose"));
        actionSongSummary = new QAction(qsmainwnd);
        actionSongSummary->setObjectName(QString::fromUtf8("actionSongSummary"));
        actionCopyCurrentSet = new QAction(qsmainwnd);
        actionCopyCurrentSet->setObjectName(QString::fromUtf8("actionCopyCurrentSet"));
        actionPasteToCurrentSet = new QAction(qsmainwnd);
        actionPasteToCurrentSet->setObjectName(QString::fromUtf8("actionPasteToCurrentSet"));
        actionImportPlaylist = new QAction(qsmainwnd);
        actionImportPlaylist->setObjectName(QString::fromUtf8("actionImportPlaylist"));
        actionImportMIDIIntoSession = new QAction(qsmainwnd);
        actionImportMIDIIntoSession->setObjectName(QString::fromUtf8("actionImportMIDIIntoSession"));
        actionTutorial = new QAction(qsmainwnd);
        actionTutorial->setObjectName(QString::fromUtf8("actionTutorial"));
        actionUserManual = new QAction(qsmainwnd);
        actionUserManual->setObjectName(QString::fromUtf8("actionUserManual"));
        actionExportProject = new QAction(qsmainwnd);
        actionExportProject->setObjectName(QString::fromUtf8("actionExportProject"));
        actionLogView = new QAction(qsmainwnd);
        actionLogView->setObjectName(QString::fromUtf8("actionLogView"));
        centralWidget = new QWidget(qsmainwnd);
        centralWidget->setObjectName(QString::fromUtf8("centralWidget"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(centralWidget->sizePolicy().hasHeightForWidth());
        centralWidget->setSizePolicy(sizePolicy1);
        centralWidget->setSizeIncrement(QSize(1, 1));
        centralWidget->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        verticalLayout = new QVBoxLayout(centralWidget);
        verticalLayout->setSpacing(4);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(4, 4, 4, 4);
        hLayoutTop = new QHBoxLayout();
        hLayoutTop->setSpacing(6);
        hLayoutTop->setObjectName(QString::fromUtf8("hLayoutTop"));
        btnSeq66 = new QPushButton(centralWidget);
        btnSeq66->setObjectName(QString::fromUtf8("btnSeq66"));
        btnSeq66->setEnabled(true);
        btnSeq66->setMinimumSize(QSize(60, 24));
        QFont font1;
        font1.setPointSize(12);
        font1.setBold(true);
        font1.setStyleStrategy(QFont::PreferDefault);
        btnSeq66->setFont(font1);
        btnSeq66->setFlat(true);

        hLayoutTop->addWidget(btnSeq66);

        btnShowHide = new QPushButton(centralWidget);
        btnShowHide->setObjectName(QString::fromUtf8("btnShowHide"));
        btnShowHide->setMinimumSize(QSize(14, 24));
        btnShowHide->setMaximumSize(QSize(20, 24));

        hLayoutTop->addWidget(btnShowHide);

        smf0Button = new QPushButton(centralWidget);
        smf0Button->setObjectName(QString::fromUtf8("smf0Button"));
        smf0Button->setEnabled(false);
        smf0Button->setMinimumSize(QSize(48, 0));
        smf0Button->setMaximumSize(QSize(48, 16777215));
        QFont font2;
        font2.setPointSize(11);
        font2.setItalic(true);
        font2.setStyleStrategy(QFont::PreferDefault);
        smf0Button->setFont(font2);
        smf0Button->setFlat(true);

        hLayoutTop->addWidget(smf0Button);

        alsaJackButton = new QPushButton(centralWidget);
        alsaJackButton->setObjectName(QString::fromUtf8("alsaJackButton"));
        alsaJackButton->setEnabled(false);
        alsaJackButton->setMinimumSize(QSize(72, 24));
        alsaJackButton->setMaximumSize(QSize(84, 24));
        alsaJackButton->setFont(font2);
        alsaJackButton->setFlat(true);

        hLayoutTop->addWidget(alsaJackButton);

        jackTransportButton = new QPushButton(centralWidget);
        jackTransportButton->setObjectName(QString::fromUtf8("jackTransportButton"));
        jackTransportButton->setEnabled(false);
        jackTransportButton->setMinimumSize(QSize(56, 24));
        jackTransportButton->setMaximumSize(QSize(56, 24));
        jackTransportButton->setFont(font2);
        jackTransportButton->setFlat(true);

        hLayoutTop->addWidget(jackTransportButton);

        cmb_ppqn = new QComboBox(centralWidget);
        cmb_ppqn->setObjectName(QString::fromUtf8("cmb_ppqn"));
        cmb_ppqn->setEnabled(true);
        cmb_ppqn->setMinimumSize(QSize(80, 28));
        cmb_ppqn->setMaximumSize(QSize(92, 28));
        cmb_ppqn->setEditable(false);
        cmb_ppqn->setCurrentText(QString::fromUtf8(""));
        cmb_ppqn->setMaxVisibleItems(20);
        cmb_ppqn->setInsertPolicy(QComboBox::InsertPolicy::NoInsert);

        hLayoutTop->addWidget(cmb_ppqn);

        cmb_global_bus = new QComboBox(centralWidget);
        cmb_global_bus->setObjectName(QString::fromUtf8("cmb_global_bus"));
        cmb_global_bus->setEnabled(true);
        cmb_global_bus->setMinimumSize(QSize(148, 28));
        cmb_global_bus->setMaximumSize(QSize(164, 28));
        cmb_global_bus->setMaxVisibleItems(20);

        hLayoutTop->addWidget(cmb_global_bus);

        cmb_beat_measure = new QComboBox(centralWidget);
        cmb_beat_measure->setObjectName(QString::fromUtf8("cmb_beat_measure"));
        QSizePolicy sizePolicy2(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(cmb_beat_measure->sizePolicy().hasHeightForWidth());
        cmb_beat_measure->setSizePolicy(sizePolicy2);
        cmb_beat_measure->setMinimumSize(QSize(60, 28));
        cmb_beat_measure->setMaximumSize(QSize(64, 28));
        cmb_beat_measure->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        cmb_beat_measure->setMaxVisibleItems(17);

        hLayoutTop->addWidget(cmb_beat_measure);

        label_ts_slash = new QLabel(centralWidget);
        label_ts_slash->setObjectName(QString::fromUtf8("label_ts_slash"));
        label_ts_slash->setMinimumSize(QSize(6, 24));
        label_ts_slash->setMaximumSize(QSize(6, 24));
        label_ts_slash->setFont(font1);
        label_ts_slash->setAlignment(Qt::AlignmentFlag::AlignCenter);

        hLayoutTop->addWidget(label_ts_slash);

        cmb_beat_length = new QComboBox(centralWidget);
        cmb_beat_length->setObjectName(QString::fromUtf8("cmb_beat_length"));
        sizePolicy2.setHeightForWidth(cmb_beat_length->sizePolicy().hasHeightForWidth());
        cmb_beat_length->setSizePolicy(sizePolicy2);
        cmb_beat_length->setMinimumSize(QSize(60, 28));
        cmb_beat_length->setMaximumSize(QSize(64, 28));
        cmb_beat_length->setFocusPolicy(Qt::FocusPolicy::NoFocus);

        hLayoutTop->addWidget(cmb_beat_length);

        btnBBTHMS = new QPushButton(centralWidget);
        btnBBTHMS->setObjectName(QString::fromUtf8("btnBBTHMS"));
        btnBBTHMS->setMinimumSize(QSize(86, 0));
        btnBBTHMS->setFont(font1);

        hLayoutTop->addWidget(btnBBTHMS);

        label_test = new QLabel(centralWidget);
        label_test->setObjectName(QString::fromUtf8("label_test"));
        label_test->setMaximumSize(QSize(32, 16777215));

        hLayoutTop->addWidget(label_test);

        verticalWidget = new QWidget(centralWidget);
        verticalWidget->setObjectName(QString::fromUtf8("verticalWidget"));
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(verticalWidget->sizePolicy().hasHeightForWidth());
        verticalWidget->setSizePolicy(sizePolicy3);
        verticalWidget->setMinimumSize(QSize(80, 18));
        verticalWidget->setMaximumSize(QSize(180, 18));
        layout_beat_ind = new QVBoxLayout(verticalWidget);
        layout_beat_ind->setSpacing(0);
        layout_beat_ind->setContentsMargins(11, 11, 11, 11);
        layout_beat_ind->setObjectName(QString::fromUtf8("layout_beat_ind"));
        layout_beat_ind->setSizeConstraint(QLayout::SizeConstraint::SetMaximumSize);
        layout_beat_ind->setContentsMargins(0, 0, 0, 0);

        hLayoutTop->addWidget(verticalWidget);


        verticalLayout->addLayout(hLayoutTop);

        tabWidget = new QTabWidget(centralWidget);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        QSizePolicy sizePolicy4(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy4.setHorizontalStretch(1);
        sizePolicy4.setVerticalStretch(1);
        sizePolicy4.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy4);
        tabWidget->setSizeIncrement(QSize(1, 1));
        QFont font3;
        font3.setPointSize(11);
        font3.setBold(true);
        font3.setStyleStrategy(QFont::PreferDefault);
        tabWidget->setFont(font3);
        tabWidget->setFocusPolicy(Qt::FocusPolicy::TabFocus);
        tabWidget->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        tabWidget->setTabShape(QTabWidget::TabShape::Rounded);
        tabWidget->setMovable(false);
        LiveTab = new QWidget();
        LiveTab->setObjectName(QString::fromUtf8("LiveTab"));
        sizePolicy1.setHeightForWidth(LiveTab->sizePolicy().hasHeightForWidth());
        LiveTab->setSizePolicy(sizePolicy1);
        LiveTab->setSizeIncrement(QSize(1, 1));
        LiveTab->setAutoFillBackground(false);
        verticalLayout_2 = new QVBoxLayout(LiveTab);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        LiveTabLayout = new QVBoxLayout();
        LiveTabLayout->setSpacing(6);
        LiveTabLayout->setObjectName(QString::fromUtf8("LiveTabLayout"));
        LiveTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        LiveTabLayout->setContentsMargins(0, -1, -1, -1);

        verticalLayout_2->addLayout(LiveTabLayout);

        tabWidget->addTab(LiveTab, QString());
        SongTab = new QWidget();
        SongTab->setObjectName(QString::fromUtf8("SongTab"));
        sizePolicy1.setHeightForWidth(SongTab->sizePolicy().hasHeightForWidth());
        SongTab->setSizePolicy(sizePolicy1);
        SongTab->setSizeIncrement(QSize(1, 1));
        verticalLayout_3 = new QVBoxLayout(SongTab);
        verticalLayout_3->setSpacing(6);
        verticalLayout_3->setContentsMargins(11, 11, 11, 11);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        SongTabLayout = new QVBoxLayout();
        SongTabLayout->setSpacing(6);
        SongTabLayout->setObjectName(QString::fromUtf8("SongTabLayout"));
        SongTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        SongTabLayout->setContentsMargins(0, -1, -1, -1);

        verticalLayout_3->addLayout(SongTabLayout);

        tabWidget->addTab(SongTab, QString());
        EditTab = new QWidget();
        EditTab->setObjectName(QString::fromUtf8("EditTab"));
        sizePolicy1.setHeightForWidth(EditTab->sizePolicy().hasHeightForWidth());
        EditTab->setSizePolicy(sizePolicy1);
        EditTab->setSizeIncrement(QSize(1, 1));
        EditTab->setBaseSize(QSize(0, 0));
        verticalLayout_4 = new QVBoxLayout(EditTab);
        verticalLayout_4->setSpacing(6);
        verticalLayout_4->setContentsMargins(11, 11, 11, 11);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        EditTabLayout = new QVBoxLayout();
        EditTabLayout->setSpacing(6);
        EditTabLayout->setObjectName(QString::fromUtf8("EditTabLayout"));
        EditTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetFixedSize);
        EditTabLayout->setContentsMargins(0, -1, -1, -1);

        verticalLayout_4->addLayout(EditTabLayout);

        tabWidget->addTab(EditTab, QString());
        EventTab = new QWidget();
        EventTab->setObjectName(QString::fromUtf8("EventTab"));
        QSizePolicy sizePolicy5(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(EventTab->sizePolicy().hasHeightForWidth());
        EventTab->setSizePolicy(sizePolicy5);
        verticalLayoutWidget = new QWidget(EventTab);
        verticalLayoutWidget->setObjectName(QString::fromUtf8("verticalLayoutWidget"));
        verticalLayoutWidget->setGeometry(QRect(10, 10, 821, 511));
        EventTabLayout = new QVBoxLayout(verticalLayoutWidget);
        EventTabLayout->setSpacing(6);
        EventTabLayout->setContentsMargins(11, 11, 11, 11);
        EventTabLayout->setObjectName(QString::fromUtf8("EventTabLayout"));
        EventTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetMaximumSize);
        EventTabLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(EventTab, QString());
        PlaylistTab = new QWidget();
        PlaylistTab->setObjectName(QString::fromUtf8("PlaylistTab"));
        QSizePolicy sizePolicy6(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(PlaylistTab->sizePolicy().hasHeightForWidth());
        PlaylistTab->setSizePolicy(sizePolicy6);
        verticalLayoutWidget_2 = new QWidget(PlaylistTab);
        verticalLayoutWidget_2->setObjectName(QString::fromUtf8("verticalLayoutWidget_2"));
        verticalLayoutWidget_2->setGeometry(QRect(10, 10, 821, 511));
        PlaylistTabLayout = new QVBoxLayout(verticalLayoutWidget_2);
        PlaylistTabLayout->setSpacing(6);
        PlaylistTabLayout->setContentsMargins(11, 11, 11, 11);
        PlaylistTabLayout->setObjectName(QString::fromUtf8("PlaylistTabLayout"));
        PlaylistTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        PlaylistTabLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(PlaylistTab, QString());
        SetMasterTab = new QWidget();
        SetMasterTab->setObjectName(QString::fromUtf8("SetMasterTab"));
        sizePolicy6.setHeightForWidth(SetMasterTab->sizePolicy().hasHeightForWidth());
        SetMasterTab->setSizePolicy(sizePolicy6);
        verticalLayoutWidget_3 = new QWidget(SetMasterTab);
        verticalLayoutWidget_3->setObjectName(QString::fromUtf8("verticalLayoutWidget_3"));
        verticalLayoutWidget_3->setGeometry(QRect(10, 8, 821, 513));
        SetsTabLayout = new QVBoxLayout(verticalLayoutWidget_3);
        SetsTabLayout->setSpacing(6);
        SetsTabLayout->setContentsMargins(11, 11, 11, 11);
        SetsTabLayout->setObjectName(QString::fromUtf8("SetsTabLayout"));
        SetsTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        SetsTabLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(SetMasterTab, QString());
        MuteMasterTab = new QWidget();
        MuteMasterTab->setObjectName(QString::fromUtf8("MuteMasterTab"));
        sizePolicy6.setHeightForWidth(MuteMasterTab->sizePolicy().hasHeightForWidth());
        MuteMasterTab->setSizePolicy(sizePolicy6);
        verticalLayoutWidget_4 = new QWidget(MuteMasterTab);
        verticalLayoutWidget_4->setObjectName(QString::fromUtf8("verticalLayoutWidget_4"));
        verticalLayoutWidget_4->setGeometry(QRect(10, 10, 821, 511));
        MutesTabLayout = new QVBoxLayout(verticalLayoutWidget_4);
        MutesTabLayout->setSpacing(6);
        MutesTabLayout->setContentsMargins(11, 11, 11, 11);
        MutesTabLayout->setObjectName(QString::fromUtf8("MutesTabLayout"));
        MutesTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        MutesTabLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(MuteMasterTab, QString());
        SessionTab = new QWidget();
        SessionTab->setObjectName(QString::fromUtf8("SessionTab"));
        sizePolicy6.setHeightForWidth(SessionTab->sizePolicy().hasHeightForWidth());
        SessionTab->setSizePolicy(sizePolicy6);
        verticalLayoutWidget_5 = new QWidget(SessionTab);
        verticalLayoutWidget_5->setObjectName(QString::fromUtf8("verticalLayoutWidget_5"));
        verticalLayoutWidget_5->setGeometry(QRect(10, 10, 821, 511));
        SessionTabLayout = new QVBoxLayout(verticalLayoutWidget_5);
        SessionTabLayout->setSpacing(6);
        SessionTabLayout->setContentsMargins(11, 11, 11, 11);
        SessionTabLayout->setObjectName(QString::fromUtf8("SessionTabLayout"));
        SessionTabLayout->setSizeConstraint(QLayout::SizeConstraint::SetDefaultConstraint);
        SessionTabLayout->setContentsMargins(0, 0, 0, 0);
        tabWidget->addTab(SessionTab, QString());

        verticalLayout->addWidget(tabWidget);

        hLayoutBottom_1 = new QHBoxLayout();
        hLayoutBottom_1->setSpacing(6);
        hLayoutBottom_1->setObjectName(QString::fromUtf8("hLayoutBottom_1"));
        setNameLabel = new QLabel(centralWidget);
        setNameLabel->setObjectName(QString::fromUtf8("setNameLabel"));
        setNameLabel->setMinimumSize(QSize(76, 24));
        setNameLabel->setMaximumSize(QSize(16777215, 28));
        setNameLabel->setFont(font1);

        hLayoutBottom_1->addWidget(setNameLabel);

        txtBankName = new QLineEdit(centralWidget);
        txtBankName->setObjectName(QString::fromUtf8("txtBankName"));
        txtBankName->setMinimumSize(QSize(240, 24));
        txtBankName->setMaximumSize(QSize(240, 28));
        QFont font4;
        font4.setPointSize(12);
        font4.setBold(false);
        font4.setStyleStrategy(QFont::PreferDefault);
        txtBankName->setFont(font4);
        txtBankName->setMaxLength(128);

        hLayoutBottom_1->addWidget(txtBankName);

        label = new QLabel(centralWidget);
        label->setObjectName(QString::fromUtf8("label"));
        label->setMinimumSize(QSize(0, 24));
        label->setMaximumSize(QSize(16777215, 28));
        label->setFont(font1);

        hLayoutBottom_1->addWidget(label);

        txtMuteName = new QLineEdit(centralWidget);
        txtMuteName->setObjectName(QString::fromUtf8("txtMuteName"));
        txtMuteName->setMinimumSize(QSize(108, 24));
        txtMuteName->setMaximumSize(QSize(16777215, 28));
        txtMuteName->setReadOnly(true);

        hLayoutBottom_1->addWidget(txtMuteName);

        txtUnderrun = new QLineEdit(centralWidget);
        txtUnderrun->setObjectName(QString::fromUtf8("txtUnderrun"));
        txtUnderrun->setEnabled(false);
        txtUnderrun->setMinimumSize(QSize(40, 24));
        txtUnderrun->setMaximumSize(QSize(44, 28));
        QFont font5;
        font5.setPointSize(9);
        font5.setBold(false);
        font5.setStyleStrategy(QFont::PreferDefault);
        txtUnderrun->setFont(font5);
        txtUnderrun->setAlignment(Qt::AlignmentFlag::AlignCenter);

        hLayoutBottom_1->addWidget(txtUnderrun);

        labelActiveSet = new QLabel(centralWidget);
        labelActiveSet->setObjectName(QString::fromUtf8("labelActiveSet"));
        labelActiveSet->setMinimumSize(QSize(60, 24));
        labelActiveSet->setMaximumSize(QSize(60, 28));
        labelActiveSet->setFont(font1);
        labelActiveSet->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        hLayoutBottom_1->addWidget(labelActiveSet);

        entry_active_set = new QLineEdit(centralWidget);
        entry_active_set->setObjectName(QString::fromUtf8("entry_active_set"));
        entry_active_set->setEnabled(false);
        entry_active_set->setMinimumSize(QSize(56, 24));
        entry_active_set->setMaximumSize(QSize(64, 28));
        entry_active_set->setFont(font4);
        entry_active_set->setReadOnly(true);

        hLayoutBottom_1->addWidget(entry_active_set);

        setHomeButton = new QPushButton(centralWidget);
        setHomeButton->setObjectName(QString::fromUtf8("setHomeButton"));
        setHomeButton->setMinimumSize(QSize(60, 24));
        setHomeButton->setMaximumSize(QSize(60, 28));
        setHomeButton->setFont(font1);

        hLayoutBottom_1->addWidget(setHomeButton);

        setResetButton = new QPushButton(centralWidget);
        setResetButton->setObjectName(QString::fromUtf8("setResetButton"));
        setResetButton->setMinimumSize(QSize(24, 24));
        setResetButton->setMaximumSize(QSize(24, 28));
        setResetButton->setFont(font1);

        hLayoutBottom_1->addWidget(setResetButton);

        setNumberLabel = new QLabel(centralWidget);
        setNumberLabel->setObjectName(QString::fromUtf8("setNumberLabel"));
        setNumberLabel->setMinimumSize(QSize(32, 24));
        setNumberLabel->setMaximumSize(QSize(40, 28));
        setNumberLabel->setFont(font3);
        setNumberLabel->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        hLayoutBottom_1->addWidget(setNumberLabel);

        spinBank = new QSpinBox(centralWidget);
        spinBank->setObjectName(QString::fromUtf8("spinBank"));
        spinBank->setMinimumSize(QSize(80, 24));
        spinBank->setMaximumSize(QSize(80, 28));
        spinBank->setFont(font4);

        hLayoutBottom_1->addWidget(spinBank);

        hLayoutBottom_1->setStretch(0, 1);
        hLayoutBottom_1->setStretch(1, 20);
        hLayoutBottom_1->setStretch(4, 1);
        hLayoutBottom_1->setStretch(5, 1);
        hLayoutBottom_1->setStretch(6, 1);
        hLayoutBottom_1->setStretch(10, 1);

        verticalLayout->addLayout(hLayoutBottom_1);

        hLayoutBottom_2 = new QHBoxLayout();
        hLayoutBottom_2->setSpacing(6);
        hLayoutBottom_2->setObjectName(QString::fromUtf8("hLayoutBottom_2"));
        lay_transport = new QHBoxLayout();
        lay_transport->setSpacing(6);
        lay_transport->setObjectName(QString::fromUtf8("lay_transport"));
        btnPanic = new QPushButton(centralWidget);
        btnPanic->setObjectName(QString::fromUtf8("btnPanic"));
        QSizePolicy sizePolicy7(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(btnPanic->sizePolicy().hasHeightForWidth());
        btnPanic->setSizePolicy(sizePolicy7);
        btnPanic->setMinimumSize(QSize(32, 24));
        btnPanic->setMaximumSize(QSize(48, 28));
        btnPanic->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnPanic->setIconSize(QSize(16, 16));
        btnPanic->setCheckable(false);

        lay_transport->addWidget(btnPanic);

        btnStop = new QPushButton(centralWidget);
        btnStop->setObjectName(QString::fromUtf8("btnStop"));
        sizePolicy7.setHeightForWidth(btnStop->sizePolicy().hasHeightForWidth());
        btnStop->setSizePolicy(sizePolicy7);
        btnStop->setMinimumSize(QSize(32, 24));
        btnStop->setMaximumSize(QSize(48, 28));
        btnStop->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnStop->setCheckable(false);

        lay_transport->addWidget(btnStop);

        btnPause = new QPushButton(centralWidget);
        btnPause->setObjectName(QString::fromUtf8("btnPause"));
        sizePolicy7.setHeightForWidth(btnPause->sizePolicy().hasHeightForWidth());
        btnPause->setSizePolicy(sizePolicy7);
        btnPause->setMinimumSize(QSize(32, 24));
        btnPause->setMaximumSize(QSize(48, 28));

        lay_transport->addWidget(btnPause);

        btnPlay = new QPushButton(centralWidget);
        btnPlay->setObjectName(QString::fromUtf8("btnPlay"));
        sizePolicy7.setHeightForWidth(btnPlay->sizePolicy().hasHeightForWidth());
        btnPlay->setSizePolicy(sizePolicy7);
        btnPlay->setMinimumSize(QSize(32, 24));
        btnPlay->setMaximumSize(QSize(48, 28));
        btnPlay->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnPlay->setCheckable(true);
        btnPlay->setAutoRepeat(false);

        lay_transport->addWidget(btnPlay);

        btnRecordEx = new QPushButton(centralWidget);
        btnRecordEx->setObjectName(QString::fromUtf8("btnRecordEx"));
        btnRecordEx->setEnabled(false);
        btnRecordEx->setMinimumSize(QSize(28, 24));
        btnRecordEx->setMaximumSize(QSize(28, 28));
        btnRecordEx->setCheckable(false);

        lay_transport->addWidget(btnRecordEx);

        btnLoop = new QPushButton(centralWidget);
        btnLoop->setObjectName(QString::fromUtf8("btnLoop"));
        btnLoop->setMinimumSize(QSize(32, 24));
        btnLoop->setMaximumSize(QSize(48, 28));
        btnLoop->setIconSize(QSize(33, 12));
        btnLoop->setCheckable(true);

        lay_transport->addWidget(btnLoop);

        btnRecord = new QPushButton(centralWidget);
        btnRecord->setObjectName(QString::fromUtf8("btnRecord"));
        btnRecord->setEnabled(false);
        sizePolicy7.setHeightForWidth(btnRecord->sizePolicy().hasHeightForWidth());
        btnRecord->setSizePolicy(sizePolicy7);
        btnRecord->setMinimumSize(QSize(28, 24));
        btnRecord->setMaximumSize(QSize(28, 28));
        btnRecord->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnRecord->setCheckable(true);

        lay_transport->addWidget(btnRecord);

        button_keep_queue = new QPushButton(centralWidget);
        button_keep_queue->setObjectName(QString::fromUtf8("button_keep_queue"));
        sizePolicy7.setHeightForWidth(button_keep_queue->sizePolicy().hasHeightForWidth());
        button_keep_queue->setSizePolicy(sizePolicy7);
        button_keep_queue->setMinimumSize(QSize(32, 24));
        button_keep_queue->setMaximumSize(QSize(48, 28));
        button_keep_queue->setFont(font1);

        lay_transport->addWidget(button_keep_queue);

        button_learn = new QPushButton(centralWidget);
        button_learn->setObjectName(QString::fromUtf8("button_learn"));
        sizePolicy7.setHeightForWidth(button_learn->sizePolicy().hasHeightForWidth());
        button_learn->setSizePolicy(sizePolicy7);
        button_learn->setMinimumSize(QSize(32, 24));
        button_learn->setMaximumSize(QSize(48, 28));

        lay_transport->addWidget(button_learn);


        hLayoutBottom_2->addLayout(lay_transport);

        testButton = new QPushButton(centralWidget);
        testButton->setObjectName(QString::fromUtf8("testButton"));
        testButton->setMinimumSize(QSize(16, 24));
        testButton->setMaximumSize(QSize(16, 28));

        hLayoutBottom_2->addWidget(testButton);

        btnMute = new QPushButton(centralWidget);
        btnMute->setObjectName(QString::fromUtf8("btnMute"));
        btnMute->setMinimumSize(QSize(48, 24));
        btnMute->setMaximumSize(QSize(56, 28));
        btnMute->setFont(font1);

        hLayoutBottom_2->addWidget(btnMute);

        btnPerfEdit = new QPushButton(centralWidget);
        btnPerfEdit->setObjectName(QString::fromUtf8("btnPerfEdit"));
        QSizePolicy sizePolicy8(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(btnPerfEdit->sizePolicy().hasHeightForWidth());
        btnPerfEdit->setSizePolicy(sizePolicy8);
        btnPerfEdit->setMinimumSize(QSize(40, 24));
        btnPerfEdit->setMaximumSize(QSize(48, 28));
        btnPerfEdit->setIconSize(QSize(28, 20));

        hLayoutBottom_2->addWidget(btnPerfEdit);

        btnSongPlay = new QPushButton(centralWidget);
        btnSongPlay->setObjectName(QString::fromUtf8("btnSongPlay"));
        sizePolicy8.setHeightForWidth(btnSongPlay->sizePolicy().hasHeightForWidth());
        btnSongPlay->setSizePolicy(sizePolicy8);
        btnSongPlay->setMinimumSize(QSize(40, 24));
        btnSongPlay->setMaximumSize(QSize(64, 28));
        btnSongPlay->setFont(font1);
        btnSongPlay->setFocusPolicy(Qt::FocusPolicy::NoFocus);
        btnSongPlay->setIconSize(QSize(24, 16));
        btnSongPlay->setCheckable(true);

        hLayoutBottom_2->addWidget(btnSongPlay);

        lineEditPpqn = new QLineEdit(centralWidget);
        lineEditPpqn->setObjectName(QString::fromUtf8("lineEditPpqn"));
        lineEditPpqn->setEnabled(false);
        lineEditPpqn->setMinimumSize(QSize(56, 24));
        lineEditPpqn->setMaximumSize(QSize(64, 28));
        lineEditPpqn->setFont(font1);
        lineEditPpqn->setMaxLength(11);
        lineEditPpqn->setFrame(true);
        lineEditPpqn->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);
        lineEditPpqn->setReadOnly(true);

        hLayoutBottom_2->addWidget(lineEditPpqn);

        lay_bpm = new QHBoxLayout();
        lay_bpm->setSpacing(6);
        lay_bpm->setObjectName(QString::fromUtf8("lay_bpm"));
        button_tap_bpm = new QPushButton(centralWidget);
        button_tap_bpm->setObjectName(QString::fromUtf8("button_tap_bpm"));
        button_tap_bpm->setMinimumSize(QSize(36, 24));
        button_tap_bpm->setMaximumSize(QSize(36, 28));
        button_tap_bpm->setFont(font1);

        lay_bpm->addWidget(button_tap_bpm);

        labelBPM = new QLabel(centralWidget);
        labelBPM->setObjectName(QString::fromUtf8("labelBPM"));
        labelBPM->setMinimumSize(QSize(34, 24));
        labelBPM->setMaximumSize(QSize(36, 28));
        labelBPM->setFont(font3);
        labelBPM->setAlignment(Qt::AlignmentFlag::AlignRight|Qt::AlignmentFlag::AlignTrailing|Qt::AlignmentFlag::AlignVCenter);

        lay_bpm->addWidget(labelBPM);

        spinBpm = new QDoubleSpinBox(centralWidget);
        spinBpm->setObjectName(QString::fromUtf8("spinBpm"));
        spinBpm->setEnabled(true);
        spinBpm->setMinimumSize(QSize(80, 24));
        spinBpm->setMaximumSize(QSize(80, 28));
        spinBpm->setFont(font4);
        spinBpm->setFocusPolicy(Qt::FocusPolicy::ClickFocus);
        spinBpm->setLayoutDirection(Qt::LayoutDirection::LeftToRight);
        spinBpm->setMinimum(2.000000000000000);
        spinBpm->setMaximum(600.000000000000000);
        spinBpm->setValue(120.000000000000000);

        lay_bpm->addWidget(spinBpm);


        hLayoutBottom_2->addLayout(lay_bpm);


        verticalLayout->addLayout(hLayoutBottom_2);

        qsmainwnd->setCentralWidget(centralWidget);
        menuBar = new QMenuBar(qsmainwnd);
        menuBar->setObjectName(QString::fromUtf8("menuBar"));
        menuBar->setGeometry(QRect(0, 0, 900, 28));
        menuBar->setFont(font3);
        menuFile = new QMenu(menuBar);
        menuFile->setObjectName(QString::fromUtf8("menuFile"));
        menuExport = new QMenu(menuFile);
        menuExport->setObjectName(QString::fromUtf8("menuExport"));
        menuImport = new QMenu(menuFile);
        menuImport->setObjectName(QString::fromUtf8("menuImport"));
        menuAbout = new QMenu(menuBar);
        menuAbout->setObjectName(QString::fromUtf8("menuAbout"));
        menuEdit = new QMenu(menuBar);
        menuEdit->setObjectName(QString::fromUtf8("menuEdit"));
        qsmainwnd->setMenuBar(menuBar);

        menuBar->addAction(menuFile->menuAction());
        menuBar->addAction(menuEdit->menuAction());
        menuBar->addAction(menuAbout->menuAction());
        menuFile->addAction(actionNew);
        menuFile->addAction(actionOpen);
        menuFile->addAction(actionOpenPlaylist);
        menuFile->addSeparator();
        menuFile->addAction(actionSave);
        menuFile->addAction(actionSave_As);
        menuFile->addSeparator();
        menuFile->addSeparator();
        menuFile->addAction(menuImport->menuAction());
        menuFile->addAction(menuExport->menuAction());
        menuFile->addAction(actionQuit);
        menuExport->addAction(actionExportProject);
        menuExport->addAction(actionExportMIDI);
        menuExport->addAction(actionExportSong);
        menuExport->addAction(actionExportSMF0);
        menuImport->addAction(actionImportProject);
        menuImport->addAction(actionImportMIDI);
        menuImport->addAction(actionImportMIDIIntoSession);
        menuImport->addAction(actionImportPlaylist);
        menuAbout->addAction(actionAbout);
        menuAbout->addAction(actionBuildInfo);
        menuAbout->addAction(actionSongSummary);
        menuAbout->addSeparator();
        menuAbout->addSeparator();
        menuAbout->addAction(actionAppInfo);
        menuAbout->addAction(actionTutorial);
        menuAbout->addAction(actionUserManual);
        menuAbout->addSeparator();
        menuAbout->addAction(actionLogView);
        menuEdit->addAction(actionPreferences);
        menuEdit->addSeparator();
        menuEdit->addAction(actionSongEditor);
        menuEdit->addAction(actionSongTranspose);
        menuEdit->addAction(actionClearMuteGroups);
        menuEdit->addAction(actionReloadMuteGroups);
        menuEdit->addSeparator();
        menuEdit->addAction(actionMuteAllTracks);
        menuEdit->addAction(actionUnmuteAllTracks);
        menuEdit->addAction(actionToggleAllTracks);
        menuEdit->addAction(actionCopyCurrentSet);
        menuEdit->addAction(actionPasteToCurrentSet);

        retranslateUi(qsmainwnd);

        cmb_beat_measure->setCurrentIndex(-1);
        tabWidget->setCurrentIndex(7);


        QMetaObject::connectSlotsByName(qsmainwnd);
    } // setupUi

    void retranslateUi(QMainWindow *qsmainwnd)
    {
        qsmainwnd->setWindowTitle(QCoreApplication::translate("qsmainwnd", "qseq66", nullptr));
#if QT_CONFIG(tooltip)
        qsmainwnd->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        actionNew->setText(QCoreApplication::translate("qsmainwnd", "&New", nullptr));
#if QT_CONFIG(tooltip)
        actionNew->setToolTip(QCoreApplication::translate("qsmainwnd", "Create a new Seq66 MIDI file, clearing all song information.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionNew->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        actionImportMIDI->setText(QCoreApplication::translate("qsmainwnd", "MIDI to &Current Set...", nullptr));
#if QT_CONFIG(tooltip)
        actionImportMIDI->setToolTip(QCoreApplication::translate("qsmainwnd", "Import MIDI file into (different) bank/set.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionImportMIDI->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+I", nullptr));
#endif // QT_CONFIG(shortcut)
        actionImportProject->setText(QCoreApplication::translate("qsmainwnd", "Pro&ject Configuraton...", nullptr));
#if QT_CONFIG(tooltip)
        actionImportProject->setToolTip(QCoreApplication::translate("qsmainwnd", "Import a configuration from another directory.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionImportProject->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+J", nullptr));
#endif // QT_CONFIG(shortcut)
        actionOpen->setText(QCoreApplication::translate("qsmainwnd", "&Open...", nullptr));
#if QT_CONFIG(tooltip)
        actionOpen->setToolTip(QCoreApplication::translate("qsmainwnd", "Open a MIDI or Seq66 MIDI file.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionOpen->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        actionOpenPlaylist->setText(QCoreApplication::translate("qsmainwnd", "Open &Playlist...", nullptr));
#if QT_CONFIG(tooltip)
        actionOpenPlaylist->setToolTip(QCoreApplication::translate("qsmainwnd", "Open a Seq66 playlist file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionRecent->setText(QCoreApplication::translate("qsmainwnd", "&Recent...", nullptr));
#if QT_CONFIG(tooltip)
        actionRecent->setToolTip(QCoreApplication::translate("qsmainwnd", "Open a Recent File.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionPreferences->setText(QCoreApplication::translate("qsmainwnd", "&Preferences...", nullptr));
#if QT_CONFIG(tooltip)
        actionPreferences->setToolTip(QCoreApplication::translate("qsmainwnd", "Preferences/Options (still in progress).", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionPreferences->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+P", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSave->setText(QCoreApplication::translate("qsmainwnd", "&Save", nullptr));
#if QT_CONFIG(tooltip)
        actionSave->setToolTip(QCoreApplication::translate("qsmainwnd", "Save MIDI file in Seq66 format.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSave->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSave_As->setText(QCoreApplication::translate("qsmainwnd", "Save &As...", nullptr));
#if QT_CONFIG(tooltip)
        actionSave_As->setToolTip(QCoreApplication::translate("qsmainwnd", "Save As a Seq66 MIDI file with a different name.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSave_As->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+Shift+S", nullptr));
#endif // QT_CONFIG(shortcut)
        actionQuit->setText(QCoreApplication::translate("qsmainwnd", "&Quit", nullptr));
        actionAbout->setText(QCoreApplication::translate("qsmainwnd", "&About...", nullptr));
        actionBuildInfo->setText(QCoreApplication::translate("qsmainwnd", "&Build Info...", nullptr));
        actionAppInfo->setText(QCoreApplication::translate("qsmainwnd", "App &Keys...", nullptr));
        actionAbout_Qt->setText(QCoreApplication::translate("qsmainwnd", "About Qt...", nullptr));
        actionTools->setText(QCoreApplication::translate("qsmainwnd", "Tools...", nullptr));
        actionExportSong->setText(QCoreApplication::translate("qsmainwnd", "&Song...", nullptr));
#if QT_CONFIG(tooltip)
        actionExportSong->setToolTip(QCoreApplication::translate("qsmainwnd", "Export song layout as a Seq66 MIDI file with\n"
"triggers consolidated into a single one for each unmuted pattern.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportMIDI->setText(QCoreApplication::translate("qsmainwnd", "&MIDI Only...", nullptr));
#if QT_CONFIG(tooltip)
        actionExportMIDI->setToolTip(QCoreApplication::translate("qsmainwnd", "Export without Seq66 SeqSpecs, just MIDI events\n"
"and messages.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionExportSMF0->setText(QCoreApplication::translate("qsmainwnd", "SMF &0...", nullptr));
#if QT_CONFIG(tooltip)
        actionExportSMF0->setToolTip(QCoreApplication::translate("qsmainwnd", "Convert to SMF 0 and export without Seq66 SeqSpecs.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSongEditor->setText(QCoreApplication::translate("qsmainwnd", "&Song Editor", nullptr));
#if QT_CONFIG(tooltip)
        actionSongEditor->setToolTip(QCoreApplication::translate("qsmainwnd", "Brings up the Song Editor in an external window.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionSongEditor->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+E", nullptr));
#endif // QT_CONFIG(shortcut)
        actionSongTranspose->setText(QCoreApplication::translate("qsmainwnd", "Apply Song &Transpose", nullptr));
#if QT_CONFIG(tooltip)
        actionSongTranspose->setToolTip(QCoreApplication::translate("qsmainwnd", "Transposes the transposable tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionClearMuteGroups->setText(QCoreApplication::translate("qsmainwnd", "&Clear Mute Groups", nullptr));
#if QT_CONFIG(tooltip)
        actionClearMuteGroups->setToolTip(QCoreApplication::translate("qsmainwnd", "Clears mute groups so one can start over again.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionReloadMuteGroups->setText(QCoreApplication::translate("qsmainwnd", "&Reload Mute Groups", nullptr));
#if QT_CONFIG(tooltip)
        actionReloadMuteGroups->setToolTip(QCoreApplication::translate("qsmainwnd", "Reloads mute groups from the \"mutes\" file.", nullptr));
#endif // QT_CONFIG(tooltip)
        actionMuteAllTracks->setText(QCoreApplication::translate("qsmainwnd", "&Mute All Tracks", nullptr));
#if QT_CONFIG(tooltip)
        actionMuteAllTracks->setToolTip(QCoreApplication::translate("qsmainwnd", "Mutes all tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionMuteAllTracks->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+M", nullptr));
#endif // QT_CONFIG(shortcut)
        actionUnmuteAllTracks->setText(QCoreApplication::translate("qsmainwnd", "&Unmute All Tracks", nullptr));
#if QT_CONFIG(tooltip)
        actionUnmuteAllTracks->setToolTip(QCoreApplication::translate("qsmainwnd", "Arms all tracks in the current set.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionUnmuteAllTracks->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+U", nullptr));
#endif // QT_CONFIG(shortcut)
        actionToggleAllTracks->setText(QCoreApplication::translate("qsmainwnd", "To&ggle All Tracks", nullptr));
#if QT_CONFIG(tooltip)
        actionToggleAllTracks->setToolTip(QCoreApplication::translate("qsmainwnd", "Toggles mute status of all tracks in the current set.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionToggleAllTracks->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        actionClose->setText(QCoreApplication::translate("qsmainwnd", "&Close", nullptr));
#if QT_CONFIG(tooltip)
        actionClose->setToolTip(QCoreApplication::translate("qsmainwnd", "Close Session", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSongSummary->setText(QCoreApplication::translate("qsmainwnd", "&Song Summary File...", nullptr));
#if QT_CONFIG(tooltip)
        actionSongSummary->setToolTip(QCoreApplication::translate("qsmainwnd", "Writes a text summary of the song to a file\n"
"in the same directory as the original, with the extension\n"
"\".text\".", nullptr));
#endif // QT_CONFIG(tooltip)
        actionCopyCurrentSet->setText(QCoreApplication::translate("qsmainwnd", "C&opy Current Set", nullptr));
        actionPasteToCurrentSet->setText(QCoreApplication::translate("qsmainwnd", "Past&e To Current Set", nullptr));
        actionPasteToCurrentSet->setIconText(QCoreApplication::translate("qsmainwnd", "P&aste To Current Set", nullptr));
        actionImportPlaylist->setText(QCoreApplication::translate("qsmainwnd", "&Playlist...", nullptr));
#if QT_CONFIG(tooltip)
        actionImportPlaylist->setToolTip(QCoreApplication::translate("qsmainwnd", "Import a playlist into the session.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(shortcut)
        actionImportPlaylist->setShortcut(QCoreApplication::translate("qsmainwnd", "Ctrl+L", nullptr));
#endif // QT_CONFIG(shortcut)
        actionImportMIDIIntoSession->setText(QCoreApplication::translate("qsmainwnd", "&MIDI into Session", nullptr));
        actionTutorial->setText(QCoreApplication::translate("qsmainwnd", "&Tutorial", nullptr));
        actionUserManual->setText(QCoreApplication::translate("qsmainwnd", "&User Manual", nullptr));
        actionExportProject->setText(QCoreApplication::translate("qsmainwnd", "Pro&ject Configuration...", nullptr));
        actionLogView->setText(QCoreApplication::translate("qsmainwnd", "&View Log", nullptr));
#if QT_CONFIG(tooltip)
        btnSeq66->setToolTip(QCoreApplication::translate("qsmainwnd", "Clicking on the logo will remove all external pattern\n"
"editors and live frames (but not the external song\n"
"editor).", nullptr));
#endif // QT_CONFIG(tooltip)
        btnSeq66->setText(QCoreApplication::translate("qsmainwnd", "Seq66", nullptr));
        btnShowHide->setText(QCoreApplication::translate("qsmainwnd", "_", nullptr));
#if QT_CONFIG(tooltip)
        smf0Button->setToolTip(QCoreApplication::translate("qsmainwnd", "Indicates the format (0 or 1) of\n"
"the MIDI tune.", nullptr));
#endif // QT_CONFIG(tooltip)
        smf0Button->setText(QCoreApplication::translate("qsmainwnd", "SMF 0", nullptr));
#if QT_CONFIG(tooltip)
        alsaJackButton->setToolTip(QCoreApplication::translate("qsmainwnd", "Indicates if in JACK/ALSA/PortMidi mode.", nullptr));
#endif // QT_CONFIG(tooltip)
        alsaJackButton->setText(QCoreApplication::translate("qsmainwnd", "PortMidi", nullptr));
#if QT_CONFIG(tooltip)
        jackTransportButton->setToolTip(QCoreApplication::translate("qsmainwnd", "Indicates if in JACK Transport Master\n"
"or Slave mode.", nullptr));
#endif // QT_CONFIG(tooltip)
        jackTransportButton->setText(QCoreApplication::translate("qsmainwnd", "Master", nullptr));
#if QT_CONFIG(tooltip)
        cmb_ppqn->setToolTip(QCoreApplication::translate("qsmainwnd", "Change the PPQN value of song for\n"
"ALL sets. This modifies the song; a\n"
"\"Save\" prompt will occur at exit.\n"
"It does not change Seq66's default\n"
"PPQN.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cmb_global_bus->setToolTip(QCoreApplication::translate("qsmainwnd", "Sets output bus for all patterns in\n"
"the current set.  Modifies the song.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        cmb_beat_measure->setToolTip(QCoreApplication::translate("qsmainwnd", "Global Beats Per Measure", nullptr));
#endif // QT_CONFIG(tooltip)
        label_ts_slash->setText(QCoreApplication::translate("qsmainwnd", "/", nullptr));
#if QT_CONFIG(tooltip)
        cmb_beat_length->setToolTip(QCoreApplication::translate("qsmainwnd", "Global Beat Width (length of beat)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnBBTHMS->setToolTip(QCoreApplication::translate("qsmainwnd", "This button displays the current time during\n"
"playback. Clicking it toggles between\n"
"B:B:T and H:M:S format.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnBBTHMS->setText(QCoreApplication::translate("qsmainwnd", "008:16:0000", nullptr));
#if QT_CONFIG(tooltip)
        label_test->setToolTip(QCoreApplication::translate("qsmainwnd", "Test", nullptr));
#endif // QT_CONFIG(tooltip)
        label_test->setText(QCoreApplication::translate("qsmainwnd", "TEST", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(LiveTab), QCoreApplication::translate("qsmainwnd", "&Live", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(SongTab), QCoreApplication::translate("qsmainwnd", "&Song", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(EditTab), QCoreApplication::translate("qsmainwnd", "E&ditor", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(EventTab), QCoreApplication::translate("qsmainwnd", "E&vents", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(PlaylistTab), QCoreApplication::translate("qsmainwnd", "&Playlists", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(SetMasterTab), QCoreApplication::translate("qsmainwnd", "Se&ts", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(MuteMasterTab), QCoreApplication::translate("qsmainwnd", "&Mutes", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(SessionTab), QCoreApplication::translate("qsmainwnd", "Sessio&n", nullptr));
        setNameLabel->setText(QCoreApplication::translate("qsmainwnd", "Set Name", nullptr));
#if QT_CONFIG(tooltip)
        txtBankName->setToolTip(QCoreApplication::translate("qsmainwnd", "Editable, this shows the name of\n"
"the current set.", nullptr));
#endif // QT_CONFIG(tooltip)
        txtBankName->setText(QCoreApplication::translate("qsmainwnd", "New", nullptr));
#if QT_CONFIG(tooltip)
        label->setToolTip(QCoreApplication::translate("qsmainwnd", "Mute-group name", nullptr));
#endif // QT_CONFIG(tooltip)
        label->setText(QCoreApplication::translate("qsmainwnd", "MG", nullptr));
        txtMuteName->setText(QCoreApplication::translate("qsmainwnd", "0: Group #1", nullptr));
#if QT_CONFIG(tooltip)
        txtUnderrun->setToolTip(QCoreApplication::translate("qsmainwnd", "Indicates the loading of the system\n"
"during playback.", nullptr));
#endif // QT_CONFIG(tooltip)
        txtUnderrun->setText(QCoreApplication::translate("qsmainwnd", "-", nullptr));
        labelActiveSet->setText(QCoreApplication::translate("qsmainwnd", "Active", nullptr));
#if QT_CONFIG(tooltip)
        entry_active_set->setToolTip(QCoreApplication::translate("qsmainwnd", "The number of the active screen-set\n"
"or active external live frame.", nullptr));
#endif // QT_CONFIG(tooltip)
        entry_active_set->setText(QCoreApplication::translate("qsmainwnd", "1024", nullptr));
#if QT_CONFIG(tooltip)
        setHomeButton->setToolTip(QCoreApplication::translate("qsmainwnd", "Immediately make set 0 active.", nullptr));
#endif // QT_CONFIG(tooltip)
        setHomeButton->setText(QCoreApplication::translate("qsmainwnd", "Set 0", nullptr));
#if QT_CONFIG(tooltip)
        setResetButton->setToolTip(QCoreApplication::translate("qsmainwnd", "Resets the play-set to the current set,\n"
"no matter what sets-mode is in force. ", nullptr));
#endif // QT_CONFIG(tooltip)
        setResetButton->setText(QCoreApplication::translate("qsmainwnd", "!", nullptr));
        setNumberLabel->setText(QCoreApplication::translate("qsmainwnd", "Set", nullptr));
#if QT_CONFIG(tooltip)
        spinBank->setToolTip(QCoreApplication::translate("qsmainwnd", "Selects the active set.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        btnPanic->setToolTip(QCoreApplication::translate("qsmainwnd", "Panic! Turns off all notes.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPanic->setText(QCoreApplication::translate("qsmainwnd", "Panic", nullptr));
#if QT_CONFIG(tooltip)
        btnStop->setToolTip(QCoreApplication::translate("qsmainwnd", "Stop playback. Shift-click rewinds transport.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnStop->setText(QCoreApplication::translate("qsmainwnd", "Stop", nullptr));
#if QT_CONFIG(tooltip)
        btnPause->setToolTip(QCoreApplication::translate("qsmainwnd", "Pause/continue. Preserves current location.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPause->setText(QCoreApplication::translate("qsmainwnd", "Pause", nullptr));
#if QT_CONFIG(tooltip)
        btnPlay->setToolTip(QCoreApplication::translate("qsmainwnd", "Start/continue playback.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPlay->setText(QCoreApplication::translate("qsmainwnd", "Play", nullptr));
#if QT_CONFIG(tooltip)
        btnRecordEx->setToolTip(QCoreApplication::translate("qsmainwnd", "Activates either the \"current\" pattern, all\n"
"input-buss patterns, or all channel patterns.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnRecordEx->setText(QCoreApplication::translate("qsmainwnd", "Rec", nullptr));
#if QT_CONFIG(tooltip)
        btnLoop->setToolTip(QCoreApplication::translate("qsmainwnd", "Activate L/R looping.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnLoop->setText(QCoreApplication::translate("qsmainwnd", "Loop", nullptr));
#if QT_CONFIG(tooltip)
        btnRecord->setToolTip(QCoreApplication::translate("qsmainwnd", "Record live sequence changes into the\n"
"Song Editor. Ctrl key disables snap for\n"
"the recording. Shift key enables recording\n"
"at start of playback.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnRecord->setText(QCoreApplication::translate("qsmainwnd", "Rec", nullptr));
#if QT_CONFIG(tooltip)
        button_keep_queue->setToolTip(QCoreApplication::translate("qsmainwnd", "Shows/toggles keep-queue status.\n"
"Does not show one-shot queue\n"
"status.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_keep_queue->setText(QCoreApplication::translate("qsmainwnd", "Q", nullptr));
#if QT_CONFIG(tooltip)
        button_learn->setToolTip(QCoreApplication::translate("qsmainwnd", "Mute Group Learn. Click, then press\n"
"a key to store the state of the sequences\n"
"in the Shift of that key.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_learn->setText(QCoreApplication::translate("qsmainwnd", "Learn", nullptr));
        testButton->setText(QString());
#if QT_CONFIG(tooltip)
        btnMute->setToolTip(QCoreApplication::translate("qsmainwnd", "Click to toggle the mute status of\n"
"all tracks.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnMute->setText(QCoreApplication::translate("qsmainwnd", "Mute", nullptr));
#if QT_CONFIG(tooltip)
        btnPerfEdit->setToolTip(QCoreApplication::translate("qsmainwnd", "Opens the external song-editor.\n"
"[ Ctrl-E ]", nullptr));
#endif // QT_CONFIG(tooltip)
        btnPerfEdit->setText(QCoreApplication::translate("qsmainwnd", "Perf", nullptr));
#if QT_CONFIG(tooltip)
        btnSongPlay->setToolTip(QCoreApplication::translate("qsmainwnd", "Toggle playback using the Song\n"
"Editor triggers.", nullptr));
#endif // QT_CONFIG(tooltip)
        btnSongPlay->setText(QCoreApplication::translate("qsmainwnd", "Live", nullptr));
#if QT_CONFIG(tooltip)
        lineEditPpqn->setToolTip(QCoreApplication::translate("qsmainwnd", "PPQN", nullptr));
#endif // QT_CONFIG(tooltip)
        lineEditPpqn->setText(QCoreApplication::translate("qsmainwnd", "192", nullptr));
#if QT_CONFIG(tooltip)
        button_tap_bpm->setToolTip(QCoreApplication::translate("qsmainwnd", "Tap in time to set beats/minute (BPM).\n"
"After 5 seconds of no taps, the counter\n"
"resets to 0.", nullptr));
#endif // QT_CONFIG(tooltip)
        button_tap_bpm->setText(QCoreApplication::translate("qsmainwnd", "0", nullptr));
        labelBPM->setText(QCoreApplication::translate("qsmainwnd", "BPM", nullptr));
#if QT_CONFIG(tooltip)
        spinBpm->setToolTip(QCoreApplication::translate("qsmainwnd", "Allows the setting of the\n"
"beats/minute.", nullptr));
#endif // QT_CONFIG(tooltip)
        menuFile->setTitle(QCoreApplication::translate("qsmainwnd", "&File", nullptr));
        menuExport->setTitle(QCoreApplication::translate("qsmainwnd", "&Export", nullptr));
        menuImport->setTitle(QCoreApplication::translate("qsmainwnd", "&Import", nullptr));
        menuAbout->setTitle(QCoreApplication::translate("qsmainwnd", "&Help", nullptr));
        menuEdit->setTitle(QCoreApplication::translate("qsmainwnd", "&Edit", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsmainwnd: public Ui_qsmainwnd {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSMAINWND_H
