/********************************************************************************
** Form generated from reading UI file 'qseqeditex.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef QSEQEDITEX_H
#define QSEQEDITEX_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qseqeditex
{
public:

    void setupUi(QWidget *qseqeditex)
    {
        if (qseqeditex->objectName().isEmpty())
            qseqeditex->setObjectName(QString::fromUtf8("qseqeditex"));
        qseqeditex->resize(889, 572);

        retranslateUi(qseqeditex);

        QMetaObject::connectSlotsByName(qseqeditex);
    } // setupUi

    void retranslateUi(QWidget *qseqeditex)
    {
        qseqeditex->setWindowTitle(QCoreApplication::translate("qseqeditex", "Pattern", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qseqeditex: public Ui_qseqeditex {};
} // namespace Ui

QT_END_NAMESPACE

#endif // QSEQEDITEX_H
