/****************************************************************************
** Meta object code from reading C++ file 'qpatternfix.hpp'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "qpatternfix.hpp"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qpatternfix.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_seq66__qpatternfix_t {
    QByteArrayData data[23];
    char stringdata0[381];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_seq66__qpatternfix_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_seq66__qpatternfix_t qt_meta_stringdata_seq66__qpatternfix = {
    {
QT_MOC_LITERAL(0, 0, 18), // "seq66::qpatternfix"
QT_MOC_LITERAL(1, 19, 17), // "slot_effect_clear"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 15), // "slot_length_fix"
QT_MOC_LITERAL(4, 54, 11), // "fixlengthid"
QT_MOC_LITERAL(5, 66, 19), // "slot_measure_change"
QT_MOC_LITERAL(6, 86, 17), // "slot_scale_change"
QT_MOC_LITERAL(7, 104, 15), // "slot_alt_change"
QT_MOC_LITERAL(8, 120, 5), // "altid"
QT_MOC_LITERAL(9, 126, 19), // "slot_tighten_change"
QT_MOC_LITERAL(10, 146, 16), // "slot_full_change"
QT_MOC_LITERAL(11, 163, 18), // "slot_jitter_change"
QT_MOC_LITERAL(12, 182, 18), // "slot_random_change"
QT_MOC_LITERAL(13, 201, 24), // "slot_random_pitch_change"
QT_MOC_LITERAL(14, 226, 17), // "slot_notemap_file"
QT_MOC_LITERAL(15, 244, 22), // "slot_align_left_change"
QT_MOC_LITERAL(16, 267, 5), // "dummy"
QT_MOC_LITERAL(17, 273, 23), // "slot_align_right_change"
QT_MOC_LITERAL(18, 297, 19), // "slot_reverse_change"
QT_MOC_LITERAL(19, 317, 21), // "slot_reverse_in_place"
QT_MOC_LITERAL(20, 339, 21), // "slot_save_note_length"
QT_MOC_LITERAL(21, 361, 8), // "slot_set"
QT_MOC_LITERAL(22, 370, 10) // "slot_reset"

    },
    "seq66::qpatternfix\0slot_effect_clear\0"
    "\0slot_length_fix\0fixlengthid\0"
    "slot_measure_change\0slot_scale_change\0"
    "slot_alt_change\0altid\0slot_tighten_change\0"
    "slot_full_change\0slot_jitter_change\0"
    "slot_random_change\0slot_random_pitch_change\0"
    "slot_notemap_file\0slot_align_left_change\0"
    "dummy\0slot_align_right_change\0"
    "slot_reverse_change\0slot_reverse_in_place\0"
    "slot_save_note_length\0slot_set\0"
    "slot_reset"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_seq66__qpatternfix[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  104,    2, 0x08 /* Private */,
       3,    1,  105,    2, 0x08 /* Private */,
       5,    0,  108,    2, 0x08 /* Private */,
       6,    0,  109,    2, 0x08 /* Private */,
       7,    1,  110,    2, 0x08 /* Private */,
       9,    0,  113,    2, 0x08 /* Private */,
      10,    0,  114,    2, 0x08 /* Private */,
      11,    0,  115,    2, 0x08 /* Private */,
      12,    0,  116,    2, 0x08 /* Private */,
      13,    0,  117,    2, 0x08 /* Private */,
      14,    0,  118,    2, 0x08 /* Private */,
      15,    1,  119,    2, 0x08 /* Private */,
      17,    1,  122,    2, 0x08 /* Private */,
      18,    1,  125,    2, 0x08 /* Private */,
      19,    1,  128,    2, 0x08 /* Private */,
      20,    1,  131,    2, 0x08 /* Private */,
      21,    0,  134,    2, 0x08 /* Private */,
      22,    0,  135,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void seq66::qpatternfix::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<qpatternfix *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slot_effect_clear(); break;
        case 1: _t->slot_length_fix((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slot_measure_change(); break;
        case 3: _t->slot_scale_change(); break;
        case 4: _t->slot_alt_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->slot_tighten_change(); break;
        case 6: _t->slot_full_change(); break;
        case 7: _t->slot_jitter_change(); break;
        case 8: _t->slot_random_change(); break;
        case 9: _t->slot_random_pitch_change(); break;
        case 10: _t->slot_notemap_file(); break;
        case 11: _t->slot_align_left_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->slot_align_right_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->slot_reverse_change((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->slot_reverse_in_place((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->slot_save_note_length((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slot_set(); break;
        case 17: _t->slot_reset(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject seq66::qpatternfix::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_seq66__qpatternfix.data,
    qt_meta_data_seq66__qpatternfix,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *seq66::qpatternfix::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *seq66::qpatternfix::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_seq66__qpatternfix.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int seq66::qpatternfix::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
